/* Revision 1.2  1997/06/25 11:20:03  slywczak
 * Free the tempBuffer as the result of purify.
 *
 * Revision 1.1  1997/05/29 18:20:42  slywczak
 * Initial development and entry into CVS
 * */
#include "TKasciiDB.h"
/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       FILE *open_db ( char *db_name )
*
*  DESCRIPTION:
*       This routine will open an ASCII text database file.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       db_name                 Name of the database file to open.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   query_ascii_db
*
*  RETURN VALUES:               File pointer of the opened file.
*
*  ROUTINES CALLED:             
*      TKreportWarning
*
******************************************************************************/
FILE *open_db ( char *db_name )
{
  static char *db_path = NULL;
  char db_file[TK_DB_STRING_LENGTH];

  if ( db_path == NULL )
    {
      db_path = getenv ( "TSDISTK" );
      if ( db_path == NULL )
	{
	  TKreportWarning ( E_TK_NULLPATH );
	  return;
	}
    }
  strcpy ( db_file, db_path );
  strcat ( db_file, "/db/" );
  strcat ( db_file, db_name );
  
  return fopen ( db_file, "r" );
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       char *extract_word ( int *i, char *buffer )
*
*  DESCRIPTION:
*       This routine will extract the next value (string) from the data
*       buffer.  It uses tabs as column separators.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       i                       The position in the character buffer.
*
*       buffer                  The data buffer to search through.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   query_db
*
*  RETURN VALUES:               The value (string) extracted from the buffer.
*
*  ROUTINES CALLED:
*          TKreportError
*
******************************************************************************/
char *extract_word ( int *i, char *buffer )
{
  int j = 0;
  char *word_buffer;

  word_buffer = (char *)calloc ( TK_DB_STRING_LENGTH, sizeof ( char ) );
  if ( word_buffer == NULL )
    {
      TKreportWarning ( E_TK_BADMEMALLCAT );
      return NULL;
    }
  while ( buffer[*i] == '\t' )
    (*i)++;

  while ( ( ( buffer[*i] != '\t' ) && ( *i != strlen ( buffer ) ) 
	   && ( buffer[*i] != '\n' ) ) )
   {
     word_buffer[j++] = buffer[*i];
     (*i)++;
   }
  word_buffer[j] = '\0';
  return word_buffer;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       int addString ( char **buffer, char *tempBuffer )
*
*  DESCRIPTION:
*       This routine will create a comma separated string.  It appends 
*       a comma to buffer and then appends tempBuffer.  If the string
*       exceeds the default length, the buffer will be resized.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*
*       buffer                  The current buffer.
*
*       tempBuffer              The string to append to the buffer.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       buffer                  The databuffer to append the next string.
*
*  CALLED BY:                   query_db
*
*  RETURN VALUES:               TK_SUCCESS, TK_FAIL
*
*  ROUTINES CALLED:
*          TKreportError
*
******************************************************************************/
int addString ( char **buffer, char *tempBuffer )
{
  int total_length = 0;

  if ( (*buffer) == NULL )
    {
      *buffer = tempBuffer;
      return TK_SUCCESS;
    }
 
  total_length = strlen ( *buffer ) + strlen ( tempBuffer ) + 4;
  if ( total_length > TK_DB_STRING_LENGTH )
    {
      *buffer = realloc ( *buffer, ( total_length + 1 ) *
			    sizeof ( char ) );
      if ( ( *buffer ) == NULL )
	{
	  TKreportWarning ( E_TK_BADMEMALLCAT );
	  return TK_FAIL;
	}
    }
  strcat ( *buffer, ", " );
  strcat ( *buffer, tempBuffer );

  free ( tempBuffer );

  return TK_SUCCESS;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       int query_db ( char *datatype, char **buffer1, char **buffer2, 
*	               char **buffer3, FILE *fin )
*
*  DESCRIPTION:
*       This routine will query the ASCII database and return the values
*       in the three buffers.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       datatype                The datatype or granule ID to query the database
*
*       fin                     The handle to the database.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       buffer1                 A buffer to information queried from the database.
*
*       buffer2                 A buffer to information queried from the database.
*
*       buffer3                 A buffer to information queried from the database.
*
*  CALLED BY:                   query_ascii_db
*
*  RETURN VALUES:               W_TK_NO_RECORD, TK_SUCCESS
*
*  ROUTINES CALLED:
*          TKreportWarning
*          extract_word
*          addString
*
******************************************************************************/
int query_db ( char *datatype, char **buffer1, char **buffer2, 
	       char **buffer3, FILE *fin )
{
  int linecount;
  char buffer[TK_DB_STRING_LENGTH];
  int buffer_pos;
  char *tempBuffer;
  int status = W_TK_NO_RECORD;

  for ( linecount = 0; fgets ( buffer, TK_DB_STRING_LENGTH, fin ); linecount++ )
    {
      buffer_pos = 0;
      if ( ( buffer[0] != '#' ) && ( buffer[0] != '\n' ) )
	if ( !strcmp ( extract_word ( &buffer_pos, buffer ), datatype ) )
	  {
	    status = TK_SUCCESS;

	    tempBuffer = extract_word ( &buffer_pos, buffer );
	    if ( tempBuffer == NULL )
	      return TK_FAIL;
	    status = addString ( buffer1, tempBuffer );
	    if ( status != TK_SUCCESS )
	      return TK_FAIL;
	    tempBuffer = extract_word ( &buffer_pos, buffer );
	    if ( tempBuffer == NULL )
	      return TK_FAIL;
	    status = addString ( buffer2, tempBuffer );
	    if ( status != TK_SUCCESS )
	      return TK_FAIL;
	    tempBuffer= extract_word ( &buffer_pos, buffer );
	    if ( tempBuffer == NULL ) 
	      return TK_FAIL;
	    status = addString ( buffer3, tempBuffer );
	    if ( status != TK_SUCCESS )
	      return TK_FAIL;
	  }
    }
  if ( status == W_TK_NO_RECORD )
    TKreportWarning ( W_TK_NO_RECORD );
  return status;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       void query_ascii_db ( IO_HANDLE *sGranuleHandle, 
*                             STORE_IM_DM_QUERY *results )
*
*  DESCRIPTION:
*       This routine is the main interface to the ASCII query database 
*       system.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the 
*				file from which data can be obtained.  It also
*                               contains metadata information.
*
*       STORE_IM_DM_QUERY       A structure to store all of the queried parameters.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   initMetadata
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*          TKreportError
*          open_db
*          query_db
*
******************************************************************************/
void query_ascii_db ( IO_HANDLE *sGranuleHandle, STORE_IM_DM_QUERY *results )
{
  FILE *sFilePtr = NULL;
  char *dataID = NULL, *contact = NULL, *algver = NULL;
  char *browse = NULL, *lmaxl = NULL, *leapsec = NULL;
  char *input = NULL, *date = NULL, *center = NULL;
  int status = TK_SUCCESS;

  sFilePtr = open_db ( "DataTypeMetadata.db" );
  if ( sFilePtr == NULL )
    {
      TKreportWarning ( W_TK_FAILOPENDTM );
      return;
    }
  
  status = query_db ( sGranuleHandle->gIDtokens.datatype, &dataID, 
		     &contact, &algver, sFilePtr );
  if ( status != TK_SUCCESS )
    {
      TKreportWarning ( W_TK_ASCII_DB_FAIL );
      return;
    }

  sprintf ( results->dataID, "\"%s\"", dataID );
  sprintf ( results->contact, "\"%s\"", contact );
  sprintf ( results->algorithmVersion, "\"%s\"", algver );

  free ( dataID );
  free ( contact );
  free ( algver );
  fclose ( sFilePtr );


  sFilePtr = open_db ( "GranuleMetadata.db" );
  if ( sFilePtr == NULL )
    {
      TKreportWarning ( W_TK_FAILOPENDTM );
      return;
    }

  status = query_db ( sGranuleHandle->gIDtokens.granule_ID, &browse, &lmaxl,
		     &leapsec, sFilePtr );
  if ( status != TK_SUCCESS )
    {
      TKreportWarning ( W_TK_ASCII_DB_FAIL );
      return;
    }

  sprintf ( results->browseName, "\"%s\"", browse );
  strcpy ( results->lonmaxlat, lmaxl );
  strcpy ( results->leapSecsFlag, leapsec );
  
  free ( browse );
  free ( lmaxl );
  free ( leapsec );
  fclose ( sFilePtr );

  sFilePtr = open_db ( "GranuleInput.db" );
  if ( sFilePtr == NULL )
    {
      TKreportWarning ( W_TK_FAILOPENDTM );
      return;
    }
  
  status = query_db ( sGranuleHandle->gIDtokens.granule_ID, &input, &date,
		     &center, sFilePtr );
  if ( status != TK_SUCCESS )
    {
      TKreportWarning ( W_TK_ASCII_DB_FAIL );
      return;
    }

  sprintf ( results->inputFiles, "\"%s\"", input );
  sprintf ( results->gendateinputfile, "\"%s\"", date );
  sprintf ( results->datacentsrc, "\"%s\"", center );

  free ( input );
  free ( date );
  free ( center );
  fclose ( sFilePtr );


  return;
}
