/* Revision 1.2  1997/03/14 20:42:15  slywczak
 * Added the CVS header ($Log&) to the source file.  This was missing on the
 * original insertion.
 * */

/**********************************************************************************
*
*  FILE NAME:           TKcopyHeadr.c
*
*  DESCRIPTION:         This file contains routines that copies the scan header
*                       from the input structure to the output structure.  The 
*                       data structure copied are the scan time, scan status,
*                       navigation, and the geolocation.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       TKcopyScanHeader        This routine is the main driver for copying the
*                               scan header.  It will check the types of the
*                               input arguments and call the appropriate copy
*                               function.
*       copy1B01ScanHeader      Auxillary function that copies the VIRS 1B01
*                               scan header.  Based on the input parameters, 
*                               this function will copy the scan header to either
*                               the temporary structure or the output structure.
*       copy1B11ScanHeader      Auxillary function that copies the TMI 1B11
*                               scan header.  Based on the input parameters, 
*                               this function will copy the scan header to either
*                               the temporary structure or the output structure.
*       copy2A12ScanHeader      Auxillary function that copies the TMI 2A12
*                               scan header.  Based on the input parameters, 
*                               this function will copy the scan header to either
*                               the temporary structure or the output structure.
*       copy1B21ScanHeader      Auxillary function that copies the PR 1B21
*                               scan header.  Based on the input parameters, 
*                               this function will copy the scan header to either
*                               the temporary structure or the output structure.
*       copy1C21ScanHeader      Auxillary function that copies the PR 1C21
*                               scan header.  Based on the input parameters, 
*                               this function will copy the scan header to either
*                               the temporary structure or the output structure.
*       copy2A21ScanHeader      Auxillary function that copies the PR 2A21
*                               scan header.  Based on the input parameters, 
*                               this function will copy the scan header to either
*                               the temporary structure or the output structure.
*       copy2A23ScanHeader      Auxillary function that copies the PR 2A23
*                               scan header.  Based on the input parameters, 
*                               this function will copy the scan header to either
*                               the temporary structure or the output structure.
*       copy2A25ScanHeader      Auxillary function that copies the PR 2A25
*                               scan header.  Based on the input parameters, 
*                               this function will copy the scan header to either
*                               the temporary structure or the output structure.
*       copy2B31ScanHeader      Auxillary function that copies the PR 2B31
*                               scan header.  Based on the input parameters, 
*                               this function will copy the scan header to either
*                               the temporary structure or the output structure.
*       TKreportWarning         This routine records warnings in a file.
*
**********************************************************************************/

#ifndef LANGUAGE_FORTRAN
#include "TKcopyHeadr.h"
#endif

/**********************************************************************************
*
*  FUNCTION STATEMENT:
*      void copy1B01ScanHeader(L1B_01_SWATHDATA *s1B01DataBuffer, 
*                              VIRS_COPY_HEADER *sVirsWorkingCopy, int numPixels,
*                              int copyDirection)
*
*  DESCRIPTION:                 This routine copies the scan header for the VIRS
*                               1B01 data product.   The scan header is copied 
*                               from the input data buffer to the temporary buffer,
*                               or the temporary data buffer to the output buffer. 
*                               The decision is based on the value of copyDirection.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s1B01DataBuffer         The input data buffer.  This buffer will hold the 
*                               scan header values that are read from the input 
*                               file.  This parameter is an input parameter 
*                               when copyDirection is set to COPY_FROM_INPUT.
*       sVirsWorkingCopy        A temporary working buffer to holds the scan header
*                               information as it is being copied from the input
*                               to the output.  This parameter is an input 
*                               parameter when copyDirection is set to 
*                               COPY_TO_OUTPUT.
*       numPixels               An integer that indicates the number of pixels
*                               per scan for the VIRS instrument.  The values
*                               are defined in IO.h.  (NOTE:  These values are 
*                               also defined in TK_GLOBAL.h.  When the toolkit
*                               has been integrated with the globals file, this
*                               flag will be modified). 
*       copyDirection           A flag value to indicate whether the data should be
*                               copied from the data buffer to the temporary buffer
*                               or the temporary buffer to the data buffer.  The
*                               values for the flag are defined in TKcopyHeadr.h,
*                               and it can take the values of COPY_FROM_INPUT or
*                               COPY_TO_OUTPUT.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s1B01DataBuffer         The output data buffer.  This buffer will hold
*                               the scan headers values copied from the temporary
*                               buffer.  This parameter is an output parameter when
*                               copyDirection is set to COPY_TO_OUTPUT.
*       sVirsWorkingCopy        A temporary working buffer the holds the scan header
*                               information.  This information is copied from the input
*                               data buffer to this structure.  This parameter is an
*                               output parameter when copyDirection is set to 
*                               COPY_FROM_INPUT.
*
*  CALLED BY:                   TKcopyScanHeader
*
*  RETURN VALUES:		None
*
*  ROUTINES CALLED:             None
*
*******************************************************************************************/

void copy1B01ScanHeader ( L1B_01_SWATHDATA *s1B01DataBuffer, 
			 VIRS_COPY_HEADER *sVirsWorkingCopy, int numPixels,
			 int copyDirection )
{
  int i, j;

  /*  
   *  Check the value of the copyDirection.  If it is set to COPY_FROM_INPUT
   *  then copy to the tempoary structure, otherwise copy to the output
   *  buffer.
   */
  switch ( copyDirection )
    { 
    case COPY_FROM_INPUT : /*  Copy the input buffer to the temporary buffer  */
      sVirsWorkingCopy->scanTime = s1B01DataBuffer->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_VIRS_PIXELS; j++ ) 
	  sVirsWorkingCopy->geolocation[j][i] = s1B01DataBuffer->geolocation[j][i];
      sVirsWorkingCopy->scanStatus = s1B01DataBuffer->scanStatus; 
      sVirsWorkingCopy->navigate = s1B01DataBuffer->navigate; 
      break;
    case COPY_TO_OUTPUT :  /*  Copy the temporary buffer to the output buffer  */
      s1B01DataBuffer->scanTime = sVirsWorkingCopy->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_VIRS_PIXELS; j++ ) 
	  s1B01DataBuffer->geolocation[j][i] = sVirsWorkingCopy->geolocation[j][i];
      s1B01DataBuffer->scanStatus = sVirsWorkingCopy->scanStatus;
      s1B01DataBuffer->navigate = sVirsWorkingCopy->navigate;
      break;
    }
  return;
}

/**********************************************************************************
*
*  FUNCTION STATEMENT:
*      void copy1B11ScanHeader(L1B_11_SWATHDATA *s1B11DataBuffer, 
*                              TMI_COPY_HEADER *sTmiWorkingCopy, int numPixels,
*                              int copyDirection)
*
*  DESCRIPTION:                 This routine copies the scan header for the TMI
*                               1B11 data product.   The scan header is copied 
*                               from the input data buffer to the temporary buffer,
*                               or the temporary data buffer to the output buffer. 
*                               The decision is based on the value of copyDirection.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s1B11DataBuffer         The input data buffer.  This buffer will hold the 
*                               scan header values that are read from the input 
*                               file.  This parameter is an input parameter 
*                               when copyDirection is set to COPY_FROM_INPUT.
*       sTmiWorkingCopy         A temporary working buffer to holds the scan header
*                               information as it is being copied from the input
*                               to the output.  This parameter is an input 
*                               parameter when copyDirection is set to 
*                               COPY_TO_OUTPUT.
*       numPixels               An integer that indicates the number of pixels
*                               per scan for the TMI instrument.  The values
*                               are defined in IO.h.  (NOTE:  These values are 
*                               also defined in TK_GLOBAL.h.  When the toolkit
*                               has been integrated with the globals file, this
*                               flag will be modified). 
*       copyDirection           A flag value to indicate whether the data should be
*                               copied from the data buffer to the temporary buffer
*                               or the temporary buffer to the data buffer.  The
*                               values for the flag are defined in TKcopyHeadr.h,
*                               and it can take the values of COPY_FROM_INPUT or
*                               COPY_TO_OUTPUT.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s1B11DataBuffer         The output data buffer.  This buffer will hold
*                               the scan headers values copied from the temporary
*                               buffer.  This parameter is an output parameter when
*                               copyDirection is set to COPY_TO_OUTPUT.
*       sTmiWorkingCopy         A temporary working buffer the holds the scan header
*                               information.  This information is copied from the input
*                               data buffer to this structure.  This parameter is an
*                               output parameter when copyDirection is set to 
*                               COPY_FROM_INPUT.
*
*  CALLED BY:                   TKcopyScanHeader
*
*  RETURN VALUES:		None
*
*  ROUTINES CALLED:             None
*
*******************************************************************************************/

void copy1B11ScanHeader ( L1B_11_SWATHDATA *s1B11DataBuffer, 
			 TMI_COPY_HEADER *sTmiWorkingCopy, int numPixels,
			 int copyDirection )
{
  int i, j;

  /*  
   *  Check the value of the copyDirection.  If it is set to COPY_FROM_INPUT
   *  then copy to the tempoary structure, otherwise copy to the output
   *  buffer.
   */
  switch ( copyDirection )
    {
    case COPY_FROM_INPUT :  /*  Copy the input buffer to the temporary buffer  */
      sTmiWorkingCopy->scanTime = s1B11DataBuffer->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_TMI_PIXELS; j++ ) 
	  sTmiWorkingCopy->geolocation[j][i] = s1B11DataBuffer->geolocation[j][i];
      sTmiWorkingCopy->scanStatus = s1B11DataBuffer->scanStatus; 
      sTmiWorkingCopy->navigate = s1B11DataBuffer->navigate; 
      break;
    case COPY_TO_OUTPUT :  /*  Copy the temporary buffer to the output buffer  */
      s1B11DataBuffer->scanTime = sTmiWorkingCopy->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_TMI_PIXELS; j++ ) 
	  s1B11DataBuffer->geolocation[j][i] = sTmiWorkingCopy->geolocation[j][i];
      s1B11DataBuffer->scanStatus = sTmiWorkingCopy->scanStatus;
      s1B11DataBuffer->navigate = sTmiWorkingCopy->navigate;
      break;
    }
  return;
}

/**********************************************************************************
*
*  FUNCTION STATEMENT:
*      void copy2A12ScanHeader(L2A_12_SWATHDATA *s2A12DataBuffer, 
*                              TMI_COPY_HEADER *sTmiWorkingCopy, int numPixels,
*                              int copyDirection)
*
*  DESCRIPTION:                 This routine copies the scan header for the TMI
*                               2A12 data product.   The scan header is copied 
*                               from the input data buffer to the temporary buffer,
*                               or the temporary data buffer to the output buffer. 
*                               The decision is based on the value of copyDirection.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s2A12DataBuffer         The input data buffer.  This buffer will hold the 
*                               scan header values that are read from the input 
*                               file.  This parameter is an input parameter 
*                               when copyDirection is set to COPY_FROM_INPUT.
*       sTmiWorkingCopy         A temporary working buffer to holds the scan header
*                               information as it is being copied from the input
*                               to the output.  This parameter is an input 
*                               parameter when copyDirection is set to 
*                               COPY_TO_OUTPUT.
*       numPixels               An integer that indicates the number of pixels
*                               per scan for the TMI instrument.  The values
*                               are defined in IO.h.  (NOTE:  These values are 
*                               also defined in TK_GLOBAL.h.  When the toolkit
*                               has been integrated with the globals file, this
*                               flag will be modified). 
*       copyDirection           A flag value to indicate whether the data should be
*                               copied from the data buffer to the temporary buffer
*                               or the temporary buffer to the data buffer.  The
*                               values for the flag are defined in TKcopyHeadr.h,
*                               and it can take the values of COPY_FROM_INPUT or
*                               COPY_TO_OUTPUT.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s2A12DataBuffer         The output data buffer.  This buffer will hold
*                               the scan headers values copied from the temporary
*                               buffer.  This parameter is an output parameter when
*                               copyDirection is set to COPY_TO_OUTPUT.
*       sTmiWorkingCopy         A temporary working buffer the holds the scan header
*                               information.  This information is copied from the input
*                               data buffer to this structure.  This parameter is an
*                               output parameter when copyDirection is set to 
*                               COPY_FROM_INPUT.
*
*  CALLED BY:                   TKcopyScanHeader
*
*  RETURN VALUES:		None
*
*  ROUTINES CALLED:             None
*
*******************************************************************************************/

void copy2A12ScanHeader ( L2A_12_SWATHDATA *s2A12DataBuffer, 
			 TMI_COPY_HEADER *sTmiWorkingCopy, int numPixels,
			 int copyDirection )
{
  int i, j;

  /*  
   *  Check the value of the copyDirection.  If it is set to COPY_FROM_INPUT
   *  then copy to the tempoary structure, otherwise copy to the output
   *  buffer.
   */
  switch ( copyDirection )
    {
    case COPY_FROM_INPUT :  /*  Copy the input buffer to the temporary buffer  */
      sTmiWorkingCopy->scanTime = s2A12DataBuffer->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_TMI_PIXELS; j++ ) 
	  sTmiWorkingCopy->geolocation[j][i] = s2A12DataBuffer->geolocation[j][i];
      sTmiWorkingCopy->scanStatus = s2A12DataBuffer->scanStatus; 
      sTmiWorkingCopy->navigate = s2A12DataBuffer->navigate; 
      break;
    case COPY_TO_OUTPUT :  /*  Copy the temporary buffer to the output buffer  */
      s2A12DataBuffer->scanTime = sTmiWorkingCopy->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_TMI_PIXELS; j++ ) 
	  s2A12DataBuffer->geolocation[j][i] = sTmiWorkingCopy->geolocation[j][i];
      s2A12DataBuffer->scanStatus = sTmiWorkingCopy->scanStatus;
      s2A12DataBuffer->navigate = sTmiWorkingCopy->navigate;
      break;
    }
  return;
}

/**********************************************************************************
*
*  FUNCTION STATEMENT:
*      void copy1B21ScanHeader(L1B_21_SWATHDATA *s1B21DataBuffer, 
*                              PR_COPY_HEADER *sPrWorkingCopy, int numPixels,
*                              int copyDirection)
*
*  DESCRIPTION:                 This routine copies the scan header for the PR
*                               1B21 data product.   The scan header is copied 
*                               from the input data buffer to the temporary buffer,
*                               or the temporary data buffer to the output buffer. 
*                               The decision is based on the value of copyDirection.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s1B21DataBuffer         The input data buffer.  This buffer will hold the 
*                               scan header values that are read from the input 
*                               file.  This parameter is an input parameter 
*                               when copyDirection is set to COPY_FROM_INPUT.
*       sPrWorkingCopy          A temporary working buffer to holds the scan header
*                               information as it is being copied from the input
*                               to the output.  This parameter is an input 
*                               parameter when copyDirection is set to 
*                               COPY_TO_OUTPUT.
*       numPixels               An integer that indicates the number of pixels
*                               per scan for the PR instrument.  The values
*                               are defined in IO.h.  (NOTE:  These values are 
*                               also defined in TK_GLOBAL.h.  When the toolkit
*                               has been integrated with the globals file, this
*                               flag will be modified). 
*       copyDirection           A flag value to indicate whether the data should be
*                               copied from the data buffer to the temporary buffer
*                               or the temporary buffer to the data buffer.  The
*                               values for the flag are defined in TKcopyHeadr.h,
*                               and it can take the values of COPY_FROM_INPUT or
*                               COPY_TO_OUTPUT.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s1B21DataBuffer         The output data buffer.  This buffer will hold
*                               the scan headers values copied from the temporary
*                               buffer.  This parameter is an output parameter when
*                               copyDirection is set to COPY_TO_OUTPUT.
*       sPrWorkingCopy          A temporary working buffer the holds the scan header
*                               information.  This information is copied from the input
*                               data buffer to this structure.  This parameter is an
*                               output parameter when copyDirection is set to 
*                               COPY_FROM_INPUT.
*
*  CALLED BY:                   TKcopyScanHeader
*
*  RETURN VALUES:		None
*
*  ROUTINES CALLED:             None
*
*******************************************************************************************/

void copy1B21ScanHeader ( L1B_21_SWATHDATA *s1B21DataBuffer, 
			 PR_COPY_HEADER *sPrWorkingCopy, int numPixels,
			 int copyDirection )
{
  int i, j;

  /*  
   *  Check the value of the copyDirection.  If it is set to COPY_FROM_INPUT
   *  then copy to the tempoary structure, otherwise copy to the output
   *  buffer.
   */
  switch ( copyDirection )
    {
    case COPY_FROM_INPUT :  /*  Copy the input buffer to the temporary buffer  */
      sPrWorkingCopy->scanTime = s1B21DataBuffer->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_PR_PIXELS; j++ ) 
	  sPrWorkingCopy->geolocation[j][i] = s1B21DataBuffer->geolocation[j][i];
      sPrWorkingCopy->scanStatus = s1B21DataBuffer->scanStatus; 
      sPrWorkingCopy->navigate = s1B21DataBuffer->navigate; 
      break;
    case COPY_TO_OUTPUT :  /*  Copy the temporary buffer to the output buffer  */
      s1B21DataBuffer->scanTime = sPrWorkingCopy->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_PR_PIXELS; j++ ) 
	  s1B21DataBuffer->geolocation[j][i] = sPrWorkingCopy->geolocation[j][i];
      s1B21DataBuffer->scanStatus = sPrWorkingCopy->scanStatus;
      s1B21DataBuffer->navigate = sPrWorkingCopy->navigate;
      break;
    }
  return;
}

/**********************************************************************************
*
*  FUNCTION STATEMENT:
*      void copy1C21ScanHeader(L1C_21_SWATHDATA *s1C21DataBuffer, 
*                              PR_COPY_HEADER *sPrWorkingCopy, int numPixels,
*                              int copyDirection)
*
*  DESCRIPTION:                 This routine copies the scan header for the PR
*                               1C21 data product.   The scan header is copied 
*                               from the input data buffer to the temporary buffer,
*                               or the temporary data buffer to the output buffer. 
*                               The decision is based on the value of copyDirection.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s1C21DataBuffer         The input data buffer.  This buffer will hold the 
*                               scan header values that are read from the input 
*                               file.  This parameter is an input parameter 
*                               when copyDirection is set to COPY_FROM_INPUT.
*       sPrWorkingCopy          A temporary working buffer to holds the scan header
*                               information as it is being copied from the input
*                               to the output.  This parameter is an input 
*                               parameter when copyDirection is set to 
*                               COPY_TO_OUTPUT.
*       numPixels               An integer that indicates the number of pixels
*                               per scan for the PR instrument.  The values
*                               are defined in IO.h.  (NOTE:  These values are 
*                               also defined in TK_GLOBAL.h.  When the toolkit
*                               has been integrated with the globals file, this
*                               flag will be modified). 
*       copyDirection           A flag value to indicate whether the data should be
*                               copied from the data buffer to the temporary buffer
*                               or the temporary buffer to the data buffer.  The
*                               values for the flag are defined in TKcopyHeadr.h,
*                               and it can take the values of COPY_FROM_INPUT or
*                               COPY_TO_OUTPUT.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s1C21DataBuffer         The output data buffer.  This buffer will hold
*                               the scan headers values copied from the temporary
*                               buffer.  This parameter is an output parameter when
*                               copyDirection is set to COPY_TO_OUTPUT.
*       sPrWorkingCopy          A temporary working buffer the holds the scan header
*                               information.  This information is copied from the input
*                               data buffer to this structure.  This parameter is an
*                               output parameter when copyDirection is set to 
*                               COPY_FROM_INPUT.
*
*  CALLED BY:                   TKcopyScanHeader
*
*  RETURN VALUES:		None
*
*  ROUTINES CALLED:             None
*
*******************************************************************************************/

void copy1C21ScanHeader ( L1C_21_SWATHDATA *s1C21DataBuffer, 
			 PR_COPY_HEADER *sPrWorkingCopy, int numPixels,
			 int copyDirection )
{
  int i, j;

  /*  
   *  Check the value of the copyDirection.  If it is set to COPY_FROM_INPUT
   *  then copy to the tempoary structure, otherwise copy to the output
   *  buffer.
   */
  switch ( copyDirection )
    {
    case COPY_FROM_INPUT :  /*  Copy the input buffer to the temporary buffer  */
      sPrWorkingCopy->scanTime = s1C21DataBuffer->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_PR_PIXELS; j++ ) 
	  sPrWorkingCopy->geolocation[j][i] = s1C21DataBuffer->geolocation[j][i];
      sPrWorkingCopy->scanStatus = s1C21DataBuffer->scanStatus; 
      sPrWorkingCopy->navigate = s1C21DataBuffer->navigate; 
      break;
    case COPY_TO_OUTPUT :  /*  Copy the temporary buffer to the output buffer  */
      s1C21DataBuffer->scanTime = sPrWorkingCopy->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_PR_PIXELS; j++ ) 
	  s1C21DataBuffer->geolocation[j][i] = sPrWorkingCopy->geolocation[j][i];
      s1C21DataBuffer->scanStatus = sPrWorkingCopy->scanStatus;
      s1C21DataBuffer->navigate = sPrWorkingCopy->navigate;
      break;
    }
  return;
}

/**********************************************************************************
*
*  FUNCTION STATEMENT:
*      void copy2A21ScanHeader(L2A_21_SWATHDATA *s2A21DataBuffer, 
*                              PR_COPY_HEADER *sPrWorkingCopy, int numPixels,
*                              int copyDirection)
*
*  DESCRIPTION:                 This routine copies the scan header for the PR
*                               2A21 data product.   The scan header is copied 
*                               from the input data buffer to the temporary buffer,
*                               or the temporary data buffer to the output buffer. 
*                               The decision is based on the value of copyDirection.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s2A21DataBuffer         The input data buffer.  This buffer will hold the 
*                               scan header values that are read from the input 
*                               file.  This parameter is an input parameter 
*                               when copyDirection is set to COPY_FROM_INPUT.
*       sPrWorkingCopy          A temporary working buffer to holds the scan header
*                               information as it is being copied from the input
*                               to the output.  This parameter is an input 
*                               parameter when copyDirection is set to 
*                               COPY_TO_OUTPUT.
*       numPixels               An integer that indicates the number of pixels
*                               per scan for the PR instrument.  The values
*                               are defined in IO.h.  (NOTE:  These values are 
*                               also defined in TK_GLOBAL.h.  When the toolkit
*                               has been integrated with the globals file, this
*                               flag will be modified). 
*       copyDirection           A flag value to indicate whether the data should be
*                               copied from the data buffer to the temporary buffer
*                               or the temporary buffer to the data buffer.  The
*                               values for the flag are defined in TKcopyHeadr.h,
*                               and it can take the values of COPY_FROM_INPUT or
*                               COPY_TO_OUTPUT.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s2A21DataBuffer         The output data buffer.  This buffer will hold
*                               the scan headers values copied from the temporary
*                               buffer.  This parameter is an output parameter when
*                               copyDirection is set to COPY_TO_OUTPUT.
*       sPrWorkingCopy          A temporary working buffer the holds the scan header
*                               information.  This information is copied from the input
*                               data buffer to this structure.  This parameter is an
*                               output parameter when copyDirection is set to 
*                               COPY_FROM_INPUT.
*
*  CALLED BY:                   TKcopyScanHeader
*
*  RETURN VALUES:		None
*
*  ROUTINES CALLED:             None
*
*******************************************************************************************/

void copy2A21ScanHeader ( L2A_21_SWATHDATA *s2A21DataBuffer, 
			 PR_COPY_HEADER *sPrWorkingCopy, int numPixels,
			 int copyDirection )
{
  int i, j;

  /*  
   *  Check the value of the copyDirection.  If it is set to COPY_FROM_INPUT
   *  then copy to the tempoary structure, otherwise copy to the output
   *  buffer.
   */
  switch ( copyDirection )
    {
    case COPY_FROM_INPUT :  /*  Copy the input buffer to the temporary buffer  */
      sPrWorkingCopy->scanTime = s2A21DataBuffer->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_PR_PIXELS; j++ ) 
	  sPrWorkingCopy->geolocation[j][i] = s2A21DataBuffer->geolocation[j][i];
      sPrWorkingCopy->scanStatus = s2A21DataBuffer->scanStatus; 
      sPrWorkingCopy->navigate = s2A21DataBuffer->navigate; 
      break;
    case COPY_TO_OUTPUT :  /*  Copy the temporary buffer to the output buffer  */
      s2A21DataBuffer->scanTime = sPrWorkingCopy->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_PR_PIXELS; j++ ) 
	  s2A21DataBuffer->geolocation[j][i] = sPrWorkingCopy->geolocation[j][i];
      s2A21DataBuffer->scanStatus = sPrWorkingCopy->scanStatus;
      s2A21DataBuffer->navigate = sPrWorkingCopy->navigate;
      break;
    }
  return;
}

/**********************************************************************************
*
*  FUNCTION STATEMENT:
*      void copy2A23ScanHeader(L2A_23_SWATHDATA *s2A23DataBuffer, 
*                              PR_COPY_HEADER *sPrWorkingCopy, int numPixels,
*                              int copyDirection)
*
*  DESCRIPTION:                 This routine copies the scan header for the PR
*                               2A23 data product.   The scan header is copied 
*                               from the input data buffer to the temporary buffer,
*                               or the temporary data buffer to the output buffer. 
*                               The decision is based on the value of copyDirection.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s2A23DataBuffer         The input data buffer.  This buffer will hold the 
*                               scan header values that are read from the input 
*                               file.  This parameter is an input parameter 
*                               when copyDirection is set to COPY_FROM_INPUT.
*       sPrWorkingCopy          A temporary working buffer to holds the scan header
*                               information as it is being copied from the input
*                               to the output.  This parameter is an input 
*                               parameter when copyDirection is set to 
*                               COPY_TO_OUTPUT.
*       numPixels               An integer that indicates the number of pixels
*                               per scan for the PR instrument.  The values
*                               are defined in IO.h.  (NOTE:  These values are 
*                               also defined in TK_GLOBAL.h.  When the toolkit
*                               has been integrated with the globals file, this
*                               flag will be modified). 
*       copyDirection           A flag value to indicate whether the data should be
*                               copied from the data buffer to the temporary buffer
*                               or the temporary buffer to the data buffer.  The
*                               values for the flag are defined in TKcopyHeadr.h,
*                               and it can take the values of COPY_FROM_INPUT or
*                               COPY_TO_OUTPUT.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s2A23DataBuffer         The output data buffer.  This buffer will hold
*                               the scan headers values copied from the temporary
*                               buffer.  This parameter is an output parameter when
*                               copyDirection is set to COPY_TO_OUTPUT.
*       sPrWorkingCopy          A temporary working buffer the holds the scan header
*                               information.  This information is copied from the input
*                               data buffer to this structure.  This parameter is an
*                               output parameter when copyDirection is set to 
*                               COPY_FROM_INPUT.
*
*  CALLED BY:                   TKcopyScanHeader
*
*  RETURN VALUES:		None
*
*  ROUTINES CALLED:             None
*
*******************************************************************************************/

void copy2A23ScanHeader ( L2A_23_SWATHDATA *s2A23DataBuffer, 
			 PR_COPY_HEADER *sPrWorkingCopy, int numPixels,
			 int copyDirection )
{
  int i, j;

  /*  
   *  Check the value of the copyDirection.  If it is set to COPY_FROM_INPUT
   *  then copy to the tempoary structure, otherwise copy to the output
   *  buffer.
   */
  switch ( copyDirection )
    {
    case COPY_FROM_INPUT :  /*  Copy the input buffer to the temporary buffer  */
      sPrWorkingCopy->scanTime = s2A23DataBuffer->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_PR_PIXELS; j++ ) 
	  sPrWorkingCopy->geolocation[j][i] = s2A23DataBuffer->geolocation[j][i];
      sPrWorkingCopy->scanStatus = s2A23DataBuffer->scanStatus; 
      sPrWorkingCopy->navigate = s2A23DataBuffer->navigate; 
      break;
    case COPY_TO_OUTPUT :  /*  Copy the temporary buffer to the output buffer  */
      s2A23DataBuffer->scanTime = sPrWorkingCopy->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_PR_PIXELS; j++ ) 
	  s2A23DataBuffer->geolocation[j][i] = sPrWorkingCopy->geolocation[j][i];
      s2A23DataBuffer->scanStatus = sPrWorkingCopy->scanStatus;
      s2A23DataBuffer->navigate = sPrWorkingCopy->navigate;
      break;
    }
  return;
}

/**********************************************************************************
*
*  FUNCTION STATEMENT:
*      void copy2A25ScanHeader(L2A_25_SWATHDATA *s2A25DataBuffer, 
*                              PR_COPY_HEADER *sPrWorkingCopy, int numPixels,
*                              int copyDirection)
*
*  DESCRIPTION:                 This routine copies the scan header for the PR
*                               2A25 data product.   The scan header is copied 
*                               from the input data buffer to the temporary buffer,
*                               or the temporary data buffer to the output buffer. 
*                               The decision is based on the value of copyDirection.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s2A25DataBuffer         The input data buffer.  This buffer will hold the 
*                               scan header values that are read from the input 
*                               file.  This parameter is an input parameter 
*                               when copyDirection is set to COPY_FROM_INPUT.
*       sPrWorkingCopy          A temporary working buffer to holds the scan header
*                               information as it is being copied from the input
*                               to the output.  This parameter is an input 
*                               parameter when copyDirection is set to 
*                               COPY_TO_OUTPUT.
*       numPixels               An integer that indicates the number of pixels
*                               per scan for the PR instrument.  The values
*                               are defined in IO.h.  (NOTE:  These values are 
*                               also defined in TK_GLOBAL.h.  When the toolkit
*                               has been integrated with the globals file, this
*                               flag will be modified). 
*       copyDirection           A flag value to indicate whether the data should be
*                               copied from the data buffer to the temporary buffer
*                               or the temporary buffer to the data buffer.  The
*                               values for the flag are defined in TKcopyHeadr.h,
*                               and it can take the values of COPY_FROM_INPUT or
*                               COPY_TO_OUTPUT.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s2A25DataBuffer         The output data buffer.  This buffer will hold
*                               the scan headers values copied from the temporary
*                               buffer.  This parameter is an output parameter when
*                               copyDirection is set to COPY_TO_OUTPUT.
*       sPrWorkingCopy          A temporary working buffer the holds the scan header
*                               information.  This information is copied from the input
*                               data buffer to this structure.  This parameter is an
*                               output parameter when copyDirection is set to 
*                               COPY_FROM_INPUT.
*
*  CALLED BY:                   TKcopyScanHeader
*
*  RETURN VALUES:		None
*
*  ROUTINES CALLED:             None
*
*******************************************************************************************/

void copy2A25ScanHeader ( L2A_25_SWATHDATA *s2A25DataBuffer, 
			 PR_COPY_HEADER *sPrWorkingCopy, int numPixels,
			 int copyDirection )
{
  int i, j;

  /*  
   *  Check the value of the copyDirection.  If it is set to COPY_FROM_INPUT
   *  then copy to the tempoary structure, otherwise copy to the output
   *  buffer.
   */
  switch ( copyDirection )
    {
    case COPY_FROM_INPUT :  /*  Copy the input buffer to the temporary buffer  */
      sPrWorkingCopy->scanTime = s2A25DataBuffer->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_PR_PIXELS; j++ ) 
	  sPrWorkingCopy->geolocation[j][i] = s2A25DataBuffer->geolocation[j][i];
      sPrWorkingCopy->scanStatus = s2A25DataBuffer->scanStatus; 
      sPrWorkingCopy->navigate = s2A25DataBuffer->navigate; 
      break;
    case COPY_TO_OUTPUT :  /*  Copy the temporary buffer to the output buffer  */
      s2A25DataBuffer->scanTime = sPrWorkingCopy->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_PR_PIXELS; j++ ) 
	  s2A25DataBuffer->geolocation[j][i] = sPrWorkingCopy->geolocation[j][i];
      s2A25DataBuffer->scanStatus = sPrWorkingCopy->scanStatus;
      s2A25DataBuffer->navigate = sPrWorkingCopy->navigate;
      break;
    }
  return;
}

/**********************************************************************************
*
*  FUNCTION STATEMENT:
*      void copy2B31ScanHeader(L2B_31_SWATHDATA *s2B31DataBuffer, 
*                              PR_COPY_HEADER *sPrWorkingCopy, int numPixels,
*                              int copyDirection)
*
*  DESCRIPTION:                 This routine copies the scan header for the PR
*                               2B31 data product.   The scan header is copied 
*                               from the input data buffer to the temporary buffer,
*                               or the temporary data buffer to the output buffer. 
*                               The decision is based on the value of copyDirection.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s2B31DataBuffer         The input data buffer.  This buffer will hold the 
*                               scan header values that are read from the input 
*                               file.  This parameter is an input parameter 
*                               when copyDirection is set to COPY_FROM_INPUT.
*       sPrWorkingCopy          A temporary working buffer to holds the scan header
*                               information as it is being copied from the input
*                               to the output.  This parameter is an input 
*                               parameter when copyDirection is set to 
*                               COPY_TO_OUTPUT.
*       numPixels               An integer that indicates the number of pixels
*                               per scan for the PR instrument.  The values
*                               are defined in IO.h.  (NOTE:  These values are 
*                               also defined in TK_GLOBAL.h.  When the toolkit
*                               has been integrated with the globals file, this
*                               flag will be modified). 
*       copyDirection           A flag value to indicate whether the data should be
*                               copied from the data buffer to the temporary buffer
*                               or the temporary buffer to the data buffer.  The
*                               values for the flag are defined in TKcopyHeadr.h,
*                               and it can take the values of COPY_FROM_INPUT or
*                               COPY_TO_OUTPUT.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s2B31DataBuffer         The output data buffer.  This buffer will hold
*                               the scan headers values copied from the temporary
*                               buffer.  This parameter is an output parameter when
*                               copyDirection is set to COPY_TO_OUTPUT.
*       sPrWorkingCopy          A temporary working buffer the holds the scan header
*                               information.  This information is copied from the input
*                               data buffer to this structure.  This parameter is an
*                               output parameter when copyDirection is set to 
*                               COPY_FROM_INPUT.
*
*  CALLED BY:                   TKcopyScanHeader
*
*  RETURN VALUES:		None
*
*  ROUTINES CALLED:             None
*
*******************************************************************************************/

void copy2B31ScanHeader ( L2B_31_SWATHDATA *s2B31DataBuffer, 
			 PR_COPY_HEADER *sPrWorkingCopy, int numPixels,
			 int copyDirection )
{
  int i, j;

  /*  
   *  Check the value of the copyDirection.  If it is set to COPY_FROM_INPUT
   *  then copy to the tempoary structure, otherwise copy to the output
   *  buffer.
   */
  switch ( copyDirection )
    {
    case COPY_FROM_INPUT :  /*  Copy the input buffer to the temporary buffer  */
      sPrWorkingCopy->scanTime = s2B31DataBuffer->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_PR_PIXELS; j++ ) 
	  sPrWorkingCopy->geolocation[j][i] = s2B31DataBuffer->geolocation[j][i];
      sPrWorkingCopy->scanStatus = s2B31DataBuffer->scanStatus; 
      sPrWorkingCopy->navigate = s2B31DataBuffer->navigate; 
      break;
    case COPY_TO_OUTPUT :  /*  Copy the temporary buffer to the output buffer  */
      s2B31DataBuffer->scanTime = sPrWorkingCopy->scanTime;
      for ( i = 0; i < 2; i++ )
	for ( j = 0; j < TK_PR_PIXELS; j++ ) 
	  s2B31DataBuffer->geolocation[j][i] = sPrWorkingCopy->geolocation[j][i];
      s2B31DataBuffer->scanStatus = sPrWorkingCopy->scanStatus;
      s2B31DataBuffer->navigate = sPrWorkingCopy->navigate;
      break;
    }
  return;
}

/**********************************************************************************
*
*  FUNCTION STATEMENT:
*       int TKcopyScanHeader(IO_HANDLE *sInGranuleHandle, void *sInDataHandle,
*                            IO_HANDLE *sOutGranuleHandle, void *sOutDataHandle)
*
*  DESCRIPTION:                 This routine is the driver for copying the scan
*                               header between the input data structure and the 
*                               output data structure.  The structures that are
*                               copied are the scan time, scan status, navigation,
*                               and the geolocation.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sInGranuleHandle        The input granule handle.  It contains the product
*                               id and the file mode associated with the input
*                               file.  Both of these parameters are validated.
*       sInDataHandle           The input data buffer.  It contains the original
*                               scan header information that is copied.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sOutGranuleHandle       The output granule handle.  This parameter contains
*                               the product id and the file mode associated with
*                               the output product.  
*       sOutDataHandle          The output data buffer.  The input scan header
*                               information will be copied to this data buffer.
*
*  CALLED BY:                   main program (science algorithm)
*
*  RETURN VALUES:		W_TK_BADMODECH, W_TK_BADIDCH, TK_SUCCESS
*
*  ROUTINES CALLED:             
*       Routine Name            Description
*       ------------            -----------
*       copy1B01ScanHeader      Copies the VIRS 1B01 Scan Header to either the output
*                               buffer or the temporary buffer depending on the input
*                               parameters.
*       copy1B11ScanHeader      Copies the TMI 1B11 Scan Header to either the output
*                               buffer or the temporary buffer depending on the input
*                               parameters.
*       copy2A12ScanHeader      Copies the TMI 2A12 Scan Header to either the output
*                               buffer or the temporary buffer depending on the input
*                               parameters.
*       copy1B21ScanHeader      Copies the PR 1B21 Scan Header to either the output
*                               buffer or the temporary buffer depending on the input
*                               parameters.
*       copy1C21ScanHeader      Copies the PR 1C21 Scan Header to either the output
*                               buffer or the temporary buffer depending on the input
*                               parameters.
*       copy2A21ScanHeader      Copies the PR 2A21 Scan Header to either the output
*                               buffer or the temporary buffer depending on the input
*                               parameters.
*       copy2A23ScanHeader      Copies the PR 2A23 Scan Header to either the output
*                               buffer or the temporary buffer depending on the input
*                               parameters.
*       copy2A25ScanHeader      Copies the PR 2A25 Scan Header to either the output
*                               buffer or the temporary buffer depending on the input
*                               parameters.
*       copy2B31ScanHeader      Copies the PR 2B31 Scan Header to either the output
*                               buffer or the temporary buffer depending on the input
*                               parameters.
*       TKreportWarning         This routine reports warning errors to the console and
*                               continues processing.
*
*******************************************************************************************/

int TKcopyScanHeader ( IO_HANDLE *sInGranuleHandle, void *sInDataBuffer,
		      IO_HANDLE *sOutGranuleHandle, void *sOutDataBuffer )
{

  /*  The following temporary variables are used to copy the scan header.  Each 
   *  strcture contains the four blocks (i.e., scan time, geolocation, scan status,
   *  and navigation).  They are broken out by instrument, since the data types of
   *  scan status and scan time are different.
   */
  TMI_COPY_HEADER tmiWorkingBuffer;
  VIRS_COPY_HEADER virsWorkingBuffer;
  PR_COPY_HEADER prWorkingBuffer;

  /*  Before attempting to copy the input scan header information to the output
   *  scan header, the file modes are checked to make sure that the input file is
   *  opened in read mode and the output file is opened in write mode.  If either,
   *  or both, of these conditions are not true, TKcopyScanHeader will return an
   *  error indicator.
   */
  if ( ( sInGranuleHandle->mode != TK_READ_ONLY ) || 
      ( sOutGranuleHandle->mode != TK_NEW_FILE ) )
    {
      TKreportWarning ( W_TK_BADMODECH );
      return ( W_TK_BADMODECH );
    }

  /*  The input product id is checked for the correct type.  Based on the type, 
   *  the correct version of the copy scan header routine will be called.
   */
  switch ( sInGranuleHandle->productID )
    {
    case TK_L1B_01 :  /*  Check for VIRS 1B01  */
      copy1B01ScanHeader ( sInDataBuffer, &virsWorkingBuffer, TK_VIRS_PIXELS,
			  COPY_FROM_INPUT );
      break;
    case TK_L1B_11 :  /*  Check for TMI 1B11  */
      copy1B11ScanHeader ( sInDataBuffer, &tmiWorkingBuffer, TK_TMI_PIXELS,
			  COPY_FROM_INPUT );
      break;
    case TK_L2A_12 :  /*  Check for TMI 2A12  */
      copy2A12ScanHeader ( sInDataBuffer, &tmiWorkingBuffer, TK_TMI_PIXELS,
			  COPY_FROM_INPUT );
      break;
    case TK_L1B_21 :  /*  Check for PR 1B21  */
      copy1B21ScanHeader ( sInDataBuffer, &prWorkingBuffer, TK_PR_PIXELS,
			  COPY_FROM_INPUT );
      break;
    case TK_L1C_21 :  /*  Check for PR 1C21  */
      copy1C21ScanHeader ( sInDataBuffer, &prWorkingBuffer, TK_PR_PIXELS,
			  COPY_FROM_INPUT );
      break;
    case TK_L2A_21 :  /*  Check for PR 2A21  */
      copy2A21ScanHeader ( sInDataBuffer, &prWorkingBuffer, TK_PR_PIXELS,
			  COPY_FROM_INPUT );
      break;
    case TK_L2A_23 :  /*  Check for PR 2A23  */
      copy2A23ScanHeader ( sInDataBuffer, &prWorkingBuffer, TK_PR_PIXELS,
			  COPY_FROM_INPUT );
      break;
    case TK_L2A_25 :  /*  Check for PR 2A25  */
      copy2A25ScanHeader ( sInDataBuffer, &prWorkingBuffer, TK_PR_PIXELS,
			  COPY_FROM_INPUT );
      break;
    case TK_L2B_31 :  /*  Check for PR 2B31  */
      copy2B31ScanHeader ( sInDataBuffer, &prWorkingBuffer, TK_PR_PIXELS,
			  COPY_FROM_INPUT );
      break;
    default : /* If the product id is not valid, then issue a warning  */
      TKreportWarning ( W_TK_BADIDCH );
      return ( W_TK_BADIDCH );
    }

  /*  The output product id is checked to be one of the correct output ids. 
   *  The appropriate copy routine is called based on the value of the 
   *  output id.
   */
  switch ( sOutGranuleHandle->productID )
    {
    case TK_L1B_01 :  /*  Check for VIRS 1B01  */
      copy1B01ScanHeader ( sOutDataBuffer, &virsWorkingBuffer, TK_VIRS_PIXELS,
			  COPY_TO_OUTPUT );
      break;
    case TK_L1B_11 :  /*  Check for TMI 1B11  */
      copy1B11ScanHeader ( sOutDataBuffer, &tmiWorkingBuffer, TK_TMI_PIXELS,
			  COPY_TO_OUTPUT );
      break;
    case TK_L2A_12 :  /*  Check for TMI 2A12  */
      copy2A12ScanHeader ( sOutDataBuffer, &tmiWorkingBuffer, TK_TMI_PIXELS,
			  COPY_TO_OUTPUT );
      break;
    case TK_L1B_21 :  /*  Check for PR 1B21  */
      copy1B21ScanHeader ( sOutDataBuffer, &prWorkingBuffer, TK_PR_PIXELS,
			  COPY_TO_OUTPUT );
      break;
    case TK_L1C_21 :  /*  Check for PR 1C21  */
      copy1C21ScanHeader ( sOutDataBuffer, &prWorkingBuffer, TK_PR_PIXELS,
			  COPY_TO_OUTPUT );
      break;
    case TK_L2A_21 :  /*  Check for PR 2A21  */
      copy2A21ScanHeader ( sOutDataBuffer, &prWorkingBuffer, TK_PR_PIXELS,
			  COPY_TO_OUTPUT );
      break;
    case TK_L2A_23 :  /*  Check for PR 2A23  */
      copy2A23ScanHeader ( sOutDataBuffer, &prWorkingBuffer, TK_PR_PIXELS,
			  COPY_TO_OUTPUT );
      break;
    case TK_L2A_25 :  /*  Check for PR 2A25  */
      copy2A25ScanHeader ( sOutDataBuffer, &prWorkingBuffer, TK_PR_PIXELS,
			  COPY_TO_OUTPUT );
      break;
    case TK_L2B_31 :  /*  Check for PR 2B31  */
      copy2B31ScanHeader ( sOutDataBuffer, &prWorkingBuffer, TK_PR_PIXELS,
			  COPY_TO_OUTPUT );
      break;
    default :  /* If the product id is not valid, then issue a warning  */
      TKreportWarning ( W_TK_BADIDCH );
      return ( W_TK_BADIDCH );
    }
  /*  If the scan header has been successfully copied, then return with a
   *  success indicator.
   */
  return TK_SUCCESS;
}


