/* DEVELOPMENT HISTORY */
/* Revision 1.24  1999/10/26 17:47:17  pkolbe
 * added and error message parameter to report Error
 *
 * Revision 1.23  1999/01/07 21:55:50  pkolbe
 * added 0-255 only exit code condition to reportException (TKreportError)
 *
 * Revision 1.22  1998/05/14 18:38:53  slywczak
 * Modified function reportWarn and reportException.  Changed reportWarn to
 * include a user defined message when one is provided.  In all three cases,
 * this message will be report to either OpenView or the user display.  Modified
 * reportException to use the new definition of reportWarn.
 *
 * Revision 1.21  1997/12/11 22:12:42  piper
 * Modified for new sendtrap calling sequence.
 *
 * Revision 1.20  1997/11/19 14:40:49  mack
 * Added thrid env.  One for developers in TSDIS.  It accesses the
 * TSDIS DB and writes to the screen.
 *
 * Revision 1.19  1997/09/15 14:14:47  piper
 * Modified priority categories:  any 'E' message is at least major.
 *
 * Revision 1.18  1997/09/12 15:36:24  piper
 * Changed Openview from displaying parent process id to group process id.
 *
 * Revision 1.17  1997/09/09 20:51:51  piper
 * Added parent process id to error messages.
 *
 * Revision 1.16  1997/05/30 13:08:20  piper
 * Modified to use new sendtrap/snmptrap and TSDIS_ERRORS.
 *
 * Revision 1.15  1997/04/17 22:02:21  piper
 * Modified getMessage to better handle anomalous conditions.
 *
 * Revision 1.14  1997/02/04 16:28:32  amak
 * Increased size of command in reportWarn and reportException.
 * Changed command array to be dynamically allocated in reportWarn and
 * reportException.
 *
 * Revision 1.13  1997/01/02 15:31:38  amak
 * Used TK_FNAME_LEN in reportWarn and reportException for hard-coded value.
 *
 * Revision 1.12  1996/12/24 14:31:31  amak
 * Incorporated walkthrough comments.
 * - Changed '<' and '>' to double quotes around the #include statements.
 *
 * Revision 1.11  1996/12/19 22:06:18  amak
 * Added two ifdef blocks for the OPERATION_ENV operation environment and
 * TSU_ENV TSU environment to handle error/warning situations in function
 * reportWarn() and reportException() respectively.
 * For OPERATION_ENV operation environment, -DOPERATION_ENV has to be used
 * when compiling.
 * For TSU_ENV TSU environment, -DTSU_ENV has to be used when compiling.
 *
 * Revision 1.10  1996/12/19 18:00:43  amak
 * Added comments for the use of -DTSDIS_TK_ENV in TSDIS environment.
 *
 * Revision 1.9  1996/12/19 16:39:50  amak
 * Added ifdef statements to reportException() and reportWarn().  The
 * use of the ifdef statements is distinguishing the method of error handling
 * in operation environment versus development environment. For the development 
 * environment within TSDIS, -DTSDIS_TK_ENV has to be used when compiling,
 * such that the error/warning message will show on System Monitoring OpenView.
 * Otherwise, error/warning message will show up in console.
 *
 * Revision 1.8  1996/12/03  16:58:11  amak
 * Fixed potential memory leak problem in reportException() and reportWarn()
 * routines by calling free().
 *
 * Revision 1.7  1996/11/01  20:46:14  amak
 * Added one more parameter to manageFiles because of the change of the API
 * of manageFiles.
 *
 * Revision 1.6  1996/10/17  19:43:28  amak
 * Modifed routines reportException() and reportWarn().
 * - Took out time stamp when sending warning/error messages to OpenView
 * - Formatted the time stamp when showing warning/error messages to console
 *
 * Revision 1.5  1996/10/07  18:08:30  amak
 * Made modifications to the file such that the warning/error message will
 * show up the corresponding mnemonic.
 * - Added parameter mnemonicString to getMessage() function.  Made
 *   corresponding modifications to use of getMessage() function in
 *   reportWarn() and reportException().
 *
 * Revision 1.4  1996/06/06  18:24:22  amak
 * Took out writing log file procedure.  This procedure has been replaced by
 * OpenView routine.
 *
 * Revision 1.3  1996/06/03  17:21:40  amak
 * Modified system calling to OpenView.
 *
 * Revision 1.2  1996/05/28  18:17:58  amak
 * Incorporated System Monitoring into Toolkit error handling routines.
 *
 * Revision 1.1  1996/04/19  15:32:30  amak
 * Adding this file to replace TKerrorRou.c and TKwarnRou.c.  This file is
 * now containing all the error handling routines for Toolkit.
 * */

/*********************************************************************
*
*  FILE NAME:	TKerrHandle.c
*
*  DESCRIPTION:
*	This file contains all the error handling routines for the
*	Toolkit.
*
*  ROUTINE CALLED:
*	Routine Name		Description
*	------------		-----------
*	getMsgFile()		This routine is used to locate the
*				ASCII message file.
*	extractMsgInfo()	This routine is used to extract information
*				from a line retrived from ASCII message
*				file and save it into code buffer.
*	getMessage()		This routine is used to search for the
*				specified code.  If found, the code 
*				information is stored in the output
*				variable.
*	reportWarn()		This routine is the warning routine that 
*				reports warnings to OpenView and the console.  
*				The warnings are reported with the line
*       			number, file name, the time and the date of 
*				the call to this routine.
*	reportException()	This is the error routine that reports errors 
*				to OpenView and the console, then terminate 
*				all the processes.  The errors are reported 
*				with the line number, file name, the time and 
*       			the date of the call to this routine.
*
**********************************************************************/

#include <math.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include "file_manager.h"
#include "TKerrHandle.h"

/**********************************************************************
*
*  FUNCTION STATEMENT:
*	getMsgFile(int msgNumber, char **filePath)
*
*  DESCRIPTION:
*	This routine is used to locate the ASCII message file. 
*
*  INPUT PARAMETERS:
*	Parameter Name		Description
*	--------------		-----------
*	msgNumber		The numeric value for error/warning
*				code generated by message compiler
*				(smfcompile).  
*
*  OUTPUT PARAMETERS:
*	Parameter Name		Description
*	--------------		-----------
*	filePath		The location of the ASCII message
*				file.
*
*  CALLED BY:
*	reportWarn()
*
*  ROUTINES CALLED:
*	None
*
***********************************************************************/
char getMsgFile(int msgNumber, char **filePath)
{
  char	 *path;
  int	 fileno;
  double tempValue, ipart;

  tempValue = msgNumber / 1000.;
  modf(tempValue, &ipart);
  fileno = (int)ipart;

#ifdef TSU_ENV
  path = getenv("TSDISTK");
  if (path != NULL)
  {
    sprintf(*filePath, "%s/include/%s_%d", path, TK_FILE_PREFIX, fileno);
#else
  path = getenv("TSDIS_ERRORS");
  if (path != NULL)
  {
    sprintf(*filePath,"%s/%s_%d", path, TK_FILE_PREFIX, fileno);
#endif
   }
  else
  {
    *filePath = NULL;
  }
}


/**********************************************************************
*
*  FUNCTION STATEMENT:
*       extractMsgInfo(TK_MSGINFO *msgInfo, char *line)
*
*  DESCRIPTION:
*	This routine is used to extract information from a line retrieved
*	from ASCII message file and store it into code buffer.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	line			Line string in the input buffer
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*      	msgInfo 		Structure that stored all the message
*				information.
*
*  CALLED BY:           
*       getMessage()
*
*  ROUTINES CALLED:             
*       None
*
***********************************************************************/

void extractMsgInfo(TK_MSGINFO *msgInfo, char *line)
{
  char  *token;                         /* string token */
  char  buf[TK_MAX_MSGBUF_SIZE];        /* temporary buffer */
  short len;                            /* length of the string */

  /**
   **   copy input line to temporary buffer because
   **   strtok does destructive editing
   **/
  strncpy(buf, line, TK_MAX_MSGBUF_SIZE);

  if (buf != (char *)NULL)
  {
    len = strlen(buf);
    if (buf[len-1] == '\n')
    {
        buf[len-1] = '\0';
    }

    /**
     **	Get code
     **/
    token = strtok(buf, ",");
    msgInfo->msgdata.code = atoi(token);

    /**
     ** Get mnemonic
     **/
    token = strtok((char *)NULL, ",");
    sprintf(msgInfo->msgdata.mnemonic, "%s", token);

    /**
     ** Get action mnemonic
     **/
    token = strtok((char *)NULL, ",");
    msgInfo->msgdata.action[0] = '\0';

    /**
     ** Get message
     **/
    token = strtok((char *)NULL, "\n");
    sprintf(msgInfo->msgdata.msg, "%s", token);
    len = strlen(msgInfo->msgdata.msg);
    msgInfo->msgdata.msg[len] = '\0';
  }
}


/***********************************************************************
*
*  FUNCTION STATEMENT:
*       getMesssage(int msgCode, char **filePath, char *msgString, 
		    char *mnemonicString)
*
*  DESCRIPTION:
*	This routine is used to search for the specified code.  If
*	found, the code information is stored in the output variable.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       msgCode			Numeric value for error/warning code
*				generated by message compiler (smfcompile).
*	filePath		The file path which shows the name and
*				the location of the ASCII message file.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       msgString		The message string which corresponds to 
*				the numeric value.
*	mnemonicString		The mnemonic of the corresponding message.
*
*  CALLED BY:           
*       reportWarn()
*
*  ROUTINES CALLED:             
*       None
*
**************************************************************************/

char getMessage(int msgCode, char **filePath, char *msgString, 
		char *mnemonicString)
{
  FILE	*msgFile;
  char	msgbuf[TK_MAX_MSGBUF_SIZE];
  TK_MSGINFO	msgInfo;

  msgFile = fopen(*filePath, "r\0");
  if (msgFile != NULL)
  {
    fgets(msgbuf, sizeof(msgbuf), msgFile);

    while(fgets(msgbuf, sizeof(msgbuf), msgFile) != (char *)NULL)
    {
	extractMsgInfo(&msgInfo, msgbuf);

	if (msgInfo.msgdata.code == msgCode)
	{
	  strcpy(mnemonicString, msgInfo.msgdata.mnemonic);
	  strcpy(msgString, msgInfo.msgdata.msg);
  	  fclose(msgFile);
	  break;
	} 
    }
    if (msgInfo.msgdata.code != msgCode)
    {
      strcpy(mnemonicString, "*** Mnemonic unavailable ***");
      sprintf(msgString, "cannot locate message # %d within file: %s.",
      msgCode,*filePath);
      fclose(msgFile);
     }
  }
  else
  {
    strcpy(mnemonicString, "*** Mnemonic unavailable ***");
    sprintf(msgString, "cannot open message file: %s for message # %d.",
    *filePath,msgCode);
  }
}


/**********************************************************************
*
*  FUNCTION STATEMENT:
*       reportWarn(int warnNumber,int linenumber, char *filename)
*
*  DESCRIPTION:
*	This routine is the warning routine that handled warning
*	differently among development environment, operation environment
*	and TSU environment.
*	For TSDIS_TK_ENV development environment, the routine will
*	report warning messages to the console.
*	For OPERATION_ENV operation environment, the routine will report
*	warning messages to System Monitoring OpenView.
*	For TSU_ENV TSU environment, the routine will report warning
*	messages to standard error where the messages will show up to
*	the console.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       number          	Numeric value for message code generated by
*				compiler (smfcompile).
*       linenumber              Line number of the call to TKreportError.
*       filename                Name of the file that calls TKreportError.
*
*  OUTPUT PARAMETERS:
*       None
*
*  CALLED BY:           
*       TKreportWarning()
*
*  ROUTINES CALLED:             
*	Routine Name		Description
*	------------		-----------
*       getMsgFile()		This routine is used to locate the
*				ASCII message file.
*	getMessage()		This routine is used to search for
*				the specified code.  If found, the code
*				information is stored in the output variable.
*	sendtrap		Sends message to OpenView.
*
************************************************************************/
void reportWarn(int number, char *message, int linenumber, char *filename)
{
  int	priority;
  char	msgString[TK_MAX_MSG_SIZE];
  char	mnemonicString[TK_MAX_MNEMONIC_SIZE];
  time_t theTime;
  pid_t gpid;
  char  command[TK_MAX_SIZE_COMMAND];
  char	filePath[TK_FNAME_LEN];
  char	*ptr = filePath;

  getMsgFile(number, &ptr);
  if (ptr != NULL)
  {
    getMessage(number, &ptr, msgString, mnemonicString);
  }
  else
  {
    strcpy(mnemonicString, "*** Mnemonic unavailable ***");
#ifdef TSU_ENV
    strcpy(msgString, "environment variable 'TSDISTK' is not defined.");
#else
    strcpy(msgString, "environment variable 'TSDIS_ERRORS' is not defined.");
#endif
  }

  time(&theTime);
  gpid = getpgrp();

/* Determine the priority to be assigned to the message. */

  if (strncmp(mnemonicString, "E_IM", 4) == 0 || 
      strncmp(mnemonicString, "E_DM", 4) == 0)
		priority = 0; /* CRITICAL */
  else if (strncmp(mnemonicString, "E", 1) == 0)
		priority = 1; /* MAJOR */
  else if (strncmp(mnemonicString, "W_DM", 4) == 0 || 
	   strncmp(mnemonicString, "W_IM", 4) == 0)
		priority = 2; /* MINOR */
  else if (strncmp(mnemonicString, "W", 1) == 0 &&
           strcmp(mnemonicString,"W_SCH_INFO") != 0)
		priority = 3; /* WARNING */
  else		priority = 4; /* NORMAL */

#ifdef TSDIS_TK_ENV
  /*
  ** Sending the message to the screen (stderr).  
  ** This is only operated in TSDIS environment where -DTSDIS_TK_ENV
  ** has to be used when compiling.
  */

  /* print the warning message to the screen. */
  if ( message == NULL )
    fprintf(stderr, "%d: %s - %s [%s Line %d %-24.24s]\n", gpid,
	    mnemonicString, msgString, filename, linenumber, ctime(&theTime));
  else
    fprintf(stderr, "%d: %s - %s (%s) [%s Line %d %-24.24s]\n", gpid,
	    mnemonicString, msgString, message, filename, linenumber, 
	    ctime(&theTime));
#endif


#ifdef OPERATION_ENV
  /*
  ** Sending warning message to System Monitoring OpenView.
  ** This is only operated in Operation environment where -DOPERATION_ENV
  ** has to be used when compiling.
  */
/* Don't include gpid in command for sendtrap case; added in sendtrap */
  if ( message == NULL )
    sprintf(command, "%s - %s [%s Line %d]", mnemonicString, 
	    msgString, filename, linenumber);
  else
    sprintf(command, "%s - %s %s [%s Line %d]", mnemonicString, 
	    msgString, message, filename, linenumber);

  sendtrap(gpid, priority, command);
#endif


#ifdef TSU_ENV
  /*
  ** Sending warning message to the console.
  ** This part is only operated in TSU environment where -DTSU_ENV
  ** has to be use when compiling.
  */
  if ( message == NULL )
    fprintf(stderr, "%d: %s - %s [%s Line %d %-24.24s]\n", gpid,
	    mnemonicString, msgString, filename, linenumber, ctime(&theTime));
  else
    fprintf(stderr, "%d: %s - %s (%s) [%s Line %d %-24.24s]\n", gpid,
	    mnemonicString, msgString, message, filename, linenumber, 
	    ctime(&theTime));
#endif

}

/***************************************************************************
*
*  FUNCTION STATEMENT:
*       reportException(int exceptionNumber,int linenumber, char *filename)
*
*  DESCRIPTION:
*       This routine is the error handling routine that handled errors
*       differently among development environment, operation environment
*       and TSU environment.
*       For TSDIS_TK_ENV development environment, the routine will
*       report error messages to the console, then it will terminate 
*       all the processes.
*       For OPERATION_ENV operation environment, the routine will report
*       error messages to System Monitoring OpenView, then it will
*	terminate all the processes.
*       For TSU_ENV TSU environment, the routine will report error
*       messages to standard error where the messages will show up to
*       the console, then it will terminate all the processes.

*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       exceptionNumber         Numeric value for error code generated by
*				message compiler (smfcompile).
*       linenumber              Line number of the call to TKreportError.
*       filename                Name of the file that calls TKreportError.
*
*  OUTPUT PARAMETERS:
*	None
*
*  CALLED BY:           
*       TKreportError()
*
*  ROUTINES CALLED:             
*	Routine Name		Description
*	------------		-----------
*	reportWarning		Sends message to screen or
OpenView.
*	manageFiles()		A routine that closes all files and terminate
*				the process.
*
**************************************************************************/
void reportException(int exceptionNumber, char *message, int linenumber, char *filename)
{
  reportWarn(exceptionNumber, message, linenumber, filename);
  manageFiles(0, 0, DELETE_ALL_FILES);
  if ((exceptionNumber>255)||(exceptionNumber<0))
      exceptionNumber=TK_FAIL;
  exit(exceptionNumber);
}

