/* DEVELOPMENT HISTORY */
/* Revision 1.9  1996/12/24 14:31:30  amak
 * Incorporated walkthrough comments.
 * - Changed '<' and '>' to double quotes around the #include statements.
 *
 * Revision 1.8  1996/04/19  15:40:47  amak
 * Made modifications to the preprocessor #include statement.  Replaced
 * TKerrorRou.h and TKwarnRou.h to TKerrHandle.h and TS_TK_15.h for new
 * error handling calls.
 *
 * Revision 1.7  1995/12/21  14:16:25  dbazell
 * Changed upper limit for longitude to 360 from 180.  Valid longitudes
 * are now -180 to +360.  Per comments from Toshio Iguchi.
 *
 * Revision 1.6  1995/12/15 16:56:47  amak
 * Took out system macro SEEK_SET for function fseek(), and replaced that
 * macro to 0 for moving the file pointer offset bytes from the beginning
 * of the file.  This change will make the code compatible to the other
 * platforms.
 *
 * Revision 1.5  1995/10/18  20:52:24  amak
 * Made modifications after some files need to be renamed.
 *
 * Revision 1.4  1995/10/17  20:00:34  amak
 * File modification.
 *
 * Revision 1.3  1995/09/18  14:21:44  thyun
 * Corrected a syntax error, inserted a few error checking statement, and
 * modified the function to read only the necessary part of data.
 *
 * Revision 1.2  1995/09/12  15:16:04  amak
 * Added more comments and took out the ifndef statement.
 *
 * Revision 1.1  1995/09/06  14:11:48  dbazell
 * Adding first version of readtopo
 *
 */
/************************************************************************************
*
* FILE NAME:       
*      TKreadTopo.c
*
* DESCRIPTION:     
*      This file reads the topgraphic database called etopo5.dat
*      taking latitude and longitude as input, returning the
*      elevation or bathymetry in meters for that point.
*
* INPUTS:
*      Parameter Name         Description
*      --------------         -----------
*      lat                    latitude of grid point
*      lon                    longitude of grid point
*
* OUTPUTS:
*      None.
*
* RETURN VALUES:
*      Elevation in meters for points over land; bathymetry in meters
*      for points below sea level.
*
* ROUTINES CALLED:
*	Routine Name		Description
*	------------		-----------
*       TKreportError()         This routine is used to process fatal errors.
*                               Calling this routine will close all open files,
*                               send an error message to a central log file, and
*                               terminate processing. 
*
*       TKreportWarning()       This routine is used for warnings and informational
*                               messages.  Calling this routine will send an error
*                               message to a central log file and return control to
*                               the calling routine.
*
************************************************************************************/
#include <etopo5.h>
#include "TKerrHandle.h"
#include "TS_TK_15.h"

short int TKreadTopo(float lat, float lon)
{
   short etopo, val;
   int   startx, starty, index;
   char  *path, fullpath[TK_PATHLEN];
   FILE  *fd;

   /* Check for out of bounds error for input lat and lon */
   if ( (lon<-180.0) || (lon>360.0) )
     TKreportWarning(W_TK_PBNDS);	/* Parameter out of bounds. */

   if ( (lat<-90.0) || (lat>90.0) )
     TKreportWarning(W_TK_PBNDS);	/* Parameter out of bounds. */

   if (lon < 0)
       lon = lon + 360.0;

   /* Get the value of the TSDISTK environment variable and prepend 
    * it to the file name.
    */
   path = getenv( "TSDISTK" );
   if (path == NULL)
     TKreportError(E_TK_NULLPATH);	/* getenv returned a null path */

   strcpy(fullpath, path);
   strcat(fullpath, TK_E5_ETOPOFILE);

   /* Open the file */
   if ((fd=fopen(fullpath,"r"))==NULL)
     TKreportError(E_TK_OPENFAIL);	/* fopen failed */
  
   /* Calculate the offset into the array */
   startx = (int)((lon - TK_E5_LONMIN)/TK_E5_DLON + 0.5);

   starty = (int)((TK_E5_LATMAX - lat)/TK_E5_DLAT + 0.5);
   index = starty * TK_E5_COLS + startx;

   /* search the position to read data, 0 means successful */
   if (fseek(fd, (index*sizeof(short)), 0) != 0)
       TKreportError(E_TK_SEEKFAIL); /* fseek failed */

   /* read the file */ 
   if (fread(&etopo, sizeof(short), 1, fd) <= 0)
       TKreportError(E_TK_READFAIL) ; /* fread failed */

   fclose(fd);

   /* extract the point */
   val = etopo;

   return(val);
 }
