/* DEVELOPMENT HISTORY
 * Revision 1.13  1999/10/19 14:41:37  pkolbe
 * removed one line of code: lon += 360;  (for the case of lon < 0 )
 * because negative longitude results were in error (DR 726)
 *
 * Revision 1.12  1996/12/24 14:31:30  amak
 * Incorporated walkthrough comments.
 * - Changed '<' and '>' to double quotes around the #include statements.
 *
 * Revision 1.11  1996/07/08  16:27:36  pshah
 * Combined the two if statements into ine if then else statement
 *
 * Revision 1.10  1996/07/08  16:17:39  pshah
 * TKreadlsm.c
 *
 * Revision 1.9  1996/04/19  15:40:48  amak
 * Made modifications to the preprocessor #include statement.  Replaced
 * TKerrorRou.h and TKwarnRou.h to TKerrHandle.h and TS_TK_15.h for new
 * error handling calls.
 *
 * Revision 1.8  1995/12/21  14:14:46  dbazell
 * Changed lower limit allowed for longitue to -180 from 3360.  Add 360
 * to longitude if it is negative.  Per comments from Toshio Iguchi.
 *
 * Revision 1.7  1995/10/18 20:52:24  amak
 * Made modifications after some files need to be renamed.
 *
 * Revision 1.6  1995/10/17  20:00:35  amak
 * File modification.
 *
 * Revision 1.5  1995/09/18  14:22:42  thyun
 * Modified the program to read a input file only once and inserted a few error
 * condition checking statements.
 *
 * Revision 1.4  1995/09/12  15:16:05  amak
 * Added more comments and took out the ifndef statement.
 *
 * Revision 1.3  1995/08/31  14:52:18  dbazell
 * corrected path for data file
 *
 * Revision 1.2  1995/08/29 22:01:52  dbazell
 * Added environment variable in file path
 *
 * Revision 1.1  1995/08/23 14:26:47  dbazell
 * Initial version
 *
 */

/************************************************************************************
*
* FILE NAME:       
*	TKreadlsm.c
*
* DESCRIPTION:     
*	This file reads the land-sea mask called dbglobe93.grd taking latitude and 
*	longitude as input, returning the code for the geography at that point.  
*	IMPORTANT:  Please note that this routine takes Upper left corner as 
*	(-90, 0) and Lower right corner as (90,360).
*
* INPUTS:
*	Parameter Name         Description
*       --------------         -----------
*       lat                    latitude of grid point (90,-90)
*       lon                    longitude of grid point (0, 360)
*
* OUTPUTS:
*       None.
*
* RETURN VALUES:
*       TK_LAND   0 = land
*       TK_ICE    4 = possible (pack) ice
*       TK_OCEAN  5 = ocean
*       TK_COAST  6 = coast
*       TK_CICE   7 = coast next to ice
*       1,2,3 are never returned, for historical reasons.
*      
*  
* ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       TKreportError()         This routine is used to process fatal errors.
*                               Calling this routine will close all open files,
*                               send an error message to a central log file, and
*                               terminate processing.
*
*************************************************************************************/
#include <stdio.h>
#include "landsea.h"
#include "TKerrHandle.h"
#include "TS_TK_15.h"


int TKreadlsm(float lat, float lon)
{
  static char   sfmask[TK_LS_ARRAY];
  static FILE   *fd = NULL;
  char          *path;
  char          fullpath[TK_LS_PATHLEN];
  int           ilat, ilon, index;

  /* If this is the first time reading the file, */
  if (fd == NULL)
  {
      /* Get the value of the TSDISTK environment variable and prepend 
       * it to the file name.
       */
      path = getenv( "TSDISTK" );
      if (path == NULL)
          TKreportError(E_TK_NULLPATH);	/* getenv returned a null path */

      strcpy( fullpath, path);
      strcat( fullpath, "/data/dbglobe93.grd");

      /* Open and read landsea mask (size is 1036800).  Entire landsea mask is 
       * read into memory.  IMPORTANT: Upper left corner is (-90.0) and lower 
       * right corner is (90,360).
       */
      if ((fd=fopen(fullpath,"r"))==NULL)
           TKreportError(E_TK_OPENFAIL);  /* fopen failed */
 
      if (fread( sfmask, TK_LS_FLDS, 4, fd) <= 0)
          TKreportError(E_TK_READFAIL) ; /* fread failed */
      fclose(fd);
  } 

  /* Check for out of bounds error for input lat and lon */
  if ( (lon < -180.0) || (lon > 360.0) )
     TKreportWarning(W_TK_PBNDS);       /* Parameter out of bounds. */

  if ( (lat<-90.0) || (lat>90.0) )
     TKreportWarning(W_TK_PBNDS);       /* Parameter out of bounds. */

  if (lon < 0)                          /* Allow for negative longitudes */
  {
    ilon = 4 * (lon + 359.875) + 0.5;
  }

  /* Calculate the other offset into the file, given the longitude.
   */

  else
  {
    ilon = 4 * (lon - 0.125) + 0.5;
  }

  /* Calculate the offset into the file given the latitude.  Original
   * code had nint(4*(lat+89.875)) to round to nearest integer.  Adding
   * 0.5 and taking the integer part accomplishes the same task.
   */

  ilat = 4*(lat + 89.875) + 0.5;

  if (ilat < 0) 
      ilat = 0;

  if (ilon < 0)
      ilon = 0;

  if (ilat > 719)
      ilat = 719;

  if (ilon > 1439)
      ilon = 1439;
 
  /* Calculate offset into array.  Check for out of bounds */
  index = ilat * 1440 + ilon;
  if (index < 0 || index >= TK_LS_ARRAY)
    TKreportWarning(W_TK_LSBNDS);	/* Index out of bounds in Land Sea array */

  /* Return the land sea value */
  return((int) sfmask[index]);
}
