
/* --- begin cvs log messages --- */

/* Revision 1.4  1999/08/13 17:31:53  pkolbe
 * added dynamic scaling code
 *
 * Revision 1.3  1997/11/13 15:31:38  slywczak
 * Modified the type def from int32 to int for TKpickL1GVvos
 *
 * Revision 1.2  1997/10/16 09:04:48  okelley
 * 	Printfs removed.  Misleading warning removed.
 * */

/* --- end cvs log messages --- */

/*
****************************** begin file header *************************

  FILE NAME:	TKutilGV.c

  DESCRIPTION:

	This file contains high-level API's for reading
	level 1 Ground Validation (GV) data.  These API's 
	call lower-level API's in IO.c.

	This file also contains internal Toolkit functions used 
	by API's in this file.

  MODIFICATIONS:

	10/14/97	O. Kelley		Implimented

	10/15/97	O. Kelley		removed printf

	10/15/97	O. Kelley		removed unnecessary warning

	  Removed warning W_TK_GV_V_NOT_NULL from function 
	  TKreadL1GV_error_check because it is not needed.

	 
  FUNCTIONS IN THIS FILE:

    Functions specific to FORTRAN

        TKreadL1GV_1byte_in_FORTRAN     internal
        TKreadL1GV_2byte_in_FORTRAN     internal
        TKreadL1GVdate_in_FORTRAN       internal

        print_in_C                      testing
        print_in_FOR                    testing
        print_in_FOR_1byte              testing

    Functions for reading volume scans in C

        TKreadL1GVbyVosNum              API
        TKreadL1GVparam                 API
        TKreadL1GVbaseSweep                 API

        TKreadL1GV_error_check          internal
        TKreadL1GV_driver               internal

    Functions for reading elements of volume scans in C

        TKreadL1GVdate          	API
        TKpickL1GVvosNum        	API

        TKreadL1GVsummary       	API
        TKprintL1GVsummary      	API

    Functions to allocate and free volume scans

        TKallocateL1GV                  internal
        TK_alloc_a_param                internal
        TK_alloc_a_1byte_param          internal
        TK_alloc_a_2byte_param          internal

        TKfreeL1GV                      API
        TK_dealloc_a_1byte_param        internal
        TK_dealloc_a_2byte_param        internal

    Miscellaneous internal Toolkit utilities

        TKresetL1GV                     internal
        TKreadL1GVsize                  internal
        TK_read_1c51_1b51_summary       internal

    Functions that copy data from C to FORTRAN structures

        TK_copy_2byte_metadata_to_FORTRAN       internal
        TK_copy_2byte_to_FORTRAN                internal
        TK_copy_1byte_metadata_to_FORTRAN       internal
        TK_copy_1byte_to_FORTRAN                internal

        TK_copy_dates_to_FORTRAN                internal

    Low level functions to read volume scan data

        TK_read_1c51_1b51_in_C          internal

        copyGV_L1_vosDes                internal
        copyGV_L1_radarDes              internal
        copyGV_L1_corrFactorDes         internal
        copyGV_L1_sweepInfo             internal
        copyGV_L1_paramDesc             internal

        readGV_L1B_1C_metadata          internal
        readGV_L1B_1C_param             internal
        readGV_L1B_1C_subset            internal

        TKreadL1GVparamPos              internal
        TKreadL1GVparamName             internal
    
****************************** end file header *************************
*/


/*
------------------------------------------
        include files
------------------------------------------
*/
	/* The IO_GV.h header files must be explicitly included
	by the source file TKutilGV.c. But IO_INTR_GV.h is not needed
	because it contains internal toolkit functions not used by
	the functions in TKutilGV.h. */

        #include "IO_GV.h"

	/* 
	-------------------------------------------------------
	The following include files are needed but are included
	else where, so they do not have to be explicitly included here.

	needed header		where it is included

	  "IO.h"			IO_GV.h
	  <math.h>			IO.h

	  <stdio.h>			hdfi.h from hdf.h from IO.h
	  <stdlib.h>			same as above
	  <string.h>			same as above
	  <unistd.h>			same as above

	--------------------------------------------------------
	*/

	/*
	--------------------------------------------------------
	The following include files are needed when compiling TKutilGV.c
	to call internal toolkit functions inside hdf_routines.c.
	The source file hdf_routines.c also includes these headerfiles. 

	  mfhdf.h     hdf_routines.h  TKerrHandle.h	TS_TK_15.h 

	None of these header files are need when compiling programs
	that use the object file created from TKutilGV.c.
	--------------------------------------------------------
	*/

	/* contains low level HDF routines*/
	#include "hdf.h"
        #include "mfhdf.h"

        /* contains toolkit wrappers for low level HDF routines */
        #include "hdf_routines.h"

        #include "TKerrHandle.h"
        #include "TS_TK_15.h"

	/* The reason for including this header file is unknown */

/*
	============================================================
	============================================================

			Functions specific to FORTRAN

	============================================================

	name				type

	TKreadL1GV_1byte_in_FORTRAN	internal
	TKreadL1GV_2byte_in_FORTRAN	internal
	TKreadL1GVdate_in_FORTRAN	internal
	
	print_in_C			testing
	print_in_FOR			testing
	print_in_FOR_1byte		testing

	============================================================
	============================================================
*/


/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TKreadL1GV_1byte_in_FORTRAN( IO_HANDLE * handle ,
  L1_GV_1BYTE *vos , int VosNum , char param_name[ 20 ],
  int read_only_bottom_sweep, VosSizeSTRUCT *VosSize )

DESCRIPTION:

	This function reads a one byte parameter for a FORTRAN
	program.  Depending on the value of read_only_bottom_sweep,
	either the whole parameter, or just the bottom sweep
	of the parameter is read.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle			IO handle for a granule that has been previously
			opened with TKopen.

VosNum			the number of the volume scan

param_name		the name of the parameter

read_only_bottom_sweep	a logical variable that equals TK_TRUE if
			only the bottom sweep is to be read from
			the given parameter, and equals TK_FALSE otherwise.

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
vos			a level 1 GV volume scan

VosSize			a structure that describes the size of a 
			level 1 GV volume scan

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
status			an error message passed from a subfunction
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
TKreadL1GV_driver		read the volume scan into the C structure

TK_copy_2byte_to_FORTRAN	copy the volume scan into the FORTRAN
				structure
 
MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
int TKreadL1GV_1byte_in_FORTRAN( IO_HANDLE * handle ,
  L1_GV_1BYTE *vos , int VosNum , char param_name[ 20 ],
  int read_only_bottom_sweep, VosSizeSTRUCT *VosSize ) 
{
	int		status ;
	L1B_1C_GV	*vos_in_c ;
 
        status          = TKreadL1GV_driver( handle, & vos_in_c, VosNum,
          param_name, read_only_bottom_sweep, VosSize ) ;

	if ( status == TK_SUCCESS )

	  status	= TK_copy_1byte_to_FORTRAN( vos_in_c , vos , VosSize);

/*
	print_in_C(   vos_in_c ) ;
	print_in_FOR_1byte( vos      ) ;
*/

	TKfreeL1GV( & vos_in_c ) ;

	return( status ) ;
}


/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TKreadL1GV_2byte_in_FORTRAN( IO_HANDLE * handle ,
  L1_GV_2BYTE *vos , int VosNum , char param_name[ 20 ],
  int read_only_bottom_sweep, VosSizeSTRUCT *VosSize )

DESCRIPTION:

	This function reads a two byte parameter for a FORTRAN
	program.  Depending on the value of read_only_bottom_sweep,
	either the whole parameter, or just the bottom sweep
	of the parameter is read.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle			IO handle for a granule that has been previously
			opened with TKopen.

VosNum			the number of the volume scan

param_name		the name of the parameter

read_only_bottom_sweep	a logical variable that equals TK_TRUE if
			only the bottom sweep is to be read from
			the given parameter, and equals TK_FALSE otherwise.

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
vos			a level 1 GV volume scan

VosSize			a structure that describes the size of a 
			level 1 GV volume scan

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
status			an error message passed from a subfunction
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
TKreadL1GV_driver		read the volume scan into the C structure

TK_copy_2byte_to_FORTRAN	copy the volume scan into the FORTRAN
				structure

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
int TKreadL1GV_2byte_in_FORTRAN( IO_HANDLE * handle ,
  L1_GV_2BYTE *vos , int VosNum , char param_name[ 20 ],
  int read_only_bottom_sweep, VosSizeSTRUCT *VosSize ) 
{
	int		status ;
	L1B_1C_GV	*vos_in_c ;
 
        status          = TKreadL1GV_driver( handle, & vos_in_c, VosNum,
          param_name, read_only_bottom_sweep, VosSize ) ;

	if ( status == TK_SUCCESS )

	  status	= TK_copy_2byte_to_FORTRAN( vos_in_c , vos , VosSize);

/*
	print_in_C(   vos_in_c ) ;
	print_in_FOR( vos      ) ;
*/

	TKfreeL1GV( & vos_in_c ) ;

	return( status ) ;
}


/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TKreadL1GVdate_in_FORTRAN( IO_HANDLE *sGranuleHandle,
        DATE_TIME_ARRAY_STR * date_array, int * num_date )

DESCRIPTION:

	This function reads the start date/times of all the volume
	scans in a granule for a FORTRAN program.

	The C version of this function returns a struct tm structure
	for which C has many functions to manipulate.  FORTRAN
	does not have functions to manipulate the struct tm structure,
	and the struct tm structure has a counter intuitive treatment
	of months and days of months.  For these reasons, this function
	copies the date into a DATE_TIME_ARRAY_STR structure which
	has an easier to understand format.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle			IO handle for a granule that has been previously
			opened with TKopen.

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
date_array		an array of dates for use in FORTRAN

num_date		the number of dates read from the granule,
			this is also equal to the number of volume scans
			in the granule.

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
status			an error message passed from a subfunction
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
TKreadL1GVdate			read the date array into the C structure

TK_copy_dates_to_FORTRAN	copy the date array into the FORTRAN
				structure
 
MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
int TKreadL1GVdate_in_FORTRAN( IO_HANDLE *sGranuleHandle,
	DATE_TIME_ARRAY_STR * date_array, int * num_date )
{
	int		status ;
	struct tm 	date_in_C[ MAX_VOS ] ;
	
	status	= TKreadL1GVdate( sGranuleHandle , date_in_C, num_date ) ;

	if ( status == TK_SUCCESS ) 

	  TK_copy_dates_to_FORTRAN( date_array, date_in_C, * num_date ) ;

	return( status ) ;
}

/*
-----------------------------------------------------------------
	print_in_C		for testing only
-----------------------------------------------------------------
*/

void print_in_C( L1B_1C_GV       *vos )
{
	int	sweep, ray, cell ;

	printf( " \n\t\t  C \n" ) ;
/*
        for ( sweep = 0 ; sweep <  10 ; sweep = sweep + 2)
	{
          for ( ray   = 0 ; ray   < 201 ; ray = ray + 50 )
	    printf( " %f ", vos->sensor.rayInfoFloat[ sweep ][ ray ][ 0 ] ) ;
	
	  printf( "\n" ) ;
	}
*/
        for ( sweep = 0 ; sweep <   1 ; sweep ++ )
        for ( ray   = 0 ; ray   < 201 ; ray = ray + 50 )
	{

        for ( cell  = 0 ; cell  < 201 ; cell = cell + 33 )
	{
          if (  vos->sensor.parm[0]->parmData2byte != NULL )

	    printf( " %8d " ,
              vos->sensor.parm[0]->parmData2byte[sweep][ray][cell] ) ;

          else if ( vos->sensor.parm[0]->parmData1byte != NULL )

	   printf( " %8d " ,
              vos->sensor.parm[0]->parmData1byte[sweep][ray][cell] ) ;
	}

	printf( " \n" ) ;

	}
}

/*
-----------------------------------------------------------------
	print_in_FOR		for testing only
-----------------------------------------------------------------
*/

void print_in_FOR( L1_GV_2BYTE       *vos )
{
	int	sweep, ray, cell ;

	printf( " \n\t\tFORTRAN \n" ) ;
/*
        for ( sweep = 0 ; sweep <  10 ; sweep = sweep + 2)
	{
          for ( ray   = 0 ; ray   < 201 ; ray = ray + 50 )
	    printf( " %f ", vos->sensor.rayInfoFloat[ sweep ][ ray ][ 0 ] ) ;
	
	  printf( "\n" ) ;
	}
*/
        for ( sweep = 0 ; sweep <   1 ; sweep ++ )
        for ( ray   = 0 ; ray   < 201 ; ray = ray + 50 )
	{

          for ( cell  = 0 ; cell  < 201 ; cell = cell + 33 )
	    printf( " %8d " , vos->sensor.parm.data[sweep][ray][cell] ) ;

	  printf( " \n" ) ;

	}
}

/*
-----------------------------------------------------------------
        print_in_FOR_1byte            for testing only
-----------------------------------------------------------------
*/

void print_in_FOR_1byte( L1_GV_1BYTE       *vos )
{
	int	sweep, ray, cell ;

	printf( " \n\t\tFORTRAN \n" ) ;
/*
        for ( sweep = 0 ; sweep <  10 ; sweep = sweep + 2)
	{
          for ( ray   = 0 ; ray   < 201 ; ray = ray + 50 )
	    printf( " %f ", vos->sensor.rayInfoFloat[ sweep ][ ray ][ 0 ] ) ;
	
	  printf( "\n" ) ;
	}
*/
        for ( sweep = 0 ; sweep <   1 ; sweep ++ )
        for ( ray   = 0 ; ray   < 201 ; ray = ray + 50 )
	{

          for ( cell  = 0 ; cell  < 201 ; cell = cell + 33 )
	    printf( " %8d " , vos->sensor.parm.data[sweep][ray][cell] ) ;

	  printf( " \n" ) ;

	}
}

/*
	============================================================
	============================================================

		Functions for reading volume scans in C

        ============================================================

	name				type

	TKreadL1GVbyVosNum		API
	TKreadL1GVparam			API
	TKreadL1GVbaseSweep			API

	TKreadL1GV_error_check		internal
	TKreadL1GV_driver		internal

	============================================================
	============================================================
*/

/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TKreadL1GVbyVosNum( IO_HANDLE * handle ,
  L1B_1C_GV ** vos , int VosNum, VosSizeSTRUCT *VosSize )

DESCRIPTION:

	This function reads all the parameters in the volume scan
	select by the user.  The user selects the volume scan
	based on its number.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle			IO handle for a granule that has been previously
			opened with TKopen.

VosNum			the number of the volume scan

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
vos			a level 1 GV volume scan

VosSize			a structure that describes the size of a 
			level 1 GV volume scan

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
status			an error message passed from a subfunction
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
TKreadL1GV_driver		The driver function for reading 
				level 1 GV volume scans.
 
MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
int TKreadL1GVbyVosNum( IO_HANDLE * handle , 
  L1B_1C_GV ** vos , int VosNum, VosSizeSTRUCT *VosSize )
{
	int		status ;
	int		read_only_bottom_sweep = TK_FALSE ;
        char            param_name[ 20 ] = TK_READ_ALL_PARAM ;

	status		= TKreadL1GV_driver( handle, vos, VosNum,
	  param_name, read_only_bottom_sweep, VosSize ) ;

	return( status ) ;
}

/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TKreadL1GVparam( IO_HANDLE * handle ,
  L1B_1C_GV ** vos , int VosNum, char param_name[ 20 ],
  VosSizeSTRUCT *VosSize )

DESCRIPTION:

	This function reads one of the parameters in the volume scan
	select by the user.  The user selects the volume scan
	based on its number, and the parameter based on its name.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle			IO handle for a granule that has been previously
			opened with TKopen.

VosNum			the number of the volume scan

param_name              the name of the parameter

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
vos			a level 1 GV volume scan

VosSize			a structure that describes the size of a 
			level 1 GV volume scan

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
status			an error message passed from a subfunction
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
TKreadL1GV_driver               The driver function for reading
                                level 1 GV volume scans.

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
int TKreadL1GVparam( IO_HANDLE * handle , 
  L1B_1C_GV ** vos , int VosNum , char param_name[ 20 ],
  VosSizeSTRUCT *VosSize )
{
	int		status ;
	int		read_only_bottom_sweep = TK_FALSE ;

	status		= TKreadL1GV_driver( handle, vos, VosNum,
	  param_name, read_only_bottom_sweep, VosSize ) ;

	return( status ) ;
}

/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TKreadL1GVbaseSweep( IO_HANDLE * handle ,
  L1B_1C_GV ** vos , int VosNum, char param_name[ 20 ],
  VosSizeSTRUCT *VosSize )

DESCRIPTION:

	This function reads the bottom sweep of one the parameters 
	in the volume scan select by the user.  The user selects the
	volume scan based on its number, and the parameter based on its name.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle			IO handle for a granule that has been previously
			opened with TKopen.

VosNum			the number of the volume scan

param_name              the name of the parameter

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
vos			a level 1 GV volume scan

VosSize			a structure that describes the size of a 
			level 1 GV volume scan

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
status			an error message passed from a subfunction
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
TKreadL1GV_driver               The driver function for reading
                                level 1 GV volume scans.

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
int TKreadL1GVbaseSweep( IO_HANDLE * handle , 
  L1B_1C_GV ** vos , int VosNum , char param_name[ 20 ],
  VosSizeSTRUCT *VosSize )
{
	int		status ;
	int		read_only_bottom_sweep = TK_TRUE ;

	status		= TKreadL1GV_driver( handle, vos, VosNum,
	  param_name, read_only_bottom_sweep, VosSize ) ;

	return( status ) ;
}

/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TKreadL1GV_error_check( IO_HANDLE * handle ,
  L1B_1C_GV ** vos , int VosNum , char param_name[ 20 ],
  int read_only_bottom_sweep, VosSizeSTRUCT *VosSize )

DESCRIPTION:

	To check the inputs of the function TKreadL1GV_driver.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle			IO handle for a granule that has been previously
			opened with TKopen.

VosNum			the number of the volume scan

param_name              the name of the parameter, set to the macro
			TK_READ_ALL_PARAM if all parameters are to be
			read.

read_only_bottom_sweep  a logical variable that equals TK_TRUE if
                        only the bottom sweep is to be read from
                        the given parameter, and equals TK_FALSE otherwise.

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
none

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
E_TK_GV_USE_TKOPEN
E_TK_GV_V_EMPTY
E_TK_GV_L1_PROD
E_TK_GV_VOS_NUM
E_TK_GV_NAME_LEN
E_TK_GV_B_SWEEP
E_TK_GV_ALLOC_VS
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
TKreportWarning                 to warn operator that an error has occurred

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 

int TKreadL1GV_error_check( IO_HANDLE * handle ,
  L1B_1C_GV ** vos , int VosNum , char param_name[ 20 ],
  int read_only_bottom_sweep, VosSizeSTRUCT *VosSize )
{
	/* ---------------------------
		check for errors
	   --------------------------- */

	/* check to see that the granule has been openned with TKopen. */

	if ( handle->cfgSTR == NULL )
	{
	  TKreportWarning( E_TK_GV_USE_TKOPEN ) ;
	  return( 	   E_TK_GV_USE_TKOPEN ) ;
	}

	/* check that the granule is not an empty granule */

	if ( handle->cfgSTR->vGroup->subVgroup == NULL )
	{
	  TKreportWarning( E_TK_GV_V_EMPTY ) ;
	  return( 	   E_TK_GV_V_EMPTY ) ;
	}

	/* check that the granule is a level 1 GV granule */

	if ( ( handle->productID != TK_L1C_GV ) &&
	     ( handle->productID != TK_L1B_GV )  )
	{
	  TKreportWarning( E_TK_GV_L1_PROD ) ;
	  return(	   E_TK_GV_L1_PROD ) ;
	}
	
	/* check that the user supplied vos number is reasonable */

	if ( ( VosNum < 0 ) || ( VosNum >= MAX_VOS ) )
	{
	  TKreportWarning( E_TK_GV_VOS_NUM ) ;
	  return( 	   E_TK_GV_VOS_NUM ) ;
	}

	/* check that the user supplied parameter name is reasonable */

	if ( param_name[0] == '\0' )
	{
	  TKreportWarning( E_TK_GV_NAME_LEN ) ;
	  return( 	   E_TK_GV_NAME_LEN ) ;
	}

	/* check read_only_bottom_sweep */

	if ( ( read_only_bottom_sweep != TK_TRUE  ) &&
	     ( read_only_bottom_sweep != TK_FALSE )  )
	{
	  TKreportWarning( E_TK_GV_B_SWEEP ) ;
	  return(	   E_TK_GV_B_SWEEP ) ;
	}

	/* check that the vos size structure was allocated by the user */

	if ( VosSize == NULL )
	{
	  TKreportWarning( E_TK_GV_ALLOC_VS ) ;
	  return( 	   E_TK_GV_ALLOC_VS ) ;
	}

	/* If there are no errors, return success */

	return( TK_SUCCESS ) ;
}


/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TKreadL1GV_driver( IO_HANDLE * handle ,
  L1B_1C_GV ** vos , int VosNum , char param_name[ 20 ],
  int read_only_bottom_sweep, VosSizeSTRUCT *VosSize )

DESCRIPTION:

	This function reads volume scan data for a variety
	of API's, including:

	  TKreadL1GVbyVosNum	(in C)
	  TKreadL1GVparam	(in C and FORTRAN)
	  TKreadL1GV1byteParam  (in FORTRAN)
	  TKreadL1GVbaseSweep	(in C and FORTRAN)

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle			IO handle for a granule that has been previously
			opened with TKopen.

VosNum			the number of the volume scan

param_name              the name of the parameter, set to the macro
			TK_READ_ALL_PARAM if all parameters are to be
			read.

read_only_bottom_sweep  a logical variable that equals TK_TRUE if
                        only the bottom sweep is to be read from
                        the given parameter, and equals TK_FALSE otherwise.

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
vos			a level 1 GV volume scan

VosSize			a structure that describes the size of a 
			level 1 GV volume scan

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
status			an error message passed from a subfunction
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
TKreadL1GV_error_check		check for errors in the inputs
TKresetL1GV			reset the SDS and V Data structures in
				the HDF file

TK_read_1c51_1b51_in_C		to read the volume scan data

TKreportWarning                 to warn operator that an error has occurred

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 

int TKreadL1GV_driver( IO_HANDLE * handle , 
  L1B_1C_GV ** vos , int VosNum , char param_name[ 20 ],
  int read_only_bottom_sweep, VosSizeSTRUCT *VosSize )
{
	/* use VosNum = 0 to read the first volume scan */

	int		status ;
	char		algorithm_ID[ TK_ALGORITHM_ID_SIZE ] ;
/*
 * ---- check inputs
 */
	status	= TKreadL1GV_error_check( handle, vos, VosNum,
	  param_name, read_only_bottom_sweep, VosSize ) ;

	if ( status != TK_SUCCESS )
	{
	  TKreportWarning( status ) ;
	  return(	   status ) ;
	}
/*
 * ---- store VosNum in handle
 */
	/* update the granule handle with the number of the
	desired vos.  Note that the first thing TKreadL1GV does
	is incriment this number by one. This value will be used
	by the primite function TKreadL1GV( ). */

	handle->currNumOfScanl	= VosNum ;

	/* place the VosNum in the VosSizeSTRUCT structure, which will be
	used when the size of the vos is determined in preparation
	for allocating the vos. */

	VosSize->which_scan	= VosNum ;
/*
 * ---- Before reading the vos, reset the Vgroups in the configuration
 *	string of the granule handle.
 */
	status  = TKresetL1GV( handle, VosNum ) ; 

	if ( status != TK_SUCCESS )
        {
          TKreportWarning( status ) ;
          return(          status ) ;
        }
/*
 * ---- read the volume scan
 */
	status	= TK_read_1c51_1b51_in_C( handle, vos , VosSize ,
          algorithm_ID, param_name, read_only_bottom_sweep ) ;

        if ( status != TK_SUCCESS ) 
        {
          TKreportWarning( status ) ;
          return(          status ) ;
        }
/*
 * ---- After reading the vos, reset the Vgroups in the configuration
 *	string of the granule handle.
 */
	status  = TKresetL1GV( handle, VosNum ) ; 

        if ( status != TK_SUCCESS ) 
        {
          TKreportWarning( status ) ;
          return(          status ) ;
        }

	return( status ) ;
}

/*
	============================================================
	============================================================

		Functions for reading elements of volume scans in C

        ============================================================

	TKreadL1GVdate		API
	TKpickL1GVvosNum	API

	TKreadL1GVsummary	API
	TKprintL1GVsummary	API

	============================================================
	============================================================
*/

/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TKreadL1GVdate( IO_HANDLE *sGranuleHandle, struct tm date[ MAX_VOS ],
        int * num_date )

DESCRIPTION:

        This function reads the start date/times of all the volume
        scans in a granule.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle			IO handle for a granule that has been previously
			opened with TKopen.

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
date_array		an array of dates for use in FORTRAN

num_date		the number of dates read from the granule,
			this is also equal to the number of volume scans
			in the granule.

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
E_TK_GV_USE_TKOPEN
E_TK_GV_D_EMPTY
E_TK_GV_L1_PROD
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
VSseek				to reset an V data structure
vdataRead			to read a V data structure

TKreportWarning                 to warn operator that an error has occurred

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
int TKreadL1GVdate( IO_HANDLE *sGranuleHandle, struct tm date[ MAX_VOS ],
	int * num_date )
{
  	VDATA_LINK 	*tempVdata;
  	VGROUP_LINK 	*currVgroup ;

  	uint8 		buffer[ MINIMAL_BUFFER_SIZE ] ;
  	uint8		* ptr ;
  	int		which_vos = 0 ;
	int		status ;

  	VOLUME_DESCRIPTORS  volDes ;  /* used in sizeof( ) */
	int		sizeof_year   = sizeof( volDes.year   ) ;
	int		sizeof_month  = sizeof( volDes.month  ) ;
	int		sizeof_day    = sizeof( volDes.day    ) ;
	int		sizeof_hour   = sizeof( volDes.hour   ) ;
	int		sizeof_minute = sizeof( volDes.minute ) ;
	int		sizeof_second = sizeof( volDes.second ) ;
	int		sizeof_header = sizeof( volDes.verNum ) +
	  sizeof( volDes.volNum ) + sizeof( volDes.sizeDataRec )+
	  sizeof( volDes.projectName ) ;

			/* to have int16 varariables to memcpy into */
  	int16		year, month, day, hour, minute, second ;
/*
 * ---- check inputs
 */
        /* check to see that the granule has been openned with TKopen. */

        if ( sGranuleHandle->cfgSTR == NULL )
        {
          TKreportWarning( E_TK_GV_USE_TKOPEN ) ;
          return(          E_TK_GV_USE_TKOPEN ) ;
        }

        /* check that the granule is not an empty granule */

        if ( sGranuleHandle->cfgSTR->vGroup->subVgroup == NULL )
        {
          TKreportWarning( E_TK_GV_D_EMPTY ) ;
          return(          E_TK_GV_D_EMPTY ) ;
        }

        /* check that the granule is a level 1 GV granule */

        if ( ( sGranuleHandle->productID != TK_L1C_GV ) &&
             ( sGranuleHandle->productID != TK_L1B_GV )  )
        {
          TKreportWarning( E_TK_GV_L1_PROD ) ;
          return(          E_TK_GV_L1_PROD ) ;
        }
/*
 * ---- get the dates for all the volume scans, starting with the
 *	first in the granule.
 */
  	currVgroup   = sGranuleHandle->cfgSTR->vGroup->subVgroup ;

  	while (currVgroup != NULL)
  	{
	  ptr	= & buffer[0] ; /* point to the start of the buffer */
/*
 * ------ find the date in the current volume scan
 *
 *	  By going to "next" you skip over the first set of data
 *	  which is the "comments".  You skip directly to the second
 *	  set of data which is the "volume descriptor" that contains
 *	  the start time of the volume scan.
 *
 *	!! include the correction factor to the date ??
 */

	  /* return to the beginning of the record. */

	  /* After TKopen, all the ID's are at the first record, and
	  after TKreadL1GV is used once they are left at the last
	  record.  If you want to access those ID's a second ,time
	  you have to directly call an HDF routine, to reset the ID!
	  The Toolkit was supposed to be designed so that it would never
	  be necessary to access HDF routines directly.  Also,
	  however, the existing toolkit makes it impossible to
	  reread a volume scan. */
	
	  /* You have to use the long name in both calls to VSseek
	  because vdataRead incriments to the next vdata. */

	  status	= VSseek( currVgroup->vdata->next->vdataID, 0 ) ; 

    	  tempVdata 	= currVgroup->vdata->next ;
    	  vdataRead( 	& tempVdata, ptr ) ;	/* read volume descriptor */

	  status	= VSseek( currVgroup->vdata->next->vdataID, 0 ) ; 

    	  ptr 		+= sizeof_header ;	/* advance past the header */
/*
 * ------ copy the date from the unformatted buffer into 2 byte integers
 */
    	  memcpy( & year ,  ptr , sizeof_year ) ;  ptr += sizeof_year ;
   	  memcpy( & month , ptr , sizeof_month ) ; ptr += sizeof_month ;
    	  memcpy( & day ,   ptr , sizeof_day ) ;   ptr += sizeof_day ;
   	  memcpy( & hour ,  ptr , sizeof_hour ) ;  ptr += sizeof_hour ;
  	  memcpy( & minute, ptr , sizeof_minute ); ptr += sizeof_minute ;
   	  memcpy( & second, ptr , sizeof_second ) ;

/*
 * ------ copy the date into the standard TSDIS date structure,
 *	  which is "struct tm".
 */
	  /* struct tm starts with 0 for the year 1900 */
	  date[ which_vos ].tm_year	= year  - 1900 ;

	  /* struct tm starts months with 0, and days with 1 */
	  date[ which_vos ].tm_mon	= month -    1 ;
	  date[ which_vos ].tm_mday	= day ;
	  date[ which_vos ].tm_hour	= hour ;
	  date[ which_vos ].tm_min	= minute ;
	  date[ which_vos ].tm_sec	= second ;

	  /* every programmer should set this flag to -1 before using
	  the "struct tm" in mktime( ), or else the input value of the
	  "struct tm" may be changed by mktime( ). */

	  date[ which_vos ].tm_isdst	= -1 ;
/*
 * ------ advance to the next volume scan
 */
	  currVgroup	= currVgroup->next ;
      	  which_vos++ ;
  	}

	* num_date	= which_vos ;

  	return( TK_SUCCESS ) ;
}

/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TKpickL1GVvosNum(IO_HANDLE * handle, struct tm target_date, int *VosNum ,
        int * min_diff_in_seconds )

DESCRIPTION:

        This function returns the volume scan number of the volume
	scan whose start date/time is closest to the user supplied
	target_date.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle			IO handle for a granule that has been previously
			opened with TKopen.

target_date		The function returns the volume scan number
			of the volume scan whose start date/time is
			closest to this user supplied date/time.
OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
VosNum			The volume scan number of the volume scan
			scan whose start date/timeis closest to the
			target_date

min_diff_in_seconds	the difference in seconds between the target date
			and the volume scan with closest start time.

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
E_TK_GV_BAD_YR
E_TK_GV_BAD_DATE
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
TKreadL1GVdate			to read the start date/times of the volume
				scans in the granule

TKreportWarning                 to warn operator that an error has occurred

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
int TKpickL1GVvosNum(IO_HANDLE * handle, struct tm target_date, int *VosNum ,
	int * min_diff_in_seconds )
{
	int		status ;
	struct tm	date[ MAX_VOS ] ;
	time_t		diff_in_seconds[ MAX_VOS ], target_seconds, a ;
	int		num_date ;
	int		i ;
/*
 * ---- check to be sure a blank date has not been passed as
 *	the target date
 */
	if ( ( target_date.tm_year <=  80 ) ||
	     ( target_date.tm_year >= 120 )  )
	{
	  TKreportWarning( E_TK_GV_BAD_YR ) ;

          * VosNum     		=   0 ;
          * min_diff_in_seconds = -99 ;

	  return( E_TK_GV_BAD_YR ) ;
	}

	/* In struct tm, months go from 0 to 11, and day of the month
	start with 1. */

	if ( ( target_date.tm_sec  < 0 ) || ( target_date.tm_min  < 0 ) || 
	     ( target_date.tm_hour < 0 ) || ( target_date.tm_mday < 1 ) ||
	     ( target_date.tm_mon  < 0 )  )
	{
	  TKreportWarning( E_TK_GV_BAD_DATE ) ;

          * VosNum     		=   0 ;
          * min_diff_in_seconds = -99 ;

	  return( E_TK_GV_BAD_DATE ) ;
	}
/*
 * ---- setup calculations
 */
	* min_diff_in_seconds	= TWENTY_YEARS ;
	target_date.tm_isdst	= -1 ;
	target_seconds		= mktime( & target_date ) ;

	status = TKreadL1GVdate( handle, date, & num_date ) ;

	if ( status != TK_SUCCESS ) 
	{
	  TKreportWarning( status ) ;
	  
          * VosNum     		=   0 ;
          * min_diff_in_seconds = -99 ;

	  return( status ) ;
	}
/*
 * ---- calculate differences between target date and the start dates
 *	of the volume scans in the granule.
 */
	for ( i = 0 ; i < num_date ; i++ )
	{
	  if ( date[ i ].tm_year == 0 )

	    diff_in_seconds[ i ]	= -1 ;

	  else
	  {
	    date[i].tm_isdst	 = -1 ;

	    diff_in_seconds[ i ] = mktime( & date[ i ] ) -  
	      target_seconds ;

	    /* fabs does not work on integers */

	    if ( diff_in_seconds[ i ] < 0 )
	
	      diff_in_seconds[ i ] = - diff_in_seconds[ i ] ;
	  }
	}
/*
 * ---- determine the volume scan number of the volume scan with
 *	start time closest to the target time.
 */
	for ( i = 0 ; i < num_date ; i++ )

	  if ( ( diff_in_seconds[ i ] < (* min_diff_in_seconds) ) &&
	       ( diff_in_seconds[ i ] != -1                     )  )
	  {
	    * min_diff_in_seconds	= diff_in_seconds[ i ] ;
	    * VosNum			= i ;
	  }

	return( TK_SUCCESS ) ;
}


/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TKreadL1GVsummary( IO_HANDLE * handle ,
  int VosNum, VosSummarySTRUCT *VosSummary )

DESCRIPTION:

        This function reads information that summarizes a volume scan.
	Based on this information, the user can decide if he wants
	to read the volume scan.

	The summary data includes the size of the data, the start
	date/time, and the names of the parameters in the volume scan.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle			IO handle for a granule that has been previously
			opened with TKopen.

VosNum                  the number of the volume scan


OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
VosSummary		a structure that contains a summary of the
			volume scan

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
E_TK_GV_USE_TKOPEN
E_TK_GV_V_EMPTY
E_TK_GV_L1_PROD
E_TK_GV_VOS_NUM
status			an error status return from a subfunction
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
TKresetL1GV                     reset the SDS and V Data structures in
                                the HDF file

TK_read_1c51_1b51_summary	to fill the summary structure

TKreportWarning                 to warn operator that an error has occurred

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 

int TKreadL1GVsummary( IO_HANDLE * handle , 
  int VosNum, VosSummarySTRUCT *VosSummary )
{
	/* use VosNum = 0 to read the first volume scan */

	int		status ;
	char		algorithm_ID[ TK_ALGORITHM_ID_SIZE ] ;
/*
 * ---- check for errors
 */
	/* check to see that the granule has been openned with TKopen. */

	if ( handle->cfgSTR == NULL )
	{
	  TKreportWarning( E_TK_GV_USE_TKOPEN ) ;
	  return( 	   E_TK_GV_USE_TKOPEN ) ;
	}

	/* check that the granule is not an empty granule */

	if ( handle->cfgSTR->vGroup->subVgroup == NULL )
	{
	  TKreportWarning( E_TK_GV_V_EMPTY ) ;
	  return( 	   E_TK_GV_V_EMPTY ) ;
	}

	/* check that the granule is a level 1 GV granule */

	if ( ( handle->productID != TK_L1C_GV ) &&
	     ( handle->productID != TK_L1B_GV )  )
	{
	  TKreportWarning( E_TK_GV_L1_PROD ) ;
	  return(	   E_TK_GV_L1_PROD ) ;
	}
	
	/* check that the user supplied vos number is reasonable */

	if ( ( VosNum < 0 ) || ( VosNum >= MAX_VOS ) )
	{
	  TKreportWarning( E_TK_GV_VOS_NUM ) ;
	  return( 	   E_TK_GV_VOS_NUM ) ;
	}
/*
 * ---- store VosNum in handle
 */
	/* update the granule handle with the number of the
	desired vos.  Note that the first thing TKreadL1GV does
	is incriment this number by one. This value will be used
	by the primite function TKreadL1GV( ). */

	handle->currNumOfScanl	= VosNum ;
/*
 * ---- Before reading the vos, reset the Vgroups in the configuration
 *	string of the granule handle.
 */
	status  = TKresetL1GV( handle, VosNum ) ; 

	if ( status != TK_SUCCESS )
        {
          TKreportWarning( status ) ;
          return(          status ) ;
        }
/*
 * ---- read the volume scan
 */
	status	= TK_read_1c51_1b51_summary( handle, VosSummary ) ;

        if ( status != TK_SUCCESS ) 
        {
          TKreportWarning( status ) ;
          return(          status ) ;
        }
/*
 * ---- After reading the vos, reset the Vgroups in the configuration
 *	string of the granule handle.
 */
	status  = TKresetL1GV( handle, VosNum ) ; 

        if ( status != TK_SUCCESS ) 
        {
          TKreportWarning( status ) ;
          return(          status ) ;
        }

	return( status ) ;
}

/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void TKprintL1GVsummary( VosSummarySTRUCT *VosSummary )

DESCRIPTION:

        This function prints to the screen information that 
	summarizes a volume scan.  Based on this information, 
	the user can decide if he wants to read the volume scan.

	The summary data includes the size of the data, the start
	date/time, and the names of the parameters in the volume scan.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
VosSummary		a structure that contains a summary of the
			volume scan

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
none

RETURN VALUES:

 Mnemonic
 --------
none
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
none

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
void TKprintL1GVsummary( VosSummarySTRUCT *VosSummary )
{
	char		 product_ID_string[ 20 ] = "N/A" ;
	int		i ;
/*
 * ---- setup
 */
	if ( VosSummary->productID == TK_L1C_GV )

	  strcpy( product_ID_string , "TK_L1C_GV" ) ;

	else if ( VosSummary->productID == TK_L1B_GV )

	  strcpy( product_ID_string , "TK_L1B_GV" ) ;
/*
 * ---- print the summary
 */
        printf( "\n" ) ;
        printf( " granule_ID %s \n\n", VosSummary->granule_ID ) ;
	printf( " product_ID %s = %d ", product_ID_string, 
	  VosSummary->productID ) ;

	printf( "\t volume scan # %d of %d \n\n",
	  VosSummary->which_scan +1, VosSummary->scans_in_granule ) ;

	printf( " start date/time %d/%d/%d %d:%02d:%02d UTC \n\n",
	  VosSummary->start_year,   VosSummary->start_month ,
	  VosSummary->start_day ,   VosSummary->start_hour ,
	  VosSummary->start_minute, VosSummary->start_second ) ;

	printf( " %-15s %8s %8s %8s %8s   %-20s \n" ,
	  "dimensions", "n param", "n sweep", "n ray", "n cell", 
	  "param name" ) ;

	printf( " %13s %8d %8d %8d %8d       %-20s \n" ,
	  " ", VosSummary->num_param , VosSummary->num_sweep ,
	       VosSummary->num_ray  ,  VosSummary->num_cell[ 0 ] ,
	       VosSummary->param_name[ 0 ] ) ;

	for ( i = 1 ; i < VosSummary->num_param ; i++ )

	  printf( " %40s %8d       %-20s \n" , " ", 
	    VosSummary->num_cell[ i ], VosSummary->param_name[ i ] ) ;

	printf( "\n" ) ;
}

/*
	============================================================
	============================================================

		Functions to allocate and free volume scans

	============================================================

	name				type

	TKallocateL1GV			internal
	TK_alloc_a_param		internal
	TK_alloc_a_1byte_param		internal
	TK_alloc_a_2byte_param		internal

	TKfreeL1GV			API
	TK_dealloc_a_1byte_param	internal
	TK_dealloc_a_2byte_param	internal
	
	============================================================
	============================================================
*/

/*
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int     TKallocateL1GV(
            VosSizeSTRUCT *VosSize, L1B_1C_GV **vos )

DESCRIPTION:

Allocate a raw volume scan which will be read from
a 1c51 or 1b51 granule.

INPUT VARIABLES:

 Parameter Name         Description
 --------------         -----------
VosSize                 a structure that contains the dimensions
                        of the volume scan.  The structure is
                        filled prior to calling this function.

OUTPUT VARIABLES:

 Parameter Name         Description
 --------------         -----------
vos                     a volume scan with memory allocated

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS                 if nothing went wrong
E_TK_MEM_ALLOC             if memory could not be allocated

FUNCTIONS CALLED:

 Function Name                          Description
 -------------                          -----------
TK_alloc_a_param

MODIFICATIONS:

 Date       Programmer       Description of change
 ----       ----------       ---------------------
 4/16/97    O. Kelley        implemented code

 8/7/97     O. Kelley        enhanced for 1B51

	Altered code to work with 1B51 as well as 1C51.
	This alteration was to allow each parameter to have 
	a different number of cells in their rays.

 10/13/97   O. Kelley        NULL pointers used to simplify deallocation

-------------------- end prolog --------------------

-------------------- begin notes --------------------

	assumptions			8/11/97

	This function and its subfunctions make the same
	assumptions as the TRMM Office's Radar Science Library (RSL).
	The RSL assumes that all 1B51 parameters are 2 byte variables.
	All 1C51 2 byte parameters are assumed to be preceded by
	a 1 byte mask.

-------------------- end notes --------------------
*/

int	TKallocateL1GV(
   	    VosSizeSTRUCT *VosSize, L1B_1C_GV **vos )
{
        int 	num_cell, pindex;
	int	status ;
/*
 * ---- allocate the pointer to the raw volume scan
 */
	/* initial the raw volume scan to null here because it
	might have been allocated and dealloced before.  Deallocation
	does not return a pointer to NULL.  This initialization is
	required in order that the error handling that follows be able
	to notice when the allocation fails. */

	* vos	= NULL ;

        * vos 	= (L1B_1C_GV *)calloc(1, sizeof(L1B_1C_GV));

        if ( * vos  == NULL )
        {
          TKreportWarning( E_TK_MEM_ALLOC ) ;
          return( 	   E_TK_MEM_ALLOC ) ;
        }
/*
 * ---- allocate the pointers inside the raw volume scan
 */
        for (pindex=0; pindex<MAX_PARM; pindex++)
        {
          if (pindex < VosSize->num_param)
          {
	    VosSize->which_param	= pindex ;

            status = TK_alloc_a_param( 
	      &( (* vos)->sensor.parm[pindex] ), VosSize);

            if ( status != TK_SUCCESS )
            {
              TKreportWarning( status ) ;
              return( 	       status ) ;
            }
          }
          else  
            (* vos) ->sensor.parm[pindex] = (PARAMETER *)NULL;
        } 

        return( TK_SUCCESS ) ;
}

/*
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TK_alloc_a_param(
        PARAMETER               **parm,
        VosSizeSTRUCT            *VosSize )

DESCRIPTION:

allocate one parameter inside a raw volume scan.

INPUT VARIABLES:

 Parameter Name         Description
 --------------         -----------
VosSize                 a structure that contains the dimensions
                        of the volume scan.  The structure is
                        filled prior to calling this function.

  This elements is used only if reading a single parameter from
  the volume scan.

    byte_size

  The following elements are used only if all parameters are being read
  from the volume scan.

    productID
    which_param

  These elements are used by subfunctions.

    num_sweep
    num_ray
    num_cell[ i ]

OUTPUT VARIABLES:

 Parameter Name         Description
 --------------         -----------
parm                    a parameter inside a 1c51 volume scan
                        with memory allocated

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS                 if nothing went wrong
E_TK_MEM_ALLOC             if memory could not be allocated

FUNCTIONS CALLED:

 Function Name                          Description
 -------------                          -----------
TK_alloc_a_1byte_param
TK_alloc_a_2byte_param

MODIFICATIONS:

 Date       Programmer       Description of change
 ----       ----------       ---------------------
 4/16/97    O. Kelley        implemented code

 8/7/97     O. Kelley        enhanced for 1B51

 10/13/97   O. Kelley        NULL pointers used to simplify deallocation

-------------------- end prolog --------------------

-------------------- begin notes --------------------

-------------------- end notes --------------------
*/

int TK_alloc_a_param(
        PARAMETER               **parm,
        VosSizeSTRUCT            * VosSize )
{
        int             status ;
/*
 * ---- allocate the parameter pointer
 */
        * parm = (PARAMETER *)calloc(1, sizeof(PARAMETER));

        if ( * parm  == NULL )
        {
          TKreportWarning( E_TK_MEM_ALLOC ) ;
          return( 	   E_TK_MEM_ALLOC ) ;
        }
/*
 * ---- it is crucial for later deallocation that param pointers
 *      be NULL if not allocated.
 */
        (* parm) ->parmData1byte        = NULL ;
        (* parm) ->parmData2byte        = NULL ;
/*
 * ---- first check if the default byte size of the parameter is
 *	being overriden.
 */
	if ( VosSize->byte_size == 1 )
	
          status = TK_alloc_a_1byte_param(
            & ( (* parm) ->parmData1byte ) , VosSize ) ;

	else if ( VosSize->byte_size == 2 )

          status = TK_alloc_a_2byte_param(
            & ( (* parm) ->parmData2byte ) , VosSize ) ;
/*
 * ---- If the byte_size field of the Vos Size structure is not set,
 *	then use the default method of determining the byte size of
 *	the parameter to be allocated.  The default method is the same
 *	method used by the TRMM Office Radar Science Library (TO RSL).
 *	All 1B51 parameters are 2 byte parameters.  All 1C51 parameters
 *	come in pairs with 1 byte Quality control mask (QCM) preceeding
 *	the 2 byte data parameter.
 */
        else if ( VosSize->productID == TK_L1C_GV )
        {
          if ( ( VosSize->which_param % 2) == 0 )
	  
            status = TK_alloc_a_1byte_param(
              & ( (* parm) ->parmData1byte ) , VosSize ) ;

	  else
	  
            status = TK_alloc_a_2byte_param(
              & ( (* parm) ->parmData2byte ) , VosSize ) ;
	}
        else if ( VosSize->productID == TK_L1B_GV )
	
          status = TK_alloc_a_2byte_param(
              & ( (* parm) ->parmData2byte ), VosSize ) ;
	
        else
        
          TKreportWarning( E_TK_GV_L1_PROD ) ;

       return( status ) ;
}

/*
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int     TK_alloc_a_1byte_param(
                int8                   **** parmData1byte ,
                VosSizeSTRUCT                   * VosSize  )

DESCRIPTION:

allocate a 1 byte parameter inside a 1c51 or 1b51 volume scan.


INPUT VARIABLES:

 Parameter Name         Description
 --------------         -----------
VosSize->num_sweep              the number of sweeps in the volume scan
VosSize->num_ray                the number of rays in a sweep
VosSize->num_cell[ i ]          the number of cells in a ray of
				the ith parameter

OUTPUT VARIABLES:

 Parameter Name         Description
 --------------         -----------
paramData1byte          a 1 byte parameter with memory allocated

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS               if nothing went wrong
E_TK_MEM_ALLOC           if memory could not be allocated

FUNCTIONS CALLED:

 Function Name                          Description
 -------------                          -----------
none

MODIFICATIONS:

 Date       Programmer       Description of change
 ----       ----------       ---------------------
 4/16/97    O. Kelley        implemented code

 8/7/97     O. Kelley        enhanced for 1B51

 10/13/97   O. Kelley        NULL pointers used to simplify deallocation

-------------------- end prolog --------------------

-------------------- begin notes --------------------

        origin:         TRMM Office Radar Science Library
                        version 1.3
        original
        source file:    toolkit_memory_mgt.c

        purpose:

        To allocate memory for the 3 dimensional 1 byte data
        for a level 1 GV volume scan.  This structure is
        of type L1B_1C_GV and is defined in IO_GV.h

        notes on memory allocation:

        A previous version of this function allocated a solid block of
        2 megabytes of memory.  The block had the following
        dimensions:

           num_sweep * num_ray * num_cell * sizeof( int 8)

        With a typical volume scan, this would be the following number
        of bytes:

            16    * 360  *  200  * 1

        The current version of this function allocates the
        same amount of memory, but the memory is allocated in
        blocks of only 200 bytes.

-------------------- end notes --------------------
*/

int TK_alloc_a_1byte_param(
        int8                   **** parmData1byte ,
        VosSizeSTRUCT                  * VosSize )
{
        int  sweep_index ,  ray_index, cell_index ;

	if ( VosSize->param_position != TK_USE_DEFAULT )

	  cell_index	= VosSize->param_position ;
	else
	  cell_index	= VosSize->which_param ;
/*
 * ---- allocate a pointer for each sweep
 *
 *	To simplify deallocation, allocate space for
 *	the maximum number of sweeps, and initialize the
 *	sweeps to point to NULL.  Later statements in this
 *	function will give the sweeps that are actuall used
 *	non NULL addresses.  This method wastes only about 20 bytes,
 *	and vastly simplifies the deallocation of volume scans.
 */
        ( * parmData1byte ) =
            (int8 ***) calloc( MAX_SWEEP , sizeof(int8 **) ) ;

        if ( ( * parmData1byte )   == NULL )
        {
          TKreportWarning( E_TK_MEM_ALLOC ) ;
          return( 	   E_TK_MEM_ALLOC ) ;
        }
/*
 * ---- theoretically the C language should have automatically
 *	initialized these pointers to NULL.  Since it is important
 *	to deallocation that they be NULL, here the initialization
 *	is done explicitly.
 */
	for ( sweep_index = 0 ; sweep_index < MAX_SWEEP  ; sweep_index++ )

	  ( * parmData1byte ) [ sweep_index ] = NULL ;
/*
 * ---- allocate space for rays and cells in the used sweeps
 */
        for ( sweep_index = 0; sweep_index < VosSize->num_sweep; sweep_index++)
        {
/*
 * ------ allocate a pointer for each ray in a given sweep
 */
          ( * parmData1byte ) [ sweep_index ]   =
             (int8 **) calloc( MAX_RAY , sizeof(int8 *) ) ;

          if ( (* parmData1byte)[ sweep_index ]   == NULL )
          {
            TKreportWarning( E_TK_MEM_ALLOC ) ;
            return( 	     E_TK_MEM_ALLOC ) ;
          }

	  /* This simplifies deallocation, and wastes very little space. */

	  for ( ray_index =0 ;  ray_index < MAX_RAY ; ray_index++ )

	    ( * parmData1byte ) [ sweep_index ][ ray_index ] = NULL ;
/*
 * ------ allocate each cell in a given ray and sweep
 *
 *        In 1B51, the reflectivity and velocity parameters have
 *        different number of cells.
 */
          for ( ray_index =0 ;  ray_index < VosSize->num_ray ;  ray_index++ )
	  {
            ( * parmData1byte ) [ sweep_index ][ ray_index ] =
              (int8 *) calloc( VosSize->num_cell[ cell_index ], 
	      sizeof(int8) ) ;

            if ( ( * parmData1byte )[ sweep_index ][ ray_index ] == NULL )
            {
              TKreportWarning( E_TK_MEM_ALLOC ) ;
              return( 	       E_TK_MEM_ALLOC ) ;
            }
	  }
        }

        return( TK_SUCCESS ) ;
}


/*
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int     TK_alloc_a_2byte_param(
                int8                   **** parmData2byte ,
                VosSizeSTRUCT                   * VosSize  )

DESCRIPTION:

allocate a 2 byte parameter in a 1c51 or 1b51 volume scan


INPUT VARIABLES:

 Parameter Name         Description
 --------------         -----------
VosSize->num_sweep              the number of sweeps in the volume scan
VosSize->num_ray                the number of rays in a sweep
VosSize->num_cell[ i ]          the number of cells in a ray of the
				ith parameter

OUTPUT VARIABLES:

 Parameter Name         Description
 --------------         -----------
paramData2byte          a 2 byte parameter with a memory allocated

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS                 if nothing went wrong
E_TK_MEM_ALLOC             if memory could not be allocated

FUNCTIONS CALLED:

 Function Name                          Description
 -------------                          -----------
none

MODIFICATIONS:

 Date       Programmer       Description of change
 ----       ----------       ---------------------
 4/16/97    O. Kelley        implemented code

 8/7/97     O. Kelley        enhanced for 1B51

 10/13/97   O. Kelley        NULL pointers used to simplify deallocation

-------------------- end prolog --------------------
-------------------- begin notes --------------------

        function name:  TK_alloc_a_2byte_param

        origin:         TRMM Office Radar Science Library
                        version 1.3
        original
        source file:    toolkit_memory_mgt.c

        purpose:

        To allocate memory for the 3 dimensional 2 byte data
        for a level 1 GV volume scan.  This structure is
        of type L1B_1C_GV and is defined in IO_GV.h

        notes on memory allocation:

        A previous version of this function allocated a solid block of
        2 megabytes of memory.  The block had the following
        dimensions:

           num_sweep * num_ray * num_cell * sizeof( int 16)

        With a typical volume scan, this would be the following number
        of bytes:

            16    * 360  *  200  * 2

        The current version of this function allocates the
        same amount of memory, but the memory is allocated in
        blocks of only 400 bytes.

-------------------- end notes --------------------
*/

int TK_alloc_a_2byte_param(
        int16                   **** parmData2byte ,
        VosSizeSTRUCT                   * VosSize )
{
        int  sweep_index ,  ray_index, cell_index ;

        if ( VosSize->param_position != TK_USE_DEFAULT )

          cell_index    = VosSize->param_position ;
        else
          cell_index    = VosSize->which_param ;
/*
 * ---- allocate a pointer for each sweep
 *
 *      To simplify deallocation, allocate space for
 *      the maximum number of sweeps, and initialize the
 *      sweeps to point to NULL.  Later statements in this
 *      function will give the sweeps that are actuall used
 *      non NULL addresses.  This method wastes only about 20 bytes,
 *      and vastly simplifies the deallocation of volume scans.
 */
        ( * parmData2byte ) =
            (int16 ***) calloc( MAX_SWEEP, sizeof(int16 **) ) ;

        if ( ( * parmData2byte )   == NULL )
        {
          TKreportWarning( E_TK_MEM_ALLOC ) ;
          return( 	   E_TK_MEM_ALLOC ) ;
        }
/*
 * ---- theoretically the C language should have automatically
 *      initialized these pointers to NULL.  Since it is important
 *      to deallocation that they be NULL, here the initialization
 *      is done explicitly.
 */
        for ( sweep_index = 0 ;  sweep_index < MAX_SWEEP  ; sweep_index++ )

          ( * parmData2byte ) [ sweep_index ] = NULL ;
/*
 * ---- allocate space for rays and cells
 */
        for ( sweep_index = 0; sweep_index < VosSize->num_sweep; sweep_index++)
        {
/*
 * ------ allocate a pointer for each ray in a given sweep
 */
          ( * parmData2byte ) [ sweep_index ]   =
             (int16 **) calloc( MAX_RAY , sizeof(int16 *) ) ;

          if ( (* parmData2byte)[ sweep_index ]   == NULL )
          {
            TKreportWarning( E_TK_MEM_ALLOC ) ;
            return( 	     E_TK_MEM_ALLOC ) ;
          }

          /* This simplifies deallocation, and wastes very little space. */

          for ( ray_index =0 ;  ray_index < MAX_RAY ; ray_index++ )

            ( * parmData2byte ) [ sweep_index ][ ray_index ] = NULL ;
/*
 * ------ allocate each cell in a given ray and sweep
 *
 *        In 1B51, the reflectivity and velocity parameters have
 *        different number of cells.
 */
          for ( ray_index =0 ;  ray_index < VosSize->num_ray ;  ray_index++ )
	  {
            ( * parmData2byte ) [ sweep_index ][ ray_index ] =
              (int16 *) calloc( VosSize->num_cell[ cell_index ], 
	      sizeof(int16) );

            if ( ( * parmData2byte )[ sweep_index ][ ray_index ] == NULL )
            {
              TKreportWarning( E_TK_MEM_ALLOC ) ;
              return( 	       E_TK_MEM_ALLOC ) ;
            }
          }
        }

        return( TK_SUCCESS ) ;
}


/*
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void TKfreeL1GV( L1B_1C_GV           *vos )

DESCRIPTION:

Deallocate a raw volume scan that was read from the input
1c51 or 1b51 granule.

INPUT VARIABLES:

 Parameter Name         Description
 --------------         -----------
vos               the raw volume scan to be deallocated

OUTPUT VARIABLES:

 Parameter Name         Description
 --------------         -----------
vos               deallocated

RETURN VALUES:

 Mnemonic
 --------
void

FUNCTIONS CALLED:

 Function Name                  Description
 -------------                  -----------
TK_dealloc_a_1byte_param        to deallocate a 1 byte parameter
                                of a 1c51 or 1b51 raw volume scan

TK_dealloc_a_2byte_param        to deallocate a 2 byte parameter
                                of a 1c51 or 1b51 raw volume scan 

MODIFICATIONS:

 Date       Programmer       Description of change
 ----       ----------       ---------------------
 4/16/97    O. Kelley        implemented code

 10/13/97   O. Kelley	     NULL pointers used to simplify deallocation

-------------------- end prolog --------------------

-------------------- begin notes --------------------

-------------------- end notes --------------------
*/

void  TKfreeL1GV(  L1B_1C_GV  **vos )
{ 
        int pindex = 0 ;
/*
 * ---- verify that the volume scan needs to be deallocated.
 */
	if ( ( vos != NULL ) && ( (*vos) != NULL ) )
	{
/*
 * ------ deallocate the pointers inside the raw volume scan
 */
          while ( ( pindex < MAX_PARM                      )  &&
	 	  ( (*vos)->sensor.parm[pindex] != NULL )   )
	  {
	    if ( (*vos)->sensor.parm[pindex]->parmData1byte != NULL )

	      TK_dealloc_a_1byte_param(
	        (*vos)->sensor.parm[pindex]->parmData1byte ) ;

	    if ( (*vos)->sensor.parm[pindex]->parmData2byte != NULL )
 
	      TK_dealloc_a_2byte_param(
	        (*vos)->sensor.parm[pindex]->parmData2byte ) ;

	    free( (*vos)->sensor.parm[pindex] ) ;

	    pindex ++ ;
	  }
/*
 * ------ deallocate the pointer to the raw volume scan, and
 *	  reset it to NULL.
 */
          free( (*vos) ) ;
	  *vos	= NULL ;
	}
}

/*
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void    TK_dealloc_a_1byte_param(
                int8                    *** parmData1byte )

DESCRIPTION:

Deallocate memory from a 1 byte parameter inside a 1c51
or 1b51 volume scan.

INPUT VARIABLES:

 Parameter Name         Description
 --------------         -----------
parmData1byte           a 1 byte parameter inside a 1c51 volume scan

OUTPUT VARIABLES:

 Parameter Name         Description
 --------------         -----------
paramData1byte          deallocated

RETURN VALUES:

 Mnemonic
 --------
none

FUNCTIONS CALLED:

 Function Name                          Description
 -------------                          -----------
none

MODIFICATIONS:

 Date       Programmer       Description of change
 ----       ----------       ---------------------
 4/16/97    O. Kelley        implemented code

 10/13/97   O. Kelley        NULL pointers used to simplify deallocation

-------------------- end prolog --------------------

-------------------- begin notes --------------------

-------------------- end notes --------------------
*/

void     TK_dealloc_a_1byte_param(
                int8                    *** parmData1byte )
{
        int  sweep_index ,  ray_index ;
/*
 * ---- deallocate the data pointers inside the 1 byte pointer
 *
 *	Check all possible sweeps, 0 to MAX_SWEEP -1.  If the
 *	sweep is not NULL, then deallocate it.
 */
        for ( sweep_index = 0 ; sweep_index < MAX_SWEEP ; sweep_index++ )
        
	  if ( parmData1byte[ sweep_index ] != NULL )
	  {
            for ( ray_index =0 ;  ray_index < MAX_RAY ;  ray_index++ )
	      
	      if ( parmData1byte[ sweep_index ][ ray_index ] != NULL )

                free( parmData1byte[ sweep_index ][ ray_index ] ) ;

            free( parmData1byte[ sweep_index ] ) ;
	  }
/*
 * ---- deallocate the 1 byte pointer
 */
        free( parmData1byte ) ;
}


/*
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void     TK_dealloc_a_2byte_param(
                int8                    *** parmData2byte )

DESCRIPTION:

Deallocate memory from a 2 byte parameter inside a 1c51
or 1b51 volume scan

INPUT VARIABLES:

 Parameter Name         Description
 --------------         -----------
parmData2byte           a 2 byte parameter inside a 1c51 volume scan

OUTPUT VARIABLES:

 Parameter Name         Description
 --------------         -----------
paramData2byte          deallocated

RETURN VALUES:

 Mnemonic
 --------
none

FUNCTIONS CALLED:

 Function Name                          Description
 -------------                          -----------
none

MODIFICATIONS:

 Date       Programmer       Description of change
 ----       ----------       ---------------------
 4/16/97    O. Kelley        implemented code

 10/13/97   O. Kelley        NULL pointers used to simplify deallocation

-------------------- end prolog --------------------

-------------------- begin notes --------------------

-------------------- end notes --------------------
*/

void     TK_dealloc_a_2byte_param(
                int16                   *** parmData2byte )
{
        int  sweep_index ,  ray_index ;
/*
 * ---- deallocate the data pointers inside the 2 byte pointer
 */
        for ( sweep_index = 0 ; sweep_index < MAX_SWEEP ; sweep_index++ )

          if ( parmData2byte[ sweep_index ] != NULL )
          {
            for ( ray_index =0 ;  ray_index < MAX_RAY ;  ray_index++ )
              
              if ( parmData2byte[ sweep_index ][ ray_index ] != NULL )

                free( parmData2byte[ sweep_index ][ ray_index ] ) ;

            free( parmData2byte[ sweep_index ] ) ;
          }
/*
 * ---- deallocate the 2 byte pointer
 */
        free( parmData2byte ) ;
}

/*
	============================================================
	============================================================

		Miscellaneous internal Toolkit utilities

	============================================================

	name				type

	TKresetL1GV			internal
	TKreadL1GVsize			internal
	TK_read_1c51_1b51_summary	internal
	
	============================================================
	============================================================
*/

/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TKresetL1GV( IO_HANDLE *sGranuleHandle, int VosNum )

DESCRIPTION:

	This function resets the HDF pointers of a volume scan
	so that the volume scan can be read a second time without
	having to close the file.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle                  IO handle for a granule that has been previously
                        opened with TKopen.

VosNum                  the number of the volume scan

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle 			with HDF pointers reset

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
E_TK_GV_NULL_VOS
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
TKreportWarning   		to warn operator that an error has occurred
 
MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

        This function actually resets all the volume scans
        after this one in the granule.  This is takes very
        little time, and is always a good idea.  In fact, from
        from the GV point of view, it would be nice if all SDS
        and Vgroups where reset immediately after being read.
        It is only for the convenience of reading satellite granules
        that reseting is NOT done immedately after reading
        satellite data.

-------------------- end notes --------------------
*/
 
int TKresetL1GV( IO_HANDLE *sGranuleHandle, int VosNum )
{
  	VGROUP_LINK 	* Vos_Vgroup ;

      	Vos_Vgroup = (VGROUP_LINK *)getCurrVgroup(
	  sGranuleHandle->cfgSTR->vGroup->subVgroup, VosNum +1 ) ;
 
      	if ( Vos_Vgroup == NULL) 
	{
	  TKreportWarning( E_TK_GV_NULL_VOS ) ;
	  return( 	   E_TK_GV_NULL_VOS ) ;
	}
	else
	{
          /* volume metadata */
          resetVgroup( Vos_Vgroup ,                        0 ) ; 

          /* sensor metadata */
          resetVgroup( Vos_Vgroup->subVgroup,              0 ) ;

          /* parameter metadata and data */
          resetVgroup( Vos_Vgroup->subVgroup->subVgroup ,  0 ) ;
	}

  	return (TK_SUCCESS);
}
 
/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int     TKreadL1GVsize(
                IO_HANDLE               * handle ,
                VosSizeSTRUCT                 * VosSize )

DESCRIPTION:

This function determines the size of a 1b51 or 1c51 volume scan.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle			the IO_HANDLE for the openned granule

VosSize->which_scan	the number of the volume scan whose size
			this function will determine.

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
the following fields of the VosSize structure are filled

  num_param			number of parameters in the volume scan

  num_sweep		number of sweeps     in the volume scan

  num_ray			number of rays       in the volume scan

  num_cell			number of cells in each parameter of the 
			volume scan

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if no errors are detected

E_TK_L1_GV		if an error occurs that prevents a level 1
			GV volume scan from being read
						
FUNCTIONS CALLED:

 Function Name   	Description
 -------------  	-----------
TKgetNparm		to read the number of parameters
TKgetNsweep		to read the number of sweeps
TKgetNray		to read the number of rays
TKgetNcell		to read the number of cells
 
MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 9/3/97     O. Kelley        implemented code
 
-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/

int	TKreadL1GVsize(
	 	IO_HANDLE		* handle ,
		VosSizeSTRUCT			* VosSize )
{
	int		status		  	= TK_SUCCESS ;
	int		which_param ;
	int		which_scan	= VosSize->which_scan ;
/*
 * ---- set defaults under the assumption that all parameters
 *	are being read from the volume scan.  A later function will
 *	alter the values from the default setting, if one one
 *	parameter is being read.
 */
	/* The default position in the num_cell array used to
	allocate the parameter is default unless overiden by
	later functions.  The default is overiden when only
	reading one parameter, which could be any one of the
	parameters in the volume scan. */

	VosSize->param_position	= TK_USE_DEFAULT ;

	/* If all parameters are being read from the file then
	this field is not used.  The byte size is determined from
	the position of the parameter in the volume scan, according
	to the method of the TRMM Office Radar Science Library (RSL). */

        VosSize->byte_size      = 0 ;
/*
 * ---- number of parameters
 */
        VosSize->num_param = TKgetNparm( handle , which_scan );

        if ( VosSize->num_param <= 0 ) 
        {
	  status		= E_TK_GV_NPARAM ;
          TKreportWarning( status ) ;
        }
	else
	{
/*
 * ------ number of sweep
 */
          VosSize->num_sweep = TKgetNsweep( handle , which_scan );

          if ( VosSize->num_sweep <= 0 ) 
          {
	    status                = E_TK_GV_NSWEEP ;
            TKreportWarning( status ) ;
          }
	  else
	  {
/*
 * -------- number of ray
 */
            VosSize->num_ray = TKgetNray( handle , which_scan );

            if ( VosSize->num_ray <= 0 ) 
            {
	      status                = E_TK_GV_NRAY ;
              TKreportWarning( status ) ;
            }
	    else
	    {
/*
 * ---------- number of cell
 *
 *	In 1B51, the reflectivity and velocity parameters have
 *	different number of cells.
 */
	      for ( which_param = 0 ;
		    which_param < VosSize->num_param ;
		    which_param ++ )
	      {
                VosSize->num_cell[ which_param ] = TKgetNcell(
                  handle , which_scan, which_param );

                if ( VosSize->num_cell[ which_param ] <= 0 ) 
                {
          	  TKreportWarning( E_TK_GV_NCELL ) ;
	          return( 	   E_TK_GV_NCELL ) ;
                }
	      }
/*
 * ---------- product ID
 */
	      VosSize->productID	= handle->productID ;

	    }
	  }
 	}

	return( status ) ;
}


/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TK_read_1c51_1b51_summary( IO_HANDLE * handle ,
  int VosNum, VosSummarySTRUCT *VosSummary )

DESCRIPTION:

        This function reads information that summarizes a volume scan.
	Based on this information, the user can decide if he wants
	to read the volume scan.

	The summary data includes the size of the data, the start
	date/time, and the names of the parameters in the volume scan.

	This is a low level function that assumes the inputs have
	been checked for errors.  The higher level function
	TKreadL1GVsummary is the API that programmers use.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle			IO handle for a granule that has been previously
			opened with TKopen.

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
VosSummary		a structure that contains a summary of the
			volume scan

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
status			an error status return from a subfunction
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
TKreadL1GVsize			to read the size of the volume scan

TKreadL1GVdate			to read the start date of the volume
				scan

TKreadL1GVparamName		to read the names of the parametes in the
				volume scan

TKreportWarning                 to warn operator that an error has occurred

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/

int	TK_read_1c51_1b51_summary( IO_HANDLE * handle,
	   VosSummarySTRUCT	*VosSummary )
{
	int		status ;
        struct tm       date[ MAX_VOS ] ;
	VosSizeSTRUCT	VosSize ;
	int		num_param, i ;
	int		num_date ;

/* There is no need to check the algorithm ID because the
   function that calls this function, checks the product ID. */

/*
 * ---- determine the size of the level 1 volume scan
 *	under the assumption that all parameters are being read.
 */
        /* place the VosNum in the VosSizeSTRUCT structure, which will be
        used when the size of the vos is determined in preparation
        for allocating the vos. */

        VosSize.which_scan     =  handle->currNumOfScanl ;

	status = TKreadL1GVsize( handle , & VosSize ) ;

	if ( status != TK_SUCCESS )
	{
	  TKreportWarning( status ) ;
	  return( 	   status ) ;
	}

	VosSummary->which_scan	=  VosSize.which_scan ;
	VosSummary->productID	=  VosSize.productID ;
	VosSummary->num_param	=  VosSize.num_param ;
	VosSummary->num_sweep	=  VosSize.num_sweep ;
	VosSummary->num_ray	=  VosSize.num_ray ;

	for ( i = 0 ; i < VosSummary->num_param ; i++ )

	  VosSummary->num_cell[ i ]  =  VosSize.num_cell[ i ] ;
/*
 * ---- read the date
 */
        status = TKreadL1GVdate( handle, date, & num_date ) ;

        if ( status != TK_SUCCESS )
        {
          TKreportWarning( status ) ;
          return( 	   status ) ;
        }

	VosSummary->start_year	 = date[ VosSize.which_scan ].tm_year +1900 ;
	VosSummary->start_month  = date[ VosSize.which_scan ].tm_mon     +1 ;
	VosSummary->start_day    = date[ VosSize.which_scan ].tm_mday ;
	VosSummary->start_hour   = date[ VosSize.which_scan ].tm_hour ;
	VosSummary->start_minute = date[ VosSize.which_scan ].tm_min  ;
	VosSummary->start_second = date[ VosSize.which_scan ].tm_sec  ;
/*
 * ---- grab the granule ID and vos in granule from the handle
 */
	strcpy( VosSummary->granule_ID , handle->dataFileName ) ;

	VosSummary->scans_in_granule	= TKgetNvos( handle ) ;
/*
 * ---- read the names of the parameters
 */
	status	= TKreadL1GVparamName( handle, VosSize.which_scan ,
	  VosSummary->param_name, & num_param ) ;

	return( TK_SUCCESS ) ;
}

/*
	============================================================
	============================================================

  	    Functions that copy data from C to FORTRAN structures

	============================================================

	name					type

	TK_copy_2byte_metadata_to_FORTRAN	internal
	TK_copy_2byte_to_FORTRAN		internal
	TK_copy_1byte_metadata_to_FORTRAN	internal
	TK_copy_1byte_to_FORTRAN		internal
	
	TK_copy_dates_to_FORTRAN		internal
	
	============================================================
	============================================================
*/

/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void     TK_copy_2byte_metadata_to_FORTRAN(
                L1B_1C_GV               * C_vos ,
                L1_GV_2BYTE             * FORTRAN_vos )

DESCRIPTION:

	This function copies metadata from a C volume scan structure
	into a 2 byte FORTRAN volume scan structure.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
C_vos			a C structure for volume scans


OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
FORTRAN_vos		a FORTRAN structure for volume scans
			with metadata copied from the C structure

RETURN VALUES:

 Mnemonic
 --------
none

FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
none
 
MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
void	 TK_copy_2byte_metadata_to_FORTRAN(
                L1B_1C_GV               * C_vos ,
                L1_GV_2BYTE             * FORTRAN_vos )
{
/* 
 * ---- volume scan metadata
 */
        memcpy( FORTRAN_vos->comments , C_vos->comments, COMMENT_LENGTH ) ;

        memcpy( & FORTRAN_vos->volDes , & C_vos->volDes,
          sizeof( VOLUME_DESCRIPTORS ) ) ;
/* 
 * ---- sensor metadata
 */
        memcpy( & FORTRAN_vos->sensor.radarDesc , 
	        & C_vos->sensor.radarDesc,
	        sizeof( RADAR_DESCRIPTOR ) ) ;

        memcpy( & FORTRAN_vos->sensor.corrFactorDesc , 
	        & C_vos->sensor.corrFactorDesc ,
	        sizeof( CORRECTION_FACTOR_DESCRIPTOR ) ) ;

        memcpy( & FORTRAN_vos->sensor.sweepInfo[ 0 ] , 
	        & C_vos->sensor.sweepInfo[ 0 ],
	        MAX_SWEEP * sizeof( SWEEP_INFO ) ) ;

        memcpy( & FORTRAN_vos->sensor.rayInfoInteger[0][0][0] , 
	        & C_vos->sensor.rayInfoInteger[0][0][0],
	        MAX_SWEEP * MAX_RAY * 7 * sizeof( int32 ) ) ;

        memcpy( & FORTRAN_vos->sensor.rayInfoFloat[0][0][0] , 
	        & C_vos->sensor.rayInfoFloat[0][0][0],
	        MAX_SWEEP * MAX_RAY * 4 * sizeof( float32 ) ) ;

        memcpy( & FORTRAN_vos->sensor.platformInfo[0][0][0] , 
	        & C_vos->sensor.platformInfo[0][0][0],
	        MAX_SWEEP * MAX_RAY * 18 * sizeof( float32 ) ) ;
/*
 * ---- parameter metadata
 */
        memcpy( & FORTRAN_vos->sensor.parm.parmDesc , 
	        & C_vos->sensor.parm[0]->parmDesc ,
	        sizeof( PARAMETER_DESCRIPTOR ) ) ;

        memcpy( & FORTRAN_vos->sensor.parm.cellRangeVector , 
	        & C_vos->sensor.parm[0]->cellRangeVector ,
	        sizeof( CELL_RANGE_VECTOR ) ) ;
}


/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void     TK_copy_2byte_to_FORTRAN(
                L1B_1C_GV               * C_vos ,
                L1_GV_2BYTE             * FORTRAN_vos,
                VosSizeSTRUCT           * VosSize )

DESCRIPTION:

	This function copies a C volume scan structure
	into a 2 byte FORTRAN volume scan structure.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
C_vos			a C structure for volume scans

VosSize			a structure that defines the size of the
			volume scan

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
FORTRAN_vos		a FORTRAN structure for volume scans
			with metadata and data copied from the C structure

RETURN VALUES:

 Mnemonic
 --------
none

FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
none
 
MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/

int     TK_copy_2byte_to_FORTRAN(
                L1B_1C_GV               * C_vos ,
                L1_GV_2BYTE             * FORTRAN_vos,
		VosSizeSTRUCT		* VosSize ) 
{
	int	sweep, ray, cell ;
	int	which_param = VosSize->param_position ;
/*
 * ---- copy the metadata
 */
	TK_copy_2byte_metadata_to_FORTRAN( C_vos, FORTRAN_vos ) ;
/*
 * ---- Before copying the data, verify that the VosSize structure
 *	has valid elements.
 */
	if ( VosSize->param_position == TK_USE_DEFAULT )
	{
	  TKreportWarning( E_TK_GV_MPSP1 ) ;
	  return( 	   E_TK_GV_MPSP1 ) ;
	}

	if ( VosSize->byte_size == 0 )
	{
	  TKreportWarning( E_TK_GV_MPSP2 ) ;
          return( 	   E_TK_GV_MPSP2 ) ;
        }
/*
 * ---- copy the parameter data
 */
	for ( sweep = 0; sweep < VosSize->num_sweep ; sweep ++ )
	for ( ray   = 0; ray   < VosSize->num_ray   ; ray   ++ )
	for ( cell  = 0; cell  < VosSize->num_cell[ which_param ]; cell++ )

	  if ( VosSize->byte_size == 2 )

	    FORTRAN_vos->sensor.parm.data[sweep][ray][cell] =
	      C_vos->sensor.parm[0]->parmData2byte[sweep][ray][cell] ;

	  else

            FORTRAN_vos->sensor.parm.data[sweep][ray][cell] =
              C_vos->sensor.parm[0]->parmData1byte[sweep][ray][cell] ;

}

/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void     TK_copy_2byte_metadata_to_FORTRAN(
                L1B_1C_GV               * C_vos ,
                L1_GV_1BYTE             * FORTRAN_vos )

DESCRIPTION:

	This function copies metadata from a C volume scan structure
	into a 1 byte FORTRAN volume scan structure.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
C_vos			a C structure for volume scans


OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
FORTRAN_vos		a FORTRAN structure for volume scans
			with metadata copied from the C structure

RETURN VALUES:

 Mnemonic
 --------
none

FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
none
 
MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
void	 TK_copy_1byte_metadata_to_FORTRAN(
                L1B_1C_GV               * C_vos ,
                L1_GV_1BYTE             * FORTRAN_vos )
{
/* 
 * ---- volume scan metadata
 */
        memcpy( FORTRAN_vos->comments , C_vos->comments, COMMENT_LENGTH ) ;

        memcpy( & FORTRAN_vos->volDes , & C_vos->volDes,
          sizeof( VOLUME_DESCRIPTORS ) ) ;
/* 
 * ---- sensor metadata
 */
        memcpy( & FORTRAN_vos->sensor.radarDesc , 
	        & C_vos->sensor.radarDesc,
	        sizeof( RADAR_DESCRIPTOR ) ) ;

        memcpy( & FORTRAN_vos->sensor.corrFactorDesc , 
	        & C_vos->sensor.corrFactorDesc ,
	        sizeof( CORRECTION_FACTOR_DESCRIPTOR ) ) ;

        memcpy( & FORTRAN_vos->sensor.sweepInfo[ 0 ] , 
	        & C_vos->sensor.sweepInfo[ 0 ],
	        MAX_SWEEP * sizeof( SWEEP_INFO ) ) ;

        memcpy( & FORTRAN_vos->sensor.rayInfoInteger[0][0][0] , 
	        & C_vos->sensor.rayInfoInteger[0][0][0],
	        MAX_SWEEP * MAX_RAY * 7 * sizeof( int32 ) ) ;

        memcpy( & FORTRAN_vos->sensor.rayInfoFloat[0][0][0] , 
	        & C_vos->sensor.rayInfoFloat[0][0][0],
	        MAX_SWEEP * MAX_RAY * 4 * sizeof( float32 ) ) ;

        memcpy( & FORTRAN_vos->sensor.platformInfo[0][0][0] , 
	        & C_vos->sensor.platformInfo[0][0][0],
	        MAX_SWEEP * MAX_RAY * 18 * sizeof( float32 ) ) ;
/*
 * ---- parameter metadata
 */
        memcpy( & FORTRAN_vos->sensor.parm.parmDesc , 
	        & C_vos->sensor.parm[0]->parmDesc ,
	        sizeof( PARAMETER_DESCRIPTOR ) ) ;

        memcpy( & FORTRAN_vos->sensor.parm.cellRangeVector , 
	        & C_vos->sensor.parm[0]->cellRangeVector ,
	        sizeof( CELL_RANGE_VECTOR ) ) ;
}


/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void     TK_copy_1byte_to_FORTRAN(
                L1B_1C_GV               * C_vos ,
                L1_GV_1BYTE             * FORTRAN_vos,
                VosSizeSTRUCT           * VosSize )

DESCRIPTION:

	This function copies a C volume scan structure
	into a 1 byte FORTRAN volume scan structure.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
C_vos			a C structure for volume scans

VosSize			a structure that defines the size of the
			volume scan

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
FORTRAN_vos		a FORTRAN structure for volume scans
			with metadata and data copied from the C structure

RETURN VALUES:

 Mnemonic
 --------
none

FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
none
 
MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/

int     TK_copy_1byte_to_FORTRAN(
                L1B_1C_GV               * C_vos ,
                L1_GV_1BYTE             * FORTRAN_vos,
		VosSizeSTRUCT		* VosSize ) 
{
	int	sweep, ray, cell ;
	int	which_param = VosSize->param_position ;
	int16	a_2byte_value ;
/*
 * ---- copy the metadata
 */
	TK_copy_1byte_metadata_to_FORTRAN( C_vos, FORTRAN_vos ) ;
/*
 * ---- Before copying the data, verify that the VosSize structure
 *	has valid elements.
 */
	if ( VosSize->param_position == TK_USE_DEFAULT )
	{
	  TKreportWarning( E_TK_GV_MPSP1 ) ;
	  return( 	   E_TK_GV_MPSP1 ) ;
	}

	if ( VosSize->byte_size == 0 )
	{
	  TKreportWarning( E_TK_GV_MPSP2 ) ;
          return( 	   E_TK_GV_MPSP2 ) ;
        }

	if ( VosSize->byte_size == 2 )
	{
	  TKreportWarning( E_TK_GV_1B2B ) ;
          return(          E_TK_GV_1B2B ) ;
        }
/*
 * ---- copy the parameter data
 *
 *	If the data is 2 byte and the user has requested a 1 byte
 *	FORTRAN structure, then force the 2 byte value to be within
 *	the valid range of a 1 byte integer using the ONE_BYTE macro.
 */
	for ( sweep = 0; sweep < VosSize->num_sweep ; sweep ++ )
	for ( ray   = 0; ray   < VosSize->num_ray   ; ray   ++ )
	for ( cell  = 0; cell  < VosSize->num_cell[ which_param ]; cell++ )

	  if ( VosSize->byte_size == 2 )
	  {
	    a_2byte_value = 
  	      C_vos->sensor.parm[0]->parmData2byte[sweep][ray][cell] ;

	    FORTRAN_vos->sensor.parm.data[sweep][ray][cell] =
	      ONE_BYTE( a_2byte_value ) ;
	  }
	  else

            FORTRAN_vos->sensor.parm.data[sweep][ray][cell] =
              C_vos->sensor.parm[0]->parmData1byte[sweep][ray][cell] ;

}


/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void     TK_copy_dates_to_FORTRAN(
                DATE_TIME_ARRAY_STR     * in_FOR,
                struct tm               in_C[ MAX_VOS ] ,
                int                     num_date )

DESCRIPTION:

	This function copies a C date array
	into a FORTRAN date array structure.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
in_C			a C date array

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
in_FOR			a FORTRAN date array

RETURN VALUES:

 Mnemonic
 --------
none

FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
none
 
MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/

void     TK_copy_dates_to_FORTRAN(
                DATE_TIME_ARRAY_STR 	* in_FOR,
                struct tm       	in_C[ MAX_VOS ] ,
		int			num_date )
{
	int	which_date ;

	for ( which_date = 0 ; which_date < num_date ; which_date ++ )
	{
	  in_FOR->tkyear[   which_date ] = in_C[ which_date ].tm_year +1900;
	  in_FOR->tkmonth[  which_date ] = in_C[ which_date ].tm_mon  + 1 ;
	  in_FOR->tkday[    which_date ] = in_C[ which_date ].tm_mday ;

	  in_FOR->tkhour[   which_date ] = in_C[ which_date ].tm_hour ;
	  in_FOR->tkminute[ which_date ] = in_C[ which_date ].tm_min ;
	  in_FOR->tksecond[ which_date ] = in_C[ which_date ].tm_sec ;
	}

	in_FOR->num_date	= num_date ;
}

/*
	============================================================
	============================================================

  	    Low level functions to read volume scan data

	============================================================

	name				type

	TK_read_1c51_1b51_in_C		internal
	
	copyGV_L1_vosDes		internal
	copyGV_L1_radarDes		internal
	copyGV_L1_corrFactorDes		internal
	copyGV_L1_sweepInfo		internal
	copyGV_L1_paramDesc		internal

	readGV_L1B_1C_metadata		internal
	readGV_L1B_1C_param		internal
	readGV_L1B_1C_subset		internal

	TKreadL1GVparamPos		internal
	TKreadL1GVparamName		internal
	
	============================================================
	============================================================
*/

/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int	TK_read_1c51_1b51_in_C( IO_HANDLE * handle,
                L1B_1C_GV               ** volume_scan ,
                VosSizeSTRUCT                 * VosSize ,
                char                    algorithm_ID[ TK_ALGORITHM_ID_SIZE ] )

DESCRIPTION:

This function determines the size of a 1c51 or 1b51 volume scan,
allocates the C structure for the volume scan, and reads the
scan from the granule.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
handle			a granule handle that has been previously used
			with TKopen to open the granule.


OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
volume_scan		the 1b51 or 1c51 volume scan

VosSize		the dimensions of the volume scan

algorithm_ID		the algorithm_ID of the volume scan,
			equal to "1B51" or "1C51" if a valid granule
			is used.  Otherwise, the algorithm_ID may
			have some other value, and no volume scan is read.

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if no errors are detected

E_TK_L1_GV		if the algorithm_ID is invalid and a level 1 GV
			volume scan cannot be read

status			an error status returned from a subfunction
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
TKreadMetadataChar		reads the algorithm_ID from the granule

TKreadL1GVsize	determines the size of the volume scan

TKallocateL1GV		allocates the volume scan
 
TKreadL1GV			reads the volume scan

UTreportWarning			reports a warning or error message

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 9/3/97     O. Kelley        implemented code
 
-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/

int	TK_read_1c51_1b51_in_C( IO_HANDLE * handle,
                L1B_1C_GV               ** volume_scan ,
                VosSizeSTRUCT                 * VosSize ,
                char                    algorithm_ID[ TK_ALGORITHM_ID_SIZE ] ,
                char                    param_name[ ],
                int                     read_only_bottom_sweep )
{
	int		status ;
/*
 * ---- determine the algorithm_ID
 */
	status		= TKreadMetadataChar( handle ,
	  TK_ALGORITHM_ID, algorithm_ID ) ;

	if ( status != TK_SUCCESS )
        {
          TKreportWarning( E_TK_ALG_ID ) ;
          return( 	   E_TK_ALG_ID ) ;
        }

	if ( ( strcmp( algorithm_ID, "1C51" ) == 0 ) ||
	     ( strcmp( algorithm_ID, "1B51" ) == 0 )  )
	{
/*
 * ------ determine the size of the level 1 volume scan
 *	  under the assumption that all parameters are being read.
 */
	  status = TKreadL1GVsize( handle , VosSize ) ;

	  if ( status != TK_SUCCESS )
	  {
	    TKreportWarning( status ) ;
	    return( 	     status ) ;
	  }
/*
 * ------ alter the values in the VosSize structure if only
 *	  one parameter is being read.
 */
	  if ( strcmp( param_name , TK_READ_ALL_PARAM ) != 0 )
	  {
	    VosSize->num_param      = 1 ;

	    /* Determine the location of the parameter
 	    This is important only in 1B51 files, which have different
 	    numbers of cells for reflectivity and velocity parameters. */

            status  = TKreadL1GVparamPos( handle, VosSize->which_scan, 
	      param_name, & VosSize->param_position ) ;

	    if ( status != TK_SUCCESS )
	    {
	      TKreportWarning( status ) ;
	      return(          status ) ;
	    }

            if ( read_only_bottom_sweep == TK_TRUE )

              VosSize->num_sweep   = 1 ;

            if ( strstr( param_name, "QCM" ) != NULL )

              VosSize->byte_size        = 1 ;
            else
              VosSize->byte_size        = 2 ;
	  }
/*
 * ------ allocate the level 1 volume scan using the Volume scan's size
 */
	  status = TKallocateL1GV( VosSize , volume_scan ) ;

	  if ( status != TK_SUCCESS )
          {
            TKreportWarning( status ) ;
            return( 	     status ) ;
          }
/*
 * ------ read the level 1 volume scan into the allocated structure
 */
	  if ( strcmp( param_name , TK_READ_ALL_PARAM ) == 0 )

            status = TKreadL1GV( handle , * volume_scan ) ;

	  else

            status = readGV_L1B_1C_subset( handle , * volume_scan,
              VosSize->productID, param_name, read_only_bottom_sweep ) ;

	  if ( status != TK_SUCCESS )
          {
            TKreportWarning( status ) ;
            return( 	     status ) ;
          }
	}
	else
	{
/*
 * ------ If the algorithm_ID is not level 1 GV, then return an error.
 */
	  TKreportWarning( E_TK_GV_L1_PROD ) ;
	  return( 	   E_TK_GV_L1_PROD ) ;
	}

	return( TK_SUCCESS ) ;
}


/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void  copyGV_L1_vosDes( L1B_1C_GV *sL1b1c , uint8 *ptr )

DESCRIPTION:

	To copy into the volume scan structure the 
	volume descriptor read from an HDF file.

INPUT VARIABLE:    ptr		a pointer to the volume descriptor

OUTPUT VARIABLES:  sL1b1c	a volume scan

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
*/

void  copyGV_L1_vosDes( L1B_1C_GV *sL1b1c , uint8 *ptr )
{
      memcpy(&sL1b1c->volDes.verNum, ptr, sizeof(sL1b1c->volDes.verNum));
      ptr += sizeof(sL1b1c->volDes.verNum);

      memcpy(&sL1b1c->volDes.volNum, ptr, sizeof(sL1b1c->volDes.volNum));
      ptr += sizeof(sL1b1c->volDes.volNum);

      memcpy(&sL1b1c->volDes.sizeDataRec, ptr, 
	sizeof(sL1b1c->volDes.sizeDataRec));
      ptr += sizeof(sL1b1c->volDes.sizeDataRec);

      memcpy(&sL1b1c->volDes.projectName, ptr, 
	sizeof(sL1b1c->volDes.projectName));
      ptr += sizeof(sL1b1c->volDes.projectName);

      memcpy(&sL1b1c->volDes.year, ptr, sizeof(sL1b1c->volDes.year));
      ptr += sizeof(sL1b1c->volDes.year);

      memcpy(&sL1b1c->volDes.month, ptr, sizeof(sL1b1c->volDes.month));
      ptr += sizeof(sL1b1c->volDes.month);

      memcpy(&sL1b1c->volDes.day, ptr, sizeof(sL1b1c->volDes.day));
      ptr += sizeof(sL1b1c->volDes.day);

      memcpy(&sL1b1c->volDes.hour, ptr, sizeof(sL1b1c->volDes.hour));
      ptr += sizeof(sL1b1c->volDes.hour);

      memcpy(&sL1b1c->volDes.minute, ptr, sizeof(sL1b1c->volDes.minute));
      ptr += sizeof(sL1b1c->volDes.minute);

      memcpy(&sL1b1c->volDes.second, ptr, sizeof(sL1b1c->volDes.second));
      ptr += sizeof(sL1b1c->volDes.second);

      memcpy(&sL1b1c->volDes.flightNum, ptr, sizeof(sL1b1c->volDes.flightNum));
      ptr += sizeof(sL1b1c->volDes.flightNum);

      memcpy(&sL1b1c->volDes.facName, ptr, sizeof(sL1b1c->volDes.facName));
      ptr += sizeof(sL1b1c->volDes.facName);

      memcpy(&sL1b1c->volDes.genYear, ptr, sizeof(sL1b1c->volDes.genYear));
      ptr += sizeof(sL1b1c->volDes.genYear);

      memcpy(&sL1b1c->volDes.genMonth, ptr, sizeof(sL1b1c->volDes.genMonth));
      ptr += sizeof(sL1b1c->volDes.genMonth);

      memcpy(&sL1b1c->volDes.genDay, ptr, sizeof(sL1b1c->volDes.genDay));
      ptr += sizeof(sL1b1c->volDes.genDay);

      memcpy(&sL1b1c->volDes.numSensorDesc,ptr,
	sizeof(sL1b1c->volDes.numSensorDesc));
}


/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void  copyGV_L1_radarDes( L1B_1C_GV *sL1b1c , uint8 *ptr )

DESCRIPTION:

	To copy into the volume scan structure the 
	radar descriptor read from an HDF file.

INPUT VARIABLE:    ptr		a pointer to the radar descriptor

OUTPUT VARIABLES:  sL1b1c	a volume scan

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
*/

void  copyGV_L1_radarDes( L1B_1C_GV *sL1b1c , uint8 *ptr )
{
      memcpy(&sL1b1c->sensor.radarDesc.radarName, ptr,
             sizeof(sL1b1c->sensor.radarDesc.radarName));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarName);
      memcpy(&sL1b1c->sensor.radarDesc.radarConstant, ptr,
             sizeof(sL1b1c->sensor.radarDesc.radarConstant));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarConstant);
      memcpy(&sL1b1c->sensor.radarDesc.nomPeakPower, ptr,
             sizeof(sL1b1c->sensor.radarDesc.nomPeakPower));
      ptr += sizeof(sL1b1c->sensor.radarDesc.nomPeakPower);
      memcpy(&sL1b1c->sensor.radarDesc.nomNoisePower, ptr,
             sizeof(sL1b1c->sensor.radarDesc.nomNoisePower));
      ptr += sizeof(sL1b1c->sensor.radarDesc.nomNoisePower);
      memcpy(&sL1b1c->sensor.radarDesc.receiverGain, ptr,
             sizeof(sL1b1c->sensor.radarDesc.receiverGain));
      ptr += sizeof(sL1b1c->sensor.radarDesc.receiverGain);
      memcpy(&sL1b1c->sensor.radarDesc.antennaGain, ptr,
             sizeof(sL1b1c->sensor.radarDesc.antennaGain));
      ptr += sizeof(sL1b1c->sensor.radarDesc.antennaGain);
      memcpy(&sL1b1c->sensor.radarDesc.radarSystemGain, ptr,
             sizeof(sL1b1c->sensor.radarDesc.radarSystemGain));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarSystemGain);
      memcpy(&sL1b1c->sensor.radarDesc.horBeamWidth, ptr,
             sizeof(sL1b1c->sensor.radarDesc.horBeamWidth));
      ptr += sizeof(sL1b1c->sensor.radarDesc.horBeamWidth);
      memcpy(&sL1b1c->sensor.radarDesc.verBeamWidth, ptr,
             sizeof(sL1b1c->sensor.radarDesc.verBeamWidth));
      ptr += sizeof(sL1b1c->sensor.radarDesc.verBeamWidth);
      memcpy(&sL1b1c->sensor.radarDesc.radarType, ptr,
             sizeof(sL1b1c->sensor.radarDesc.radarType));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarType);
      memcpy(&sL1b1c->sensor.radarDesc.scanMode, ptr,
             sizeof(sL1b1c->sensor.radarDesc.scanMode));
      ptr += sizeof(sL1b1c->sensor.radarDesc.scanMode);
      memcpy(&sL1b1c->sensor.radarDesc.nomScanRate, ptr,
             sizeof(sL1b1c->sensor.radarDesc.nomScanRate));
      ptr += sizeof(sL1b1c->sensor.radarDesc.nomScanRate);
      memcpy(&sL1b1c->sensor.radarDesc.nomStartAngle, ptr,
             sizeof(sL1b1c->sensor.radarDesc.nomStartAngle));
      ptr += sizeof(sL1b1c->sensor.radarDesc.nomStartAngle);
      memcpy(&sL1b1c->sensor.radarDesc.nomStopAngle, ptr,
             sizeof(sL1b1c->sensor.radarDesc.nomStopAngle));
      ptr += sizeof(sL1b1c->sensor.radarDesc.nomStopAngle);
      memcpy(&sL1b1c->sensor.radarDesc.numParmDesc, ptr,
             sizeof(sL1b1c->sensor.radarDesc.numParmDesc));
      ptr += sizeof(sL1b1c->sensor.radarDesc.numParmDesc);
      memcpy(&sL1b1c->sensor.radarDesc.numDesc, ptr,
             sizeof(sL1b1c->sensor.radarDesc.numDesc));
      ptr += sizeof(sL1b1c->sensor.radarDesc.numDesc);
      memcpy(&sL1b1c->sensor.radarDesc.dataComp, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.dataComp));
      ptr += sizeof(sL1b1c->sensor.radarDesc.dataComp);
      memcpy(&sL1b1c->sensor.radarDesc.dataReductAlg, ptr,
             sizeof(sL1b1c->sensor.radarDesc.dataReductAlg));
      ptr += sizeof(sL1b1c->sensor.radarDesc.dataReductAlg);
      memcpy(&sL1b1c->sensor.radarDesc.dataReductParm1, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.dataReductParm1));
      ptr += sizeof(sL1b1c->sensor.radarDesc.dataReductParm1);
      memcpy(&sL1b1c->sensor.radarDesc.dataReductParm2, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.dataReductParm2));
      ptr += sizeof(sL1b1c->sensor.radarDesc.dataReductParm2);
      memcpy(&sL1b1c->sensor.radarDesc.radarLon, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.radarLon));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarLon);
      memcpy(&sL1b1c->sensor.radarDesc.radarLat, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.radarLat));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarLat);
      memcpy(&sL1b1c->sensor.radarDesc.radarAlt, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.radarAlt));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarAlt);
      memcpy(&sL1b1c->sensor.radarDesc.velocity, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.velocity));
      ptr += sizeof(sL1b1c->sensor.radarDesc.velocity);
      memcpy(&sL1b1c->sensor.radarDesc.range, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.range));
      ptr += sizeof(sL1b1c->sensor.radarDesc.range);
      memcpy(&sL1b1c->sensor.radarDesc.numTransfreqency, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.numTransfreqency));
      ptr += sizeof(sL1b1c->sensor.radarDesc.numTransfreqency);
      memcpy(&sL1b1c->sensor.radarDesc.numInterPulsePeriods, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.numInterPulsePeriods));
      ptr += sizeof(sL1b1c->sensor.radarDesc.numInterPulsePeriods);
      memcpy(&sL1b1c->sensor.radarDesc.frequency1, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.frequency1));
      ptr += sizeof(sL1b1c->sensor.radarDesc.frequency1);
      memcpy(&sL1b1c->sensor.radarDesc.frequency2, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.frequency2));
      ptr += sizeof(sL1b1c->sensor.radarDesc.frequency2);
      memcpy(&sL1b1c->sensor.radarDesc.frequency3, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.frequency3));
      ptr += sizeof(sL1b1c->sensor.radarDesc.frequency3);
      memcpy(&sL1b1c->sensor.radarDesc.frequency4, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.frequency4));
      ptr += sizeof(sL1b1c->sensor.radarDesc.frequency4);
      memcpy(&sL1b1c->sensor.radarDesc.frequency5, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.frequency5));
      ptr += sizeof(sL1b1c->sensor.radarDesc.frequency5);
      memcpy(&sL1b1c->sensor.radarDesc.interPulsePeriod1, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod1));
      ptr += sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod1);
      memcpy(&sL1b1c->sensor.radarDesc.interPulsePeriod2, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod2));
      ptr += sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod2);
      memcpy(&sL1b1c->sensor.radarDesc.interPulsePeriod3, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod3));
      ptr += sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod3);
      memcpy(&sL1b1c->sensor.radarDesc.interPulsePeriod4, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod4));
      ptr += sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod4);
      memcpy(&sL1b1c->sensor.radarDesc.interPulsePeriod5, ptr,
             sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod5)); 
}

/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void  copyGV_L1_corrFactorDes( L1B_1C_GV *sL1b1c , uint8 *ptr )

DESCRIPTION:

	To copy into the volume scan structure the 
	correction factor read from an HDF file.

INPUT VARIABLE:    ptr		a pointer to the correction factor

OUTPUT VARIABLES:  sL1b1c	a volume scan

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
*/

void  copyGV_L1_corrFactorDes( L1B_1C_GV *sL1b1c , uint8 *ptr )
{
      memcpy(&sL1b1c->sensor.corrFactorDesc.azimuth, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.azimuth));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.azimuth);
      memcpy(&sL1b1c->sensor.corrFactorDesc.elevation, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.elevation));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.elevation);
      memcpy(&sL1b1c->sensor.corrFactorDesc.rangeDelay, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.rangeDelay));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.rangeDelay);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarLon, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.radarLon));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarLon);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarLat, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.radarLat));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarLat);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPressAlt, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.radarPressAlt));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPressAlt);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarAltAboveGround, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.radarAltAboveGround));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarAltAboveGround);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedEW, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedEW));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedEW);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedNS, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedNS));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedNS);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPlatVerticalVelocity, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatVerticalVelocity));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatVerticalVelocity);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPlatHeading, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatHeading));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatHeading);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPlatRoll, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatRoll));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatRoll);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPlatPitch, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatPitch));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatPitch);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPlatDrift, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatDrift));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatDrift);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarRotAngle, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.radarRotAngle));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarRotAngle);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarTiltAngle, ptr,
        sizeof(sL1b1c->sensor.corrFactorDesc.radarTiltAngle));
}

/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void  copyGV_L1_sweepInfo( L1B_1C_GV *sL1b1c , uint8 *ptr )

DESCRIPTION:

	To copy into the volume scan structure the 
	sweep information read from an HDF file.

INPUT VARIABLE:    ptr		a pointer to the sweep information

OUTPUT VARIABLES:  sL1b1c	a volume scan

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
*/

void  copyGV_L1_sweepInfo( L1B_1C_GV *sL1b1c , uint8 *ptr, int32 i )
{
           memcpy(&sL1b1c->sensor.sweepInfo[i].radarName, ptr,
                  sizeof(sL1b1c->sensor.sweepInfo[i].radarName));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].radarName);
           memcpy(&sL1b1c->sensor.sweepInfo[i].sweepNum, ptr,
                  sizeof(sL1b1c->sensor.sweepInfo[i].sweepNum));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].sweepNum);
           memcpy(&sL1b1c->sensor.sweepInfo[i].numRays, ptr,
                  sizeof(sL1b1c->sensor.sweepInfo[i].numRays));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].numRays);
           memcpy(&sL1b1c->sensor.sweepInfo[i].trueStartAngle, ptr,
                  sizeof(sL1b1c->sensor.sweepInfo[i].trueStartAngle));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].trueStartAngle);
           memcpy(&sL1b1c->sensor.sweepInfo[i].trueStopAngle, ptr,
                  sizeof(sL1b1c->sensor.sweepInfo[i].trueStopAngle));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].trueStopAngle);
           memcpy(&sL1b1c->sensor.sweepInfo[i].fixedAngle, ptr,
                  sizeof(sL1b1c->sensor.sweepInfo[i].fixedAngle));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].fixedAngle);
           memcpy(&sL1b1c->sensor.sweepInfo[i].filterFlag, ptr,
                  sizeof(sL1b1c->sensor.sweepInfo[i].filterFlag));
}

/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void  copyGV_L1_paramDesc( L1B_1C_GV *sL1b1c , uint8 *ptr )

DESCRIPTION:

	To copy into the volume scan structure the 
	parameter descriptor read from an HDF file.

INPUT VARIABLE:    ptr		a pointer to the parameter descriptor

OUTPUT VARIABLES:  sL1b1c	a volume scan

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
*/

void  copyGV_L1_paramDesc( L1B_1C_GV *sL1b1c , uint8 *ptr, int32 i )
{
        ptr += sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.parmName);

        memcpy(&sL1b1c->sensor.parm[ i ]->parmDesc.parmDesc, ptr,
           sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.parmDesc));
        ptr += sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.parmDesc);

        memcpy(&sL1b1c->sensor.parm[ i ]->parmDesc.parmUnits, ptr,
           sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.parmUnits));
        ptr += sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.parmUnits);

        memcpy(&sL1b1c->sensor.parm[ i ]->parmDesc.interPulsePeriod, ptr,
           sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.interPulsePeriod));
        ptr += sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.interPulsePeriod);

        memcpy(&sL1b1c->sensor.parm[ i ]->parmDesc.transFreq, ptr,
           sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.transFreq));
        ptr += sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.transFreq);

        memcpy(&sL1b1c->sensor.parm[ i ]->parmDesc.receiverBandwidth, ptr,
      	   sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.receiverBandwidth));
        ptr += sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.receiverBandwidth);

        memcpy(&sL1b1c->sensor.parm[ i ]->parmDesc.pulseWidth, ptr,
           sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.pulseWidth));
        ptr += sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.pulseWidth);

        memcpy(&sL1b1c->sensor.parm[ i ]->parmDesc.polarTransWave, ptr,
           sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.polarTransWave));
        ptr += sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.polarTransWave);

        memcpy(&sL1b1c->sensor.parm[ i ]->parmDesc.numOfsamples, ptr,
           sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.numOfsamples));
        ptr += sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.numOfsamples);

        memcpy(&sL1b1c->sensor.parm[ i ]->parmDesc.parmDataType, ptr,
           sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.parmDataType));
        ptr += sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.parmDataType);

        memcpy(&sL1b1c->sensor.parm[ i ]->parmDesc.thresholdField, ptr,
           sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.thresholdField));
        ptr += sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.thresholdField);

        memcpy(&sL1b1c->sensor.parm[ i ]->parmDesc.thresholdValue, ptr,
           sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.thresholdValue));
        ptr += sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.thresholdValue);

        memcpy(&sL1b1c->sensor.parm[ i ]->parmDesc.scaleFactor, ptr,
           sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.scaleFactor));
        ptr += sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.scaleFactor);

        memcpy(&sL1b1c->sensor.parm[ i ]->parmDesc.offsetFactor, ptr,
           sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.offsetFactor));
        ptr += sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.offsetFactor);

      memcpy(&sL1b1c->sensor.parm[ i ]->parmDesc.deletedOrMissDataFlag, ptr,
        sizeof(sL1b1c->sensor.parm[ i ]->parmDesc.deletedOrMissDataFlag));
}


/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void readGV_L1B_1C_metadata(IO_HANDLE *sGranuleHandle, L1B_1C_GV *sL1b1c,
        int productID, VGROUP_LINK *currVgroup ,
        uint8 **DBL_databuf , int * DBL_bufsize ,
        int32 num_cell[ MAX_PARM ], int32 * ptr_num_ray,
        int32 * ptr_num_sweep )

DESCRIPTION:

	This function reads volume scan metadata for a variety
	of API's, including:

	  TKreadL1GVbyVosNum	(in C)
	  TKreadL1GVparam	(in C and FORTRAN)
	  TKreadL1GV1byteParam  (in FORTRAN)
	  TKreadL1GVbaseSweep	(in C and FORTRAN)

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
sGranuleHandle		IO handle for a granule that has been previously
			opened with TKopen.

productID		(not used)

currVgroup		contains the HDF pointers

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
sL1b1c			a level 1 GV volume scan

DBL_databuf		a double pointer to the databuffer
DBL_bufsize		a pointer to the buffer size
num_cell		the number of cells in rays of the volume scan
ptr_num_ray		a pointer to the number of rays in the volume scan
ptr_num_sweep		a pointer to the number of sweeps in the volume scan

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
E_TK_MEMFAILREADL1GV	
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
TKgetNray
TKgetNsweep
TKgetNparm
TKgetNcell
TKreportError

vdataRead
copyGV_L1_vosDes
copyGV_L1_radarDes
copyGV_L1_corrFactorDes
copyGV_L1_sweepInfo
sdsRead

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
void readGV_L1B_1C_metadata(IO_HANDLE *sGranuleHandle, L1B_1C_GV *sL1b1c, 
	VGROUP_LINK *currVgroup , uint8 **DBL_databuf , int * DBL_bufsize , 
	int32 num_cell[ MAX_PARM ], int32 * ptr_num_ray, 
	int32 * ptr_num_sweep )
{
  int scaleFactor;
  SDS_LINK 	*tempSDS;
  VDATA_LINK 	*tempVdata;
  VDATA_LINK 	*tempVdata1;
  VGROUP_LINK 	*parmVgroup;
  int32 	num_param, num_ray, num_sweep;
  uint8 	*databuf, *ptr;
  int32 	i, j, k;
  int           max_num_cell = 0;  /* Max. number of cells per ray */
  int           bufsize;

  /* unix starts with 0 as the number of the first scan,
  while the handle starts with 1 as the number of
  the first scan. */

  int	zero_based_num_scan = sGranuleHandle->currNumOfScanl - 1 ;

/*
 * Get the number of ray, number of sweeps, number of parameters 
 * and number of cells in order to read sL1b1c data in to HDF file.
 */
  if (currVgroup != NULL)
  {
      num_ray   = TKgetNray(sGranuleHandle,   zero_based_num_scan ) ;
      num_sweep = TKgetNsweep(sGranuleHandle, zero_based_num_scan ) ;
      num_param  = TKgetNparm(sGranuleHandle,  zero_based_num_scan ) ;

	* ptr_num_ray	= num_ray ;
	* ptr_num_sweep	= num_sweep ;

      for (i = 0; i < num_param; ++i)
      {
        num_cell[i] = TKgetNcell(sGranuleHandle, zero_based_num_scan , i ) ;
	if (num_cell[i] > max_num_cell)
                max_num_cell = num_cell[i];
      }
/*
 * 	Allocate memory for the (hopefully) largest required buffer
 * 	we will need when packing the VOS components
 */
      bufsize = 2 * max_num_cell * num_ray * num_sweep;
      if (bufsize < MINIMAL_BUFFER_SIZE)
          bufsize = MINIMAL_BUFFER_SIZE;
      databuf = (uint8 *)calloc(1, bufsize);

      if (databuf == NULL)
	/* Memory allocation failed in readGV_L1B_1C(). */
	TKreportError(E_TK_MEMFAILREADL1GV);
/* 
 * 	Read comments from the HDF file.
 */
      tempVdata = currVgroup->vdata;

      ptr = databuf ;
      vdataRead(&tempVdata, ptr);

      memcpy(&sL1b1c->comments, ptr, sizeof (sL1b1c->comments));
/*
 * 	Read volume descriptor from the HDF file.
 */
      bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
      ptr = databuf;
      vdataRead(&tempVdata, ptr);

	copyGV_L1_vosDes( sL1b1c , ptr ) ;
/*
 * 	Read Sensor information from the HDF file.
 */
      tempSDS = currVgroup->subVgroup->sds;
      tempVdata = currVgroup->subVgroup->vdata;

      bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
      ptr = databuf;
      vdataRead(&tempVdata, ptr);

	copyGV_L1_radarDes( sL1b1c , ptr ) ;

      bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
      ptr = databuf;
      vdataRead(&tempVdata, ptr);

	copyGV_L1_corrFactorDes( sL1b1c , ptr ) ;

      bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
      tempVdata1 = tempVdata;
      for (i = 0; i < num_sweep; ++i)
      {
	   tempVdata = tempVdata1;
           ptr = databuf;
           vdataRead(&tempVdata, ptr);

	   copyGV_L1_sweepInfo( sL1b1c , ptr, i ) ;
      }
/*
 * 	Allocate memory to read packed ray info integer information from 
 * 	the HDF file. 4 represents the byte size of one ray info integer.
 */
      bufManage(&databuf, &bufsize, (4 * 7 * num_ray * num_sweep));
      sdsRead(&tempSDS, &scaleFactor, databuf);
      ptr = databuf;
      for (i = 0; i < num_sweep; ++i)
      {
           memcpy(&sL1b1c->sensor.rayInfoInteger[i], ptr, (4 * 7 * num_ray));
           ptr += (4 * 7 * num_ray);
      }
/*
 * Allocate memory to read packed ray info float information from
 * the HDF file.  4 represents the byte size of one ray info float.
 */
      bufManage(&databuf, &bufsize, (4 * 4 * num_ray * num_sweep));
      sdsRead(&tempSDS, &scaleFactor, databuf);
      ptr = databuf;
      for (i = 0; i < num_sweep; ++i)
      {
           memcpy(&sL1b1c->sensor.rayInfoFloat[i], ptr, (4 * 4 * num_ray));
           ptr +=  (4 * 4 * num_ray);
      }
/*
 * Allocate memory to read packed platform Info information from
 * the HDF file.  4 represents the byte size of one platform Info.
 */
      bufManage(&databuf, &bufsize, (4 * 18 * num_ray * num_sweep));
      sdsRead(&tempSDS, &scaleFactor, databuf);
      ptr = databuf;
      for (i = 0; i < num_sweep; ++i)
      {
           memcpy(&sL1b1c->sensor.platformInfo[i], ptr, (4 * 18 * num_ray));
           ptr += (4 * 18 * num_ray);
      }

	* DBL_databuf 	= databuf ;
	* DBL_bufsize   = bufsize ;
  }

}


/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

void readGV_L1B_1C_param(IO_HANDLE *sGranuleHandle, L1B_1C_GV *sL1b1c,
          int productID,
          uint8 **DBL_databuf , int * DBL_bufsize ,
          VDATA_LINK    *tempVdata , SDS_LINK      *input_tempSDS,
          int32 num_cell[ MAX_PARM ], int32 * ptr_num_ray,
          int32 * ptr_num_sweep,
          int32 param_position_in_file, uint8 *ptr ,
          int read_only_bottom_sweep )

DESCRIPTION:

	This function reads volume scan parameter data for a variety
	of API's, including:

	  TKreadL1GVbyVosNum	(in C)
	  TKreadL1GVparam	(in C and FORTRAN)
	  TKreadL1GV1byteParam  (in FORTRAN)
	  TKreadL1GVbaseSweep	(in C and FORTRAN)

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
sGranuleHandle		IO handle for a granule that has been previously
			opened with TKopen.

productID		identifies the granule as coming from 1C51 or 1B51

DBL_databuf		a double pointer to the databuffer
DBL_bufsize		a pointer to the buffer size

tempVdata		an HDF pointer to a V group
input_tempSDS		and HDF pointer to a SDS

num_cell		the number of cells in rays of the volume scan
ptr_num_ray		a pointer to the number of rays in the volume scan
ptr_num_sweep		a pointer to the number of sweeps in the volume scan

param_position_in_file	the position of a parameter in a volume scan

ptr			a pointer to a large data buffer

read_only_bottom_sweep	equals TK_TRUE if only the bottom sweep is to
			be read, otherwise it equals TK_FALSE
OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
sL1b1c			a level 1 GV volume scan


RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
vdataRead
sdsRead

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
void readGV_L1B_1C_param(IO_HANDLE *sGranuleHandle, L1B_1C_GV *sL1b1c, 
	  int productID,
	  uint8 **DBL_databuf , int * DBL_bufsize ,
	  VDATA_LINK    *tempVdata , SDS_LINK      *input_tempSDS,
	  int32 num_cell[ MAX_PARM ], int32 * ptr_num_ray, 
	  int32 * ptr_num_sweep,
	  int32 param_position_in_file, uint8 *ptr ,
	  int read_only_bottom_sweep )
{
  int scaleFactor;
  VGROUP_LINK 	*parmVgroup;
  VGROUP_LINK 	*currVgroup;
  SDS_LINK	*tempSDS = input_tempSDS ;
  int32 	num_param ;
  int32		num_ray = * ptr_num_ray ;
  int32		num_sweep = * ptr_num_sweep ;
  uint8 	*databuf = *DBL_databuf ;
  int32 	i = param_position_in_file , j, k;
  int           bufsize = *DBL_bufsize;

  int		which_sweep, bottom_sweep=0, bottom_elevation ;
  int32		num_sweep_to_read	= * ptr_num_sweep ;
/*
 * ---- copy parameter descriptor into L1 GV structure
 */
	copyGV_L1_paramDesc( sL1b1c , ptr, 0 ) ; 
/*
 * ---- Read the packed cell range vector information from databuf.
 */ 
        bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
        ptr = databuf;
        vdataRead(&tempVdata, ptr);

        memcpy(&sL1b1c->sensor.parm[ 0 ]->cellRangeVector.numOfCells, ptr,
          sizeof(sL1b1c->sensor.parm[ 0 ]->cellRangeVector.numOfCells));
        ptr += sizeof(sL1b1c->sensor.parm[ 0 ]->cellRangeVector.numOfCells);

	for (j = 0; j < num_cell[i]; ++j)
	{
           memcpy(
	     &sL1b1c->sensor.parm[ 0 ]->cellRangeVector.distanceToCell[j], 
	     ptr,
	     sizeof(
	     sL1b1c->sensor.parm[ 0 ]->cellRangeVector.distanceToCell[j] ) ) ;

	   ptr += sizeof(
	     sL1b1c->sensor.parm[ 0 ]->cellRangeVector.distanceToCell[j]);
        }
/*
 * ---- set variables in the event only the bottom sweep is
 *	is to be read.
 */
	if ( read_only_bottom_sweep == TK_TRUE )
	{
          bottom_elevation        =
            sL1b1c->sensor.rayInfoFloat[ bottom_sweep ]
            [ ray_zero ][ elevation_index ] ;

          for ( which_sweep = 0 ; which_sweep < num_sweep ; which_sweep ++ )

            if ( sL1b1c->sensor.rayInfoFloat[ which_sweep ]
               [ ray_zero ][ elevation_index ] < bottom_elevation  )
            {
              bottom_sweep        = which_sweep ;
              bottom_elevation    = sL1b1c->sensor.rayInfoFloat[ which_sweep ]
               [ ray_zero ][ elevation_index ] ;
            }

	    tempSDS->start[1]	= bottom_sweep ;
	    tempSDS->edge[1]	= 1 ;
	    num_sweep_to_read	= 1 ;
	}
/*
 * ---- 	   CCC   1C51   CCC
 *
 *	1C51 files alternate 1 byte mask parameters with
 *	2 byte data parameters.  This is the way the TRMM office
 *	radar science library works, and all of TSDIS copied that
 *	approach.
 */
        if (productID == TK_L1C_GV)
        {
 	  if (( param_position_in_file % 2) == 0)
	  {
            bufManage(&databuf, &bufsize, 
	      (1 * num_cell[i] * num_ray * num_sweep));

            sdsRead(&tempSDS, &scaleFactor, databuf);
            ptr = databuf;

            for (j = 0; j < num_sweep_to_read ; ++j)
            {
              for (k = 0; k < num_ray; ++k)
              {
	        memcpy ( ( uint8 * ) 
		  sL1b1c->sensor.parm[ 0 ]->parmData1byte[j][k],
	          ptr, (1 * num_cell[i]));

                ptr += (1 * num_cell[i]);
              }
            }
	  }
	  else
	  {
            bufManage(&databuf, &bufsize, 
	      (2 * num_cell[i] * num_ray * num_sweep ));

            sdsRead(&tempSDS, &scaleFactor, databuf);
            ptr = databuf;

            for (j = 0; j < num_sweep_to_read ; ++j)
            {
              for (k = 0; k < num_ray; ++k)
              {
	        memcpy( (uint16 * ) 
	          sL1b1c->sensor.parm[ 0 ]->parmData2byte[j][k],
	          ptr, (2 * num_cell[i]));

                ptr += (2 * num_cell[i]);
              }
            }
	  }
	}
/*
 * ---- 	BBB   1B51   BBB
 *
 *	1B51 files all parameters are 2 bytes.
 *	This is the way the TRMM office radar science library works,
 *	and all of TSDIS copied that approach.i
 */
        if (productID == TK_L1B_GV)
        {
          bufManage(&databuf, &bufsize, 
	    (2 * num_cell[i] * num_ray * num_sweep ));

          sdsRead(&tempSDS, &scaleFactor, databuf);
          ptr = databuf;

          for (j = 0; j < num_sweep_to_read ; ++j)
          {
            for (k = 0; k < num_ray; ++k)
            {
	      memcpy( ( uint16 * ) 
	 	sL1b1c->sensor.parm[ 0 ]->parmData2byte[j][k],
	        ptr, (2 * num_cell[i]));

              ptr += (2 * num_cell[i]);
            }
          }
	}
/*
 * ---- reset variables in the event only the bottom sweep is
 *	has been read.  Since reading the SDS increments to
 *	the next element of the linked list, you must explicitly
 *	use the input SDS here.
 */
	if ( read_only_bottom_sweep == TK_TRUE )
	{
	  input_tempSDS->start[1]	= 0 ;
	  input_tempSDS->edge[1]	= num_sweep ;
	}
}

/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int readGV_L1B_1C_subset(
        IO_HANDLE *sGranuleHandle,
        L1B_1C_GV *sL1b1c, int productID,
        char target_param_name[ ], int read_only_bottom_sweep )

DESCRIPTION:

	This function reads volume scan metadata and parameter data
	for a variety of API's, including:

	  TKreadL1GVbyVosNum	(in C)
	  TKreadL1GVparam	(in C and FORTRAN)
	  TKreadL1GV1byteParam  (in FORTRAN)
	  TKreadL1GVbaseSweep	(in C and FORTRAN)

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
sGranuleHandle		IO handle for a granule that has been previously
			opened with TKopen.

productID		identifies the granule as coming from 1C51 or 1B51

target_param_name	the name of the parameter to be read from the
			volume scan

read_only_bottom_sweep	equals TK_TRUE if only the bottom sweep is to
			be read, otherwise it equals TK_FALSE

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
sL1b1c			a level 1 GV volume scan


RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
E_TK_GV_NO_SUCH_P
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
getCurrVgroup
readGV_L1B_1C_metadata
vdataRead
readGV_L1B_1C_param
TKreportWarning

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
int readGV_L1B_1C_subset(
	IO_HANDLE *sGranuleHandle, 
	L1B_1C_GV *sL1b1c, int productID,
   	char target_param_name[ ], int read_only_bottom_sweep )
{
  	SDS_LINK 	*tempSDS;
  	VDATA_LINK 	*tempVdata;
  	VGROUP_LINK 	*parmVgroup;
  	VGROUP_LINK 	*currVgroup;
  	int32 		num_cell[MAX_PARM], num_ray, num_sweep;
  	uint8 		*databuf, *ptr;
  	int32		param_position_in_file ;
  	int           	bufsize, status ;
  	char		param_name[ MAX_PARAM_LENGTH ] ;
        int             parameter_was_found = TK_FALSE ;

/*
 * ---- incremenet the counter, as done by TKreadL1GV.  The subfunctions
 *      assume this is done.
 */
        sGranuleHandle->currNumOfScanl ++ ;
/*
 * ---- Get the vgroup position of for next volume scan number to write.
 */
  	currVgroup 	= (VGROUP_LINK *) getCurrVgroup(
	  sGranuleHandle->cfgSTR->vGroup->subVgroup,
          sGranuleHandle->currNumOfScanl);

	readGV_L1B_1C_metadata( sGranuleHandle, sL1b1c, currVgroup,
	  & databuf, & bufsize, num_cell, & num_ray , & num_sweep ) ;
/*
 * ---- Read parameter information from the HDF file.
 */
      	param_position_in_file = 0 ;
      	parmVgroup 	= currVgroup->subVgroup->subVgroup ;

      	while ( ( parmVgroup != NULL              ) &&
		( parameter_was_found == TK_FALSE )  )
      	{
          tempSDS 	= parmVgroup->sds ;
          tempVdata 	= parmVgroup->vdata ;
 
          bufManage( & databuf, & bufsize , MINIMAL_BUFFER_SIZE ) ;
          ptr 		= databuf;
          vdataRead( & tempVdata , ptr ) ;

          memcpy( & sL1b1c->sensor.parm[ 0 ]->parmDesc.parmName , ptr ,
            sizeof( sL1b1c->sensor.parm[ 0 ]->parmDesc.parmName ) ) ;

	  strcpy( param_name , sL1b1c->sensor.parm[ 0 ]->parmDesc.parmName ) ;

	  if ( strcmp( param_name, target_param_name ) == 0 )
	  {
	    readGV_L1B_1C_param( sGranuleHandle, sL1b1c, productID,
	      & databuf, & bufsize, tempVdata, tempSDS,
	      num_cell, & num_ray , & num_sweep, param_position_in_file,
	      ptr, read_only_bottom_sweep ) ;

	    parameter_was_found = TK_TRUE ;
	  }
	  else
	  {
	    parmVgroup = parmVgroup->next;
	    param_position_in_file ++ ;
	  }
      	}

      	free( databuf ) ;

	if ( parameter_was_found == TK_FALSE )
        {
          TKreportWarning( E_TK_GV_NO_SUCH_P ) ;
          return(          E_TK_GV_NO_SUCH_P ) ;
        }
        else
   	  return( TK_SUCCESS ) ;
}


/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TKreadL1GVparamPos(
        IO_HANDLE *sGranuleHandle, int VosNum,
        char target_param_name[ ], int *param_position )

DESCRIPTION:

	This function determines the position of a parameter in
	a volume scan.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
sGranuleHandle		IO handle for a granule that has been previously
			opened with TKopen.

VosNum			the number of the volume scan in which to
			look for the target parameter name

target_param_name	the name of the parameter to be read from the
			volume scan


OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
param_position		the position of the parameter,
			0 means the 1st parameter

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
E_TK_GV_USE_TKOPEN
E_TK_GV_P_EMPTY
E_TK_GV_NO_SUCH_P
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
getCurrVgroup
vdataRead
TKreportWarning

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
int TKreadL1GVparamPos(
	IO_HANDLE *sGranuleHandle, int VosNum, 
	char target_param_name[ ], int *param_position )
{
  	VDATA_LINK 	*tempVdata;
  	VGROUP_LINK 	*parmVgroup;
 	VGROUP_LINK 	*currVgroup;
	uint8           buffer[ MINIMAL_BUFFER_SIZE ] ;
  	int           	status ;
  	char		param_name[ MAX_PARAM_LENGTH ] ;
	int		parameter_was_found = TK_FALSE ;
/*
 * ---- Since this is an internal Toolkit function, only minimal
 *	error checking is needed here.  It is assumed that the
 *	API functions that interface with outside the Toolkit will
 *	have more thoughout error checking.  This will prevent
 *	user errors from making it to this internal function.
 */
        /* check to see that the granule has been openned with TKopen. */

        if ( sGranuleHandle->cfgSTR == NULL )
        {
          TKreportWarning( E_TK_GV_USE_TKOPEN ) ;
          return(          E_TK_GV_USE_TKOPEN ) ;
        }

        /* check that the granule is not an empty granule */

        if ( sGranuleHandle->cfgSTR->vGroup->subVgroup == NULL )
        {
          TKreportWarning( E_TK_GV_P_EMPTY ) ;
          return(          E_TK_GV_P_EMPTY ) ;
        }
/*
 * ---- Get the vgroup position of for next volume scan number to write.
 */
  	currVgroup = (VGROUP_LINK *) getCurrVgroup(
	  sGranuleHandle->cfgSTR->vGroup->subVgroup, VosNum + 1 );
/*
 * ---- initialize variables
 */
      	* param_position = -1 ;
	param_name[ 0 ]	 = '\0' ;
      	parmVgroup = currVgroup->subVgroup->subVgroup;
/*
 * ---- determine the position of the target parameter
 */
      	while ( ( parmVgroup != NULL              ) &&
		( parameter_was_found == TK_FALSE )  )
      	{
	  ( * param_position ) ++ ;

          tempVdata = parmVgroup->vdata;
          vdataRead( & tempVdata , buffer ) ;
          strcpy( param_name , buffer  ) ;
	  parmVgroup = parmVgroup->next ;

	  if ( strcmp( param_name, target_param_name ) == 0 )
	
	    parameter_was_found = TK_TRUE ;

      	}

        /* parameter metadata and data */
       	resetVgroup( currVgroup->subVgroup->subVgroup ,  0 ) ;
/*
 * ---- check that the requested parameter was found in the volume scan
 */
	if ( parameter_was_found == TK_FALSE )
	{
	  TKreportWarning( E_TK_GV_NO_SUCH_P ) ;
	  return( 	   E_TK_GV_NO_SUCH_P ) ;
	}
	else
  	  return( TK_SUCCESS ) ;
}


/* 
-------------------- begin prolog --------------------
FUNCTION STATEMENT:

int TKreadL1GVparamName(
        IO_HANDLE *sGranuleHandle, int VosNum,
        char param_name[ MAX_PARM ][ MAX_PARAM_LENGTH ], int *num_param )

DESCRIPTION:

	This function reads the names of all the parameters in
	a volume scan.

INPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
sGranuleHandle		IO handle for a granule that has been previously
			opened with TKopen.

VosNum			the number of the volume scan in which to
			look for the target parameter name

OUTPUT VARIABLES:

 Parameter Name   	Description
 --------------  	-----------
param_name		a character array of the names of all the parameters
			in the user selected volume scan

num_param		the number of parameters in the volume scan

RETURN VALUES:

 Mnemonic
 --------
TK_SUCCESS		if nothing went wrong
E_TK_GV_USE_TKOPEN
E_TK_GV_P_EMPTY
						
FUNCTIONS CALLED:

 Function Name   		Description
 -------------  		-----------
getCurrVgroup
vdataRead
TKreportWarning

MODIFICATIONS:
 
 Date       Programmer       Description of change
 ----       ----------       ---------------------
 11/13/97   O. Kelley        implemented code

-------------------- end prolog --------------------
 
-------------------- begin notes --------------------

-------------------- end notes --------------------
*/
 
int TKreadL1GVparamName(
	IO_HANDLE *sGranuleHandle, int VosNum, 
	char param_name[ MAX_PARM ][ MAX_PARAM_LENGTH ], int *num_param )
{
  	VDATA_LINK 	*tempVdata;
  	VGROUP_LINK 	*parmVgroup;
 	VGROUP_LINK 	*currVgroup;
	uint8           buffer[ MINIMAL_BUFFER_SIZE ] ;
  	int           	status ;
/*
 * ---- this is an internal toolkit function, minimal error checking 
 *	of inputs needed.
 */
        /* check to see that the granule has been openned with TKopen. */

        if ( sGranuleHandle->cfgSTR == NULL )
        {
          TKreportWarning( E_TK_GV_USE_TKOPEN ) ;
          return(          E_TK_GV_USE_TKOPEN ) ;
        }

        /* check that the granule is not an empty granule */

        if ( sGranuleHandle->cfgSTR->vGroup->subVgroup == NULL )
        {
          TKreportWarning( E_TK_GV_P_EMPTY ) ;
          return(          E_TK_GV_P_EMPTY ) ;
        }
/*
 * ---- Get the vgroup position of for next volume scan number to write.
 */
  	currVgroup = (VGROUP_LINK *) getCurrVgroup(
	  sGranuleHandle->cfgSTR->vGroup->subVgroup, VosNum + 1 );
/*
 * ---- initialize variables
 */
	* num_param	= 0 ;
      	parmVgroup	= currVgroup->subVgroup->subVgroup;
/*
 * ---- determine the position of the target parameter
 */
      	while ( parmVgroup != NULL ) 
      	{
          tempVdata = parmVgroup->vdata;
          vdataRead( & tempVdata , buffer ) ;

          strcpy( param_name[ ( * num_param ) ] , buffer  ) ;

	  parmVgroup = parmVgroup->next ;

	  ( * num_param ) ++ ;
      	}

        /* parameter metadata and data */
       	resetVgroup( currVgroup->subVgroup->subVgroup ,  0 ) ;

  	return( TK_SUCCESS ) ;
}

